<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\Account\Account;
use App\Models\Account\Voucher;
use App\Models\Apis\ApiCounter;
use App\Models\Bill\Bill;
use App\Models\Booking\BookGr;
use App\Models\Booking\PackagePosition;
use App\Models\Business\Company;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\GoodsMove\Challan;
use App\Models\GoodsMove\ChallanReceipt;
use App\Models\Master\Employee;
use App\Models\Master\Party;
use Barryvdh\Snappy\Facades\SnappyPdf;
use Carbon\Carbon;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\View;

class WhatsappController extends Controller
{
    public function sender(Request $request){

        $type = $request->type;
        $company_name = $request->company['company_name'];
        $mail_to = $request->mail_to;

        $company = Company::find($request->company['id']);
         
        
        if($type == 'GR' ){
            
            $book_gr = BookGr::with([
                'branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name,city,state,tele_number,gst_number,pan_number,pincode,address1', 'consignee:id,party_name,city,state,tele_number,gst_number,pan_number,pincode,address1', 'vehicle:id,vehicle_number,vehicle_type','driver:id,emp_name',
                'gr_desc',
                'gr_desc.material' => function ($query) {
                    $query->select('id', 'material_name');
                },
                'eway_details',
                'acc_party' => function ($query) {
                    $query->select('id', 'account_name','account_type');
                },
                'acc_party.party:id,account_id,party_name,city,state,tele_number,gst_number,pan_number,pincode,address1',
                'acc_party_branch:id,branch_name',
                
                'consignor_branch:id,branch_name',
                'consignee_branch:id,branch_name',
            ])->where('gr_number',$request->gr_number)->first();

            $gr_date = date('d-m-Y', strtotime($book_gr->gr_date));

            $subject = 'GR Number : '.$request->gr_number.'   Dated:'.$gr_date;

            $source = $book_gr->source['station_name'];
            $destination = $book_gr->destination['station_name'];
            $vehicle = $book_gr->vehicle?$book_gr->vehicle['vehicle_number']:'';
            
            $pdf = SnappyPdf::loadHTML(
                View::make('docs.gr_print', ['gr' => $book_gr,'company'=>$company])->render()
            );
            
            // Set paper size and orientation
            $pdf->setOption('orientation', 'portrait');
            $pdf->setOption('page-size', 'A4');

           
            // Get the PDF content
            $pdfContent = $pdf->output();

             // Replace the forward slash in gr_number for the filename
            $safe_gr_number = str_replace('/', '_', $request->gr_number);
            $filename = $safe_gr_number . '.pdf';
            $attachmentPath = storage_path('app/public/gr_attachments/' . $filename);
            
           
    
            file_put_contents($attachmentPath, $pdfContent);
            
              $attachmentPath = url('storage/gr_attachments/' . $filename);
             
              //dd($attachmentPath);

            if($mail_to=='Consignor' || $mail_to=='Both'){
                $party_name = $book_gr->consignor['party_name'];

                $receiver_number = Party::select('mob_number')->where('party_name',$party_name)->value('mob_number');
     

                $body =  "Dear *$party_name*, \n\nWe hope this message finds you well. Thank you for selecting $company_name as your logistics partner. We are pleased to inform you that your Goods Receipt (GR) request has been successfully processed. \n\nPlease find attached the PDF document containing the details of your GR: \n\nGR Number: *$request->gr_number* \nDate of GR: $gr_date \nPickup Location: $source \nDelivery Location: $destination\nVehicle: $vehicle \n\nBest regards,\n$company_name";
                if($receiver_number){
                $response =  $this->sendWhatsapp($type,$receiver_number, $subject, $body, $request->company_name, true, $attachmentPath, $filename);
                }
            }

            if($mail_to=='Consignee' || $mail_to=='Both'){
                $party_name = $book_gr->consignee['party_name'];

                $receiver_number = Party::select('mob_number')->where('party_name',$party_name)->value('mob_number');

                      

                $body =  "Dear *$party_name*, \n\nWe hope this message finds you well. Thank you for selecting $company_name as your logistics partner. We are pleased to inform you that your Goods Receipt (GR) request has been successfully processed. \n\nPlease find attached the PDF document containing the details of your GR: \n\nGR Number: *$request->gr_number* \nDate of GR: $gr_date \nPickup Location: $source \nDelivery Location: $destination\nVehicle: $vehicle \n\nBest regards,\n$company_name";
                if($receiver_number){
                $response =  $this->sendWhatsapp($type,$receiver_number, $subject, $body, $request->company_name, true, $attachmentPath, $filename);
                }
            }

            if($mail_to=='Driver'){
                $party_name = $book_gr->driver_name;

                $receiver_number = $book_gr->driver_mob_number;
     

                $body =  "Dear *$party_name*, \n\nPlease find attached the PDF document containing the details of your GR: \n\nGR Number: *$request->gr_number* \nDate of GR: $gr_date \nPickup Location: $source \nDelivery Location: $destination\nVehicle: $vehicle \n\nBest regards,\n$company_name";
                if($receiver_number){
                $response =  $this->sendWhatsapp($type,$receiver_number, $subject, $body, $request->company_name, true, $attachmentPath, $filename);
                }
            }


        }

        if($type == 'Voucher' ){
            
            $voucher = Voucher::with(['branch:id,branch_name','voucher_acc_details:id,voucher_id,account_id,nature,against,dr_amount,cr_amount', 'voucher_descs:id,voucher_id,v_acc_det_id,against_id,gr_no,against,against_number,date,due_amt,receipt_amt,received_amt,tds_percent,tds,deduction,balance_amt,from_opening','voucher_acc_details.account:id,account_name,account_type'])->where('id',$request->voucher_id)->first();

            $voucher_date = date('d-m-Y', strtotime($voucher->voucher_date));

            $subject = 'Voucher Number : '.$request->voucher_number.'   Dated:'.$voucher_date;

            $dr_amount = $voucher->total_dr_amount;
            $cr_amount = $voucher->total_cr_amount;

            $total = $dr_amount;
            
            // $pdf = SnappyPdf::loadHTML(
            //     View::make('docs.gr_print', ['gr' => $voucher,'company'=>$company])->render()
            // );
            
            // // Set paper size and orientation
            // $pdf->setOption('orientation', 'portrait');
            // $pdf->setOption('page-size', 'A4');

           
            // Get the PDF content
            // $pdfContent = $pdf->output();

            //  // Replace the forward slash in gr_number for the filename
            // $safe_gr_number = str_replace('/', '_', $request->gr_number);
            // $filename = $safe_gr_number . '.pdf';
            // $attachmentPath = storage_path('app/public/gr_attachments/' . $filename);
            
           
    
            // file_put_contents($attachmentPath, $pdfContent);
            
            //   $attachmentPath = url('storage/gr_attachments/' . $filename);
             
              //dd($attachmentPath);

            if($mail_to){
                $account = Account::with(['party:id,party_name,mob_number,account_id','employee:id,emp_name,contact_number,account_id'])->where('account_name',$mail_to)->first();

                $party_name =  $account->account_name;
                if($account->party)
                $receiver_number = Party::select('mob_number')->where('party_name',$account->account_name)->value('mob_number');
                if($account->employee)
                $receiver_number = Employee::select('contact_number')->where('emp_name',$account->account_name)->value('contact_number');
                
    

                $body =  "Dear *$party_name*, \n\nWe hope this message finds you well.\n\n A $voucher->voucher_name has been issue with your name :- \n\nVoucher Number: \n*$voucher->voucher_number* \nDate of Voucher: $voucher_date \nT. Amount: $total \n\nBest regards,\n$company_name";
                if($receiver_number){
                $response =  $this->sendWhatsapp($type,$receiver_number, $subject, $body, $request->company_name, false, '','');
                }
            }


        }

        if($type == 'Diesel Receipt' ){
            
            $dr = DieselReceipt::with([
                            'branch:id,branch_name,branch_code',
                            'vehicle:id,vehicle_number',
                            'dr_on_account:id,account_name',
                            'driver_on_account:id,account_name',
                            'filling_station:id,account_name'
                        ])->where('dr_number',$request->dr_number)->first();

            $dr_date = date('d-m-Y', strtotime($dr->booking_date));

            $subject = 'Diesel R. : '.$request->dr_number.'   Dated:'.$dr_date;

            $cash_amount = $dr->cash_amount;
            $fuel_amount = $dr->amount;
            
            $fuel_qty = $dr->qty;

            $copy = json_decode(json_encode($dr));
            $filling_station = $copy->filling_station->account_name;

            
            // $pdf = SnappyPdf::loadHTML(
            //     View::make('docs.gr_print', ['gr' => $dr,'company'=>$company])->render()
            // );
            
            // // Set paper size and orientation
            // $pdf->setOption('orientation', 'portrait');
            // $pdf->setOption('page-size', 'A4');

           
            // Get the PDF content
            // $pdfContent = $pdf->output();

            //  // Replace the forward slash in gr_number for the filename
            // $safe_gr_number = str_replace('/', '_', $request->gr_number);
            // $filename = $safe_gr_number . '.pdf';
            // $attachmentPath = storage_path('app/public/gr_attachments/' . $filename);
            
           
    
            // file_put_contents($attachmentPath, $pdfContent);
            
            //   $attachmentPath = url('storage/gr_attachments/' . $filename);
             
              //dd($attachmentPath);

            if($mail_to){
                $account = Account::with(['employee:id,emp_name,contact_number,account_id'])->where('account_name',$mail_to)->first();
                $party_name = $account->account_name;
                if($account->employee)
                $receiver_number = Employee::select('contact_number')->where('emp_name',$account->account_name)->value('contact_number');
                
       

                $body =  "Dear *$party_name*, \n\nWe hope this message finds you well.\n\n A Diesel Receipt has been issue with your name :- \n\nDiesel Receipt: \n*$dr->dr_number* \nDated: $dr_date\nFilling Stn.: $filling_station\nFuel. Amount: $fuel_amount \nFuel Qty: $fuel_qty \nCash. Amount: $cash_amount \n\nBest regards,\n$company_name";
                if($receiver_number){
                $response =  $this->sendWhatsapp($type,$receiver_number, $subject, $body, $request->company_name, false, '','');
                }
            }


        }

        if($type == 'Challan' ){
            
            $challan = Challan::with([
                'branch:id,branch_name,branch_code',
                'source:id,station_name',
                'destination:id,station_name',
                'vehicle:id,vehicle_number',
                'driver:id,emp_name',
                'receiver_branch:id,branch_name,branch_code',
                'challan_desc',
                'challan_desc.gr_desc:id,gr_id,gr_number,pkg,free_pkgs,rate,material_id,material_weight',
                'challan_desc.gr_desc.material:id,material_name',
                'challan_desc.gr_desc.bookGr:id,branch_id,gr_number,gr_date,consignor_name,consignee_name,consignor_phone_number,consignee_phone_number,destination_name,consignor_id,acc_party_id,consignee_id,destination_id,pay_mode,grand_total',
                'challan_desc.gr_desc.bookGr.consignor:id,party_name',
                'challan_desc.gr_desc.bookGr.acc_party:id,account_name',
                'challan_desc.gr_desc.bookGr.consignee:id,party_name',
                'challan_desc.gr_desc.bookGr.destination:id,station_name',
                'challan_desc.gr_desc.bookGr.eway_details',
                'broker:id,party_name',
                'lorry_hire_from:id,account_name',
                'dl_comm_branch:id,account_name',
                'commission_account:id,account_name',
                'dr_on_account:id,account_name',
                'paid_using:id,account_name',
            ])->where('challan_number',$request->challan_number)->first();

            $challan_branch = $challan->branch_id;

        $challan->challan_desc->transform(function ($desc) use ($challan_branch) {
            $free = PackagePosition::where('gr_desc_id', $desc->gr_desc_id)->where('position', $challan_branch)->sum('pkg');
            return [
                'branch_id' => $desc->gr_desc->bookGr->branch_id, 
                'challan_desc_id' => $desc->id,
                'gr_id' => $desc->gr_desc->gr_id,
                'gr_number' => $desc->gr_desc->gr_number,
                'gr_date' => $desc->gr_desc->bookGr->gr_date,
                'gr_desc' => $desc->gr_desc,
                'acc_party' => $desc->gr_desc->bookGr->acc_party,
                'challan_id' => $desc->challan_id,
                'challan_number' => $desc->challan_number,
                'consignor_phone_number' => $desc->gr_desc->bookGr->consignor_phone_number ,
                'consignee_phone_number' => $desc->gr_desc->bookGr->consignee_phone_number ,
                'gr_desc_id' => $desc->gr_desc_id,
                'pkg' => $desc->gr_desc->pkg,
                'wt' => $desc->gr_desc->material_weight,
                'rate' => $desc->gr_desc->rate,
                'free_pkgs' => $free,
                'loaded_pkgs' => $desc->loaded_pkgs,
                'material' => $desc->gr_desc->material['material_name'],
                'consignor' => $desc->gr_desc->bookGr->consignor_id ? $desc->gr_desc->bookGr->consignor['party_name'] : $desc->gr_desc->bookGr->consignor_name,
                'consignee' => $desc->gr_desc->bookGr->consignee_id ? $desc->gr_desc->bookGr->consignee['party_name'] : $desc->gr_desc->bookGr->consignee_name,
                'destination' => $desc->gr_desc->bookGr->destination_id ? $desc->gr_desc->bookGr->destination['station_name'] : $desc->gr_desc->bookGr->destination_name,
                'grand_total' => $desc->gr_desc->bookGr->grand_total,
                'pay_mode' => $desc->gr_desc->bookGr->pay_mode,
                'eway_details' => $desc->gr_desc->bookGr->eway_details 
            ];
        });




            $challan_date = date('d-m-Y', strtotime($challan->challan_date));

            $subject = 'Challan No.: '.$request->challan_number.'   Dated: '.$challan_date;

            if ($request->mail_to) {
            
                // Split mail_to with ',' and loop all records to send message
                $mailToArray = explode(',', $request->mail_to); // Splitting the mail_to string
            
                // Challan data preparation
                $source = $challan->source->station_name;
                $destination = $challan->destination->station_name;
                $total_pkg = $challan->lh_total_pkg;

                // Generate PDF
                $pdf = SnappyPdf::loadHTML(
                    View::make('docs.challan_print', ['challan' => $challan, 'company' => $company])->render()
                );
                $pdf->setOption('orientation', 'portrait');
                $pdf->setOption('page-size', 'A4');
            
                // Get the PDF content
                $pdfContent = $pdf->output();
            
                // Replace the forward slash in gr_number for the filename
                $safe_gr_number = str_replace('/', '_', $request->challan_number);
                $filename = $safe_gr_number . '.pdf';
                $attachmentPath = storage_path('app/public/challan_attachments/' . $filename);
                
            
                file_put_contents($attachmentPath, $pdfContent);
                $attachmentPath = url('storage/challan_attachments/' . $filename);
                
              
                // Loop through each mail_to record
                foreach ($mailToArray as $mail_to) {
                    $mail_to = trim($mail_to); // Trim any whitespace

                   
                    $receiver_number = $mail_to;
            
                    // Prepare the message body
                    $body = "A challan has been generated\n\nChallan No : *" . $request->challan_number . "*\n\nDated : *$challan_date*\n\nFrom : *" . $source . "* \n\nTo : *" . $destination . "* \n\n" . "Pkg *" . $total_pkg . "* \n\n has been securely despatched. \n\nBest regards,\n$company_name";
                    if($receiver_number){
                    // Send message via WhatsApp
                    $response = $this->sendWhatsapp('CHALLAN WITH PRINT', $receiver_number, $subject, $body, $request->company_name, true, $attachmentPath, $filename);
                    }
                }
            }
            


            foreach ($challan->challan_desc as $challan_desc) {
    
                $gr_no = $challan_desc['gr_desc']->gr_number;

                $gr_date = date('d-m-Y', strtotime($challan_desc['gr_desc']->bookGr->gr_date));
                $consignor = $challan_desc['gr_desc']->bookGr->consignor;
                $consignee = $challan_desc['gr_desc']->bookGr->consignee;
                $destination = $challan_desc['gr_desc']->bookGr->destination_name;
                $source = $challan_desc['gr_desc']->bookGr->source_name;
                $total_pkg = $challan_desc['loaded_pkgs'];
                $challan_no = $challan_desc['challan_number'];

                $mail_to = $challan_desc['gr_desc']->bookGr->consignor_name;
                if($mail_to){
                    $receiver_number = Party::select('mob_number')->where('party_name',$mail_to)->value('mob_number');

    
                    $body = "Dear *M/S " . $mail_to . "*, Your consignment vide \n\nCN No : *" . $gr_no ."*\n\nChallan No : *" . $challan_no . "*\n\nDated : *$gr_date*\n\nFrom : *" . $source . "* \n\nTo : *" . $destination . "* \n\n". "Pkg *" . $total_pkg . "* \n\n has been securely despatched. \n\nBest regards,\n$company_name";
                    if($receiver_number){
                    $response =  $this->sendWhatsapp($type,$receiver_number, $subject, $body, $request->company_name, false, '','');
                    }
                }

            }

            if($challan->receiver_branch){
                $copy = json_decode(json_encode($challan));

                $mail_to = $copy->receiver_branch->branch_name;
                $receiver_number = $copy->receiver_branch->mob_number;
                $source = $copy->source->station_name;
                $destination = $copy->destination->station_name;
                $total_pkg = $copy->lh_total_pkg;

                


                $challan_date = date('d-m-Y', strtotime($challan->challan_date));

    
                    $body = "A challan has been generated your way \n\nChallan No : *" . $request->challan_number ."*\n\nDated : *$challan_date*\n\nFrom : *" . $source . "* \n\nTo : *" . $destination . "* \n\n". "Pkg *" . $total_pkg . "* \n\n has been securely despatched. \n\nBest regards,\n$company_name";
                    if($receiver_number){
                    $response =  $this->sendWhatsapp($type,$receiver_number, $subject, $body, $request->company_name, false, '','');
                    }
            }

        

            


        }

        if($type == 'Challan Receipt' ){
            
            $challan_receipt = ChallanReceipt::with([
                'branch:id,branch_name,branch_code',
                'arrival_from:id,branch_name,branch_code',
                'vehicle:id,vehicle_number',
                'driver:id,emp_name',
                'challan:id,challan_number',
                'challan_rec_desc',
                'challan_rec_desc.challan_desc:id,loaded_pkgs',
                'challan_rec_desc.gr_desc:id,gr_id,gr_number,pkg,free_pkgs,material_id,material_weight',
                'challan_rec_desc.gr_desc.material:id,material_name',
                'challan_rec_desc.gr_desc.bookGr:id,gr_number,gr_date,consignor_id,consignee_id,destination_id,consignor_name,consignee_name,source_name,destination_name,pay_mode,grand_total',
                'challan_rec_desc.gr_desc.bookGr.consignor:id,party_name',
                'challan_rec_desc.gr_desc.bookGr.consignee:id,party_name',
                'challan_rec_desc.gr_desc.bookGr.destination:id,station_name',
            ])->where('challan_rec_number',$request->challan_rec_number)->first();

            $challan_rec_date = date('d-m-Y', strtotime($challan_receipt->arrival_date));

            $subject = 'Challan Receipt No.: '.$request->challan_number.'   Dated: '.$challan_rec_date;

            $branch_name =$challan_receipt->branch->branch_name;

            foreach ($challan_receipt->challan_rec_desc as $challan_rec_desc) {
                $gr_no = $challan_rec_desc->gr_number;
                $challan_rec_number = $challan_receipt->challan_rec_number;
                
                $gr_date = date('d-m-Y', strtotime($challan_rec_desc->gr_desc->bookGr->gr_date));
                $consignor = $challan_rec_desc->gr_desc->bookGr->consignor_name;
                $consignee = $challan_rec_desc->gr_desc->bookGr->consigee_name;
                $destination = $challan_rec_desc->gr_desc->bookGr->destination_name;
                $source = $challan_rec_desc->gr_desc->bookGr->source_name;
                $total_pkg = $challan_rec_desc->unloaded_pkgs;
                $challan_no = $challan_rec_desc->challan_number;


                $mail_to = $challan_rec_desc->gr_desc->bookGr->consignor_name;
                if($mail_to){
                    $receiver_number = Party::select('mob_number')->where('party_name',$mail_to)->value('mob_number');

                    
    
                    $body = "Dear *M/S " . $mail_to . "*, Your consignment vide \n\nArrival No : *" . $challan_rec_number ."*\n\nArrival Date : *" . $challan_rec_date ."*\n\nCN No : *" . $gr_no ."*\n\nDated : *$gr_date*\n\nFrom : *" . $source . "* \n\nTo : *" . $destination . "* \n\n". "Pkg *" . $total_pkg . "* \n\n has reached our *$branch_name* station.  Please collect your consignment (if booked for Godown Delivery) as soon as possible to avoid demurrage charges. \n\nBest regards,\n$company_name";
                    if($receiver_number){
                    $response =  $this->sendWhatsapp($type,$receiver_number, $subject, $body, $request->company_name, false, '','');
                    }
                }

            }

        }

        if($type == 'Bill' ){
            
            $bill = Bill::with([
                'branch:id,branch_name,branch_code',
                'acc_party:id,account_name',
                'acc_party_branch:id,branch_name',
                'acc_party.party:id,account_id,party_name,address1,address2,state,tele_number,gst_number,pan_number',
                'income:id,account_name',
    
                'bill_desc',
                'bill_desc.bookGr:id,gr_number,po_no,shipment_number,gr_date,consignor_id,consignee_id,tpt_mode,vehicle_id,source_id,destination_id,consignor_name,consignee_name,total_chg_qty,source_name,destination_name',
                'bill_desc.bookGr.consignor:id,party_name',
                'bill_desc.bookGr.consignee:id,party_name',
                'bill_desc.bookGr.source:id,station_name',
                'bill_desc.bookGr.destination:id,station_name',
                'bill_desc.pod:id,pod_number',
                'bill_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',
                'bill_desc.bookGr.gr_desc',
                'bill_desc.loading_slip'
            ])->where('bill_number',$request->bill_number)->first();

            $bill->bill_desc->transform(function ($desc) {
                $data = [
                    'bill_desc_id' => $desc->id,
                    'bill_id' => $desc->bill_id,
                    'total_without_gst' => $desc->total_without_gst,
                    'igst_amount' => $desc->igst_amount,
                    'cgst_amount' => $desc->cgst_amount,
                    'sgst_amount' => $desc->sgst_amount,
                    'rounded_off' => $desc->rounded_off,
                    'grand_total' => $desc->grand_total,
                ];
    
                
            
                if ($desc->selection_type == 'GR') {
                    $data += [
                        'gr_id' => $desc->gr_id,
                        'gr_number' => $desc->gr_number,
                        'gr_date' => $desc->bookGr->gr_date,
                        'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name],
                        'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name],
                        'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name],
                        'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                        'total_pkg' => $desc->total_pkg,
                        'detention' => $desc->detention,
                        'total_material_weight' => $desc->total_material_weight,
                        'total_chg_qty' => $desc->bookGr->total_chg_qty,
                        'total_desc_value' => $desc->total_desc_value,
                        'total_invoice_value' => $desc->total_invoice_value,
                        'freight' => $desc->freight,
                        'po_no' => $desc->bookGr->po_no,
                        'shipment_number' => $desc->bookGr->shipment_number,
                        'rate'=> $desc->gr_desc ? json_decode($desc->gr_desc)[0]->rate : null,
                        'gr_chg' => $desc->gr_chg,
                        'tpt_mode' =>  $desc->bookGr->tpt_mode,
                        'bookGr' => $desc->bookGr,
                        'gr_desc' => json_decode($desc->gr_desc),
                        'pod_id' => $desc->pod ? $desc->pod_id : null,
                        'pod_number' =>  $desc->pod ? $desc->pod->pod_number:null,
                        'vehicle' => $desc->bookGr->vehicle_id ? $desc->bookGr->vehicle : ['id'=> null, 'vehicle_number'=>$desc->bookGr->vehicle_number],
                        'load_detention' => $desc->load_detention,
                        'unload_detention' => $desc->unload_detention,
                        'payment_dynamic_fields' => json_decode($desc->payment_dynamic_fields)??[],
                        'pod_payment_dynamic_fields' => json_decode($desc->pod_payment_dynamic_fields)??[],
                    ];
                }
            
                if ($desc->selection_type == 'Loading Slip') {
                    $data += [
                        'ls_id' => $desc->ls_id,
                        'ls_number' => $desc->ls_number,
                        'ls_date' => $desc->loading_slip->ls_date,
                        'branch' => $desc->loading_slip->branch,
                        'vehicle' => $desc->loading_slip->vehicle,
                        'source' => $desc->loading_slip->source,
                        'gr_number' => $desc->loading_slip->gr_number,
                        'material_name'=>$desc->loading_slip->material_name,
                        'material_weight'=>$desc->loading_slip->material_weight,
                        'rate'=>$desc->loading_slip->rate,
    
                        'destination' => $desc->loading_slip->destination,
                        'freight' => $desc->loading_slip->freight,
                        'munshiana' => $desc->loading_slip->munshiana,
                        'dalla' => $desc->loading_slip->dalla,
                        'advance_amt' => $desc->loading_slip->advance_amt,
                        'balance_received_in_amt' => $desc->loading_slip->balance_received_in_amt,
                        'diesel_amt' => $desc->loading_slip->diesel_amt,
                        'payment_and_collection' => $desc->loading_slip->payment_and_collection,
                        'labour' => $desc->loading_slip->labour,
                        'detention' => $desc->loading_slip->detention,
                        'unload_detention' => $desc->loading_slip->unload_detention,
                        'balance_receivable' => $desc->loading_slip->balance_receivable,
                    ];
                }
    
                if ($desc->selection_type == 'Supplementary') {
                    $data += [
                        'supplementary_desc' => json_decode($desc->supplementary_desc),
                    ];
                }
            
                return $data;
            });  


            $bill_date = date('d-m-Y', strtotime($bill->bill_date));

            $subject = 'Bill Number : '.$request->gr_number.'   Dated:'.$bill_date;

            if($bill->selection_type == 'GR'){
                $bill_print = 'docs.bill_print';
            }elseif($bill->selection_type == 'Loading Slip'){
                $bill_print = 'docs.ls_bill_print';
            } elseif($bill->selection_type == 'Supplementary'){
                $bill_print = 'docs.manual_bill_print';
            } 
            $pdf = SnappyPdf::loadHTML(
                View::make($bill_print, ['bill' => $bill,'company'=>$company])->render()
            );
            
            // Set paper size and orientation
            $pdf->setOption('orientation', 'portrait');
            $pdf->setOption('page-size', 'A4');

           
            // Get the PDF content
            $pdfContent = $pdf->output();

             // Replace the forward slash in gr_number for the filename
            $safe_gr_number = str_replace('/', '_', $request->bill_number);
            $filename = $safe_gr_number . '.pdf';
            $attachmentPath = storage_path('app/public/bill_attachments/' . $filename);
            
           
    
            file_put_contents($attachmentPath, $pdfContent);
            
              $attachmentPath = url('storage/bill_attachments/' . $filename);
             
              //dd($attachmentPath);


              $grand_total = $bill->grand_total_amount;
            $mail_to = $bill->acc_party->account_name;
            if($mail_to){
                $party_name = $mail_to;


                $receiver_number = Party::select('mob_number')->where('party_name',$party_name)->value('mob_number');
      

                $body =  "Dear *$party_name*, \n\nWe hope this message finds you well. Thank you for selecting $company_name as your logistics partner.  \n\nPlease find attached the PDF document containing the details of your Bill: \n\nBill Number: *$request->bill_number* \nDate of Bill: $bill_date \nAmount: $grand_total \n\nBest regards,\n$company_name";
                if($receiver_number){
                $response =  $this->sendWhatsapp($type,$receiver_number, $subject, $body, $request->company_name, true, $attachmentPath, $filename);
                }
            }



        }


       return response()->json(['status'=>$response->original, 'success'=>$response->original['success']] ,$response->original['success']=="Success"?200:500);

    }

    function sendWhatsapp($type, $toMail, $subject, $body, $company_name, $attachments = true, $attachmentPath, $filename)
    {
        //$apikey = "649BC85494254";
        $access_token = "65292433ed7b9";
        $instance_id = "66E1679578246";
        $BASE_URL = "https://mblaster.in/api/send?instance_id=$instance_id&access_token=$access_token&";

        $number = $this->mobileNumberFormater($toMail);
        $body = urlencode($body);

        try {

            // $send = $BASE_URL."number=$number&message=$body&media_url=$attachmentPath&type=media&filename=BTMS_WHATSAPP";
            if($type=='GR' || $type=='Bill' || $type=='CHALLAN WITH PRINT') 
            $send = $BASE_URL."number=$number&message=$body&type=media&media_url=$attachmentPath&filename=$filename"; 
            else
            $send = $BASE_URL."number=$number&message=$body&type=text"; 

            $client = new \GuzzleHttp\Client();
            $response = $client->post($send);      

            $result = json_decode($response->getBody());

            $this->increase_api_count();
            
            return response()->json(['success' => true, 'message' => 'Whatsapp sent successfully', 'data'=>$result]);
        } catch (Exception $e) {
            return response()->json(['success' => false, 'message' => 'Error sending whatsapp: ' . $e->getMessage()], 500);
        }
    }

    function mobileNumberFormater($mobile)
    {
        // Remove any non-digit characters from the input
        $mobile = preg_replace('/\D/', '', $mobile);

        // If the number already starts with "91", return it as-is
        if (substr($mobile, 0, 2) == "91") {
            return $mobile;
        }

        // Otherwise, add "91" to the beginning and return the result
        return ($mobile == '' || $mobile == '0' || $mobile == ',' || $mobile == ', ' || $mobile == ' ,0') ? "" : "91" . $mobile;
    }


    function increase_api_count(){
        $api_name = 'whatsapp_api';
    
        $today = Carbon::now()->format('Y-m-d');
            $api_count = ApiCounter::where('api_name',$api_name)->where('date',$today)->first();
        
            if($api_count){
                $api_count->api_count = $api_count->api_count+1;
                $api_count->save();
            }
            else{
                $api_count = new ApiCounter();
                $api_count->api_count = 1;
                $api_count->date = $today;
                $api_count->api_name = $api_name;
                $api_count->save();
            }
    
    }
}
