<?php

namespace App\Http\Controllers\GoodsMove;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Booking\BookGr;
use App\Models\Booking\GrDesc;
use App\Models\GoodsMove\Challan;
use App\Models\GoodsMove\ChallanDesc;
use App\Models\Booking\PackagePosition;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\Master\IssueBooks;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class ChallanController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Challan');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $challans = Challan::with([
            'branch:id,branch_name,branch_code',
            'source:id,station_name',
            'destination:id,station_name',
            'vehicle:id,vehicle_number',
            'driver:id,emp_name',
            'receiver_branch:id,branch_name,branch_code',
            'challan_desc',
            'challan_desc.gr_desc:id,gr_id,gr_number,pkg,free_pkgs,material_id,material_weight',
            'challan_desc.gr_desc.material:id,material_name',
            'challan_desc.gr_desc.bookGr:id,gr_number,consignor_id,consignor_name,consignee_name,destination_name,consignee_id,destination_id,pay_mode,grand_total',
            'challan_desc.gr_desc.bookGr.consignor:id,party_name',
            'challan_desc.gr_desc.bookGr.consignee:id,party_name',
            'challan_desc.gr_desc.bookGr.destination:id,station_name',
            'lorry_hire_from:id,account_name',
            'broker:id,party_name',
            'dl_comm_branch:id,account_name',
            'commission_account:id,account_name',
            'dr_on_account:id,account_name',
            'paid_using:id,account_name',
            'diesel_party1:id,account_name',
            'diesel_party2:id,account_name',
            'diesel_party3:id,account_name',
        ])->get();

            $challans->challan_desc->transform(function ($desc) {
                return [
                    'challan_desc_id' => $desc->id,
                    'gr_id' => $desc->gr_desc->gr_id,
                    'gr_number' => $desc->gr_desc->gr_number,
                    'challan_id' => $desc->challan_id,
                    'challan_number' => $desc->challan_number,
                    'gr_desc_id' => $desc->gr_desc_id,
                    'pkg' => $desc->gr_desc->pkg,
                    'wt' => $desc->gr_desc->material_weight,
                    'free_pkgs' => $desc->gr_desc->free_pkgs,
                    'loaded_pkgs' => $desc->loaded_pkgs,
                    'material' => $desc->gr_desc->material['material_name'],
                    'consignor' => $desc->gr_desc->bookGr->consignor_id ? $desc->gr_desc->bookGr->consignor['party_name'] : $desc->gr_desc->bookGr->consignor_name,
                    'consignee' => $desc->gr_desc->bookGr->consignee_id ? $desc->gr_desc->bookGr->consignee['party_name'] : $desc->gr_desc->bookGr->consignee_name,
                    'destination' => $desc->gr_desc->bookGr->destination_id ? $desc->gr_desc->bookGr->destination['station_name'] : $desc->gr_desc->bookGr->destination_name,
                    'grand_total' => $desc->gr_desc->bookGr->grand_total,
                    'pay_mode' => $desc->gr_desc->bookGr->pay_mode,
                ];
            });
        return response()->json($challans);
    }

    public function show(Request $request, $pk){
        // FETCH AND SEND
        $challan = Challan::with([
            'branch:id,branch_name,branch_code',
            'source:id,station_name',
            'destination:id,station_name',
            'vehicle:id,vehicle_number,self_market',
            'driver:id,emp_name',
            'receiver_branch:id,branch_name,branch_code',
            'challan_desc',
            'challan_desc.gr_desc:id,gr_id,gr_number,pkg,free_pkgs,material_id,material_weight',
            'challan_desc.gr_desc.material:id,material_name',
            'challan_desc.gr_desc.bookGr:id,branch_id,gr_number,gr_date,consignor_name,consignee_name,destination_name,consignor_id,consignee_id,destination_id,pay_mode,grand_total',
            'challan_desc.gr_desc.bookGr.consignor:id,party_name',
            'challan_desc.gr_desc.bookGr.consignee:id,party_name',
            'challan_desc.gr_desc.bookGr.destination:id,station_name',
            'broker:id,party_name',
            'lorry_hire_from:id,account_name',
            'dl_comm_branch:id,account_name',
            'commission_account:id,account_name',
            'dr_on_account:id,account_name',
            'paid_using:id,account_name',
            'diesel_party1:id,account_name',
            'diesel_party2:id,account_name',
            'diesel_party3:id,account_name',
            'voucher_desc'
        ])->find($pk);

        $challan_branch = $challan->branch_id;

        $challan->challan_desc->transform(function ($desc) use ($challan_branch) {
            $free = PackagePosition::where('gr_desc_id', $desc->gr_desc_id)->where('position', $challan_branch)->sum('pkg');
            $diesel_amount = DieselReceipt::where('booking_id', $desc->gr_id)->where('against','GR')->sum('amount');
            return [
                'branch_id' => $desc->gr_desc->bookGr->branch_id, 
                'challan_desc_id' => $desc->id,
                'gr_id' => $desc->gr_desc->gr_id,
                'gr_number' => $desc->gr_desc->gr_number,
                'gr_date' => $desc->gr_desc->bookGr->gr_date,
                'challan_id' => $desc->challan_id,
                'challan_number' => $desc->challan_number,
                'diesel_amount' => $diesel_amount,
                'gr_desc_id' => $desc->gr_desc_id,
                'pkg' => $desc->gr_desc->pkg,
                'wt' => $desc->gr_desc->material_weight,
                'free_pkgs' => $free,
                'loaded_pkgs' => $desc->loaded_pkgs,
                'material' => $desc->gr_desc->material['material_name'],
                'consignor' => $desc->gr_desc->bookGr->consignor_id ? $desc->gr_desc->bookGr->consignor['party_name'] : $desc->gr_desc->bookGr->consignor_name,
                    'consignee' => $desc->gr_desc->bookGr->consignee_id ? $desc->gr_desc->bookGr->consignee['party_name'] : $desc->gr_desc->bookGr->consignee_name,
                    'destination' => $desc->gr_desc->bookGr->destination_id ? $desc->gr_desc->bookGr->destination['station_name'] : $desc->gr_desc->bookGr->destination_name,
                'grand_total' => $desc->gr_desc->bookGr->grand_total,
                'pay_mode' => $desc->gr_desc->bookGr->pay_mode,
            ];
        });

        return response()->json(['challan'=>$challan]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'Challan');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'challan_type' => 'required',
            'challan_number' => 'required|unique:challans,challan_number',
            'source' => 'required',
            'destination' => 'required'
        ], [
            'branch.required' => 'Required',
            'challan_type.required' => 'Required',
            'challan_number.required' => 'Required',
            'source.required' => 'Required',
            'destination.required' => 'Required',
            'challan_number.unique' => 'Already Exist !',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            

            // IN CASE THE CHALLAN TYPE IS 'PRINTED', WE NEED TO CHECK IF THE BOOK IS ACTUALLY ISSUED TO THE BRANCH OR NOT
            $max_number = $this->get_max_number_from_challan_number($request->challan_number);
            if($request->challan_type == 'PRINTED'){
                $check = IssueBooks::select('prefix','party_id')->where('prefix',$request->prefix)->where('branch_id', $request->branch['id'])->where('book_type','Challan')->where('issue_from', '<=', $max_number)->where('issue_to', '>=', $max_number)->first();
                if(!$check){
                    return response()->json(['message'=>'Book Not Issued To The Branch'], 422);
                }
                
            }
            
            
            // SAVING A NEW GR
            $challan = new Challan();
            $challan->branch_id = $request->branch['id'];
            $challan->challan_type = $request->challan_type;
            $challan->prefix = $request->prefix;
            $challan->max_number = $max_number;
            $challan->challan_number = $request->challan_number;
            $challan->challan_date = $request->challan_date;
            $challan->challan_time = $request->challan_time;
            $challan->movement_type = $request->movement_type;

            $challan->source_id = $request->source['id'];
            $challan->destination_id = $request->destination['id'];

            $vehicle_id = $this->add_if_not_in_master('Vehicle', $request->vehicle, $request->current_year);
            $challan->vehicle_id = $vehicle_id;
            $challan->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;

            $challan->vehicle_type = $request->vehicle_type;
            $challan->driver_id = $request->driver ? $request->driver['id'] : null;
            $challan->broker_id = $request->broker ? $request->broker['id'] : null;
            $challan->lorry_hire_from = $request->lorry_hire_from ? $request->lorry_hire_from['id'] : null;
            $challan->remarks = $request->remarks;
            $challan->receiver_branch = $request->receiver_branch ? $request->receiver_branch['id'] :null;
            $challan->dr_on_account = $request->dr_on_account ? $request->dr_on_account['id'] : null;

            $challan->total_freight = $request->total_freight;

            $challan->lh_total_pkg = $request->lh_total_pkg;
            $challan->lh_total_wt = $request->lh_total_wt;
            $challan->lh_unit = $request->lh_unit;
            $challan->lh_rate = $request->lh_rate;
            $challan->lh_freight = $request->lh_freight;
            $challan->lh_extra_point = $request->lh_extra_point;
            $challan->lh_net_freight = $request->lh_net_freight;
            $challan->lh_munshiana = $request->lh_munshiana;
            $challan->lh_advance = $request->lh_advance;
            $challan->lh_shortage = $request->lh_shortage;
            $challan->material_cost = $request->material_cost;
            $challan->lh_payment = $request->lh_payment;
            $challan->lh_tds_percent = $request->lh_tds_percent;
            $challan->lh_tds = $request->lh_tds;
            $challan->lh_balance = $request->lh_balance;
            $challan->paid_using = $request->paid_using ? $request->paid_using['id'] :null;

            $challan->diesel_amt1 = $request->diesel_amt1;
            $challan->diesel_date1 = $request->diesel_date1;
            $challan->diesel_party1_id = $request->diesel_party1 ? $request->diesel_party1['id'] :null;

            $challan->diesel_amt2 = $request->diesel_amt2;
            $challan->diesel_date2 = $request->diesel_date2;
            $challan->diesel_party2_id = $request->diesel_party2 ? $request->diesel_party2['id'] :null;

            $challan->diesel_amt3 = $request->diesel_amt3;
            $challan->diesel_date3 = $request->diesel_date3;
            $challan->diesel_party3_id = $request->diesel_party3 ? $request->diesel_party3['id'] :null;

            $challan->load_date = $request->load_date;
            $challan->loading_labour = $request->loading_labour;
            $challan->loading_detention = $request->loading_detention;
            $challan->unload_date = $request->unload_date;
            $challan->unloading_labour = $request->unloading_labour;
            $challan->unloading_detention = $request->unloading_detention;
            $challan->collection_chg = $request->collection_chg;
            $challan->other_add_label = $request->other_add_label;
            $challan->other_add = $request->other_add;
            $challan->other_minus_label = $request->other_minus_label;
            $challan->other_minus = $request->other_minus;

            $challan->dl_comm_branch = $request->dl_comm_branch ? $request->dl_comm_branch['id'] : null;
            $challan->commission_account = $request->commission_account ? $request->commission_account['id'] : null;
            $challan->dl_comm_unit = $request->dl_comm_unit;
            $challan->dl_comm_total_pkg = $request->dl_comm_total_pkg;
            $challan->dl_comm_total_wt = $request->dl_comm_total_wt;
            $challan->dl_comm_rate = $request->dl_comm_rate;
            $challan->dl_comm_to_pay = $request->dl_comm_to_pay;
            $challan->dl_comm_dynamic = $request->dl_comm_dynamic;
            $challan->dl_comm_branch_balance = $request->dl_comm_branch_balance;

            $challan->fin_year = $request->current_year;

            $challan->created_by = $request->user->id;

            // Save the model to the database
            $challan->save();


            // SAVING PACKAGE DETAILS
            $challan_desc = $request->challan_desc;
            foreach ($challan_desc as $cd) {
                if($cd['loaded_pkgs'] < 0)
                continue;
                
                $ch_dc = new ChallanDesc();
                $ch_dc->challan_id = $challan->id;
                $ch_dc->challan_number = $challan->challan_number;
                $ch_dc->gr_id = $cd['gr_id'];
                $ch_dc->gr_number = $cd['gr_number'];
                $ch_dc->gr_desc_id = $cd['gr_desc_id'];
                $ch_dc->loaded_pkgs = $cd['loaded_pkgs'];
                $ch_dc->fin_year = $request->current_year;
                $ch_dc->save();

                

                $gr_desc = GrDesc::find($cd['gr_desc_id']);
                $gr_desc->free_pkgs = $gr_desc->free_pkgs - $cd['loaded_pkgs'];
                $gr_desc->save();

                
                // SAVING PACKAGE POSITIONS
                $pkg_position = PackagePosition::where('gr_desc_id', $cd['gr_desc_id'])->where('position' ,$challan->branch_id)->first();

                
                if($pkg_position->pkg == $cd['loaded_pkgs']){
                    $pkg_position->position = 'IN TRANSIT';    
                    $pkg_position->challan_desc_id =  $ch_dc->id; 
                    $pkg_position->save();
                }
                else{
                    $pkg_position->pkg = $pkg_position->pkg - $cd['loaded_pkgs'];
                    $pkg_position->save();

                    $pkg_pos = new PackagePosition();
                    $pkg_pos->gr_desc_id = $cd['gr_desc_id'];
                    $pkg_pos->gr_id = $cd['gr_id'];
                    $pkg_pos->gr_number = $cd['gr_number'];
                    $pkg_pos->position = 'IN TRANSIT';
                    $pkg_pos->pkg = $cd['loaded_pkgs'];
                    $pkg_pos->challan_desc_id =  $ch_dc->id;
                    $pkg_pos->origin = $pkg_position->origin;
                    $pkg_pos->save();
                }
                // PACKAGE POSITIONS SAVED


            }
            // PACKAGE DETAILS SAVED
            

            


            // RECORD THE TRANSACTION IN ACCOUNTS
           $this->record_transaction_in_accounts($request,$challan);
            // TRANSACTION RECORDED IN ACCOUNTS


             
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($challan->id);
            $this->createEditLog('created', 'challans', $challan->id, $challan->challan_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
          

            
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$challan->id, 'challan_number'=>$challan->challan_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'Challan');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $challan = Challan::find($pk);

        $challan_rec_check = $this->check_in_challan_receipt_desc('challan_rec_number','challan_rec_number', 'challan_id', $pk);
        $voucher_check2  = $this->check_in_voucher_desc('voucher_number','voucher_number','CHALLAN',$challan->challan_number);
        $voucher_check['status'] = false;
        $lorry_hire_check = $this->check_in_lorry_hire_desc('lh_number','lh_number','Challan',$challan->challan_number);
        $diesel_receipt_check = $this->check_in_diesel_receipt('dr_number','dr_number','CHALLAN',$challan->challan_number);
        $driver_settlement_desc_check = $this->check_in_driver_stl_desc('ds_number','ds_number','CHALLAN',$challan->challan_number);
        $lorry_hire_bill_check = $this->check_in_lorry_hire_bill_desc('lhb_number','lhb_number','Lorry Hire Bill',$challan->challan_number);

        if($challan_rec_check['status'] || $voucher_check['status'] || $lorry_hire_check['status'] || $diesel_receipt_check['status'] || $driver_settlement_desc_check['status'] || $lorry_hire_bill_check['status']){
           $status= $challan_rec_check['status'] ? $challan_rec_check : ( $voucher_check['status'] ? $voucher_check : ( $lorry_hire_check['status'] ? $lorry_hire_check : ( $diesel_receipt_check['status'] ? $diesel_receipt_check :($driver_settlement_desc_check['status'] ? $driver_settlement_desc_check : $lorry_hire_bill_check ) ) ) );
           return response()->json(['error'=>'Document in use', 'data'=>$status],500);
        };


        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($challan->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'challan_type' => 'required',
            'challan_number' => 'required',
            'source' => 'required',
            'destination' => 'required'
        ], [
            'branch.required' => 'Required',
            'challan_type.required' => 'Required',
            'challan_number.required' => 'Required',
            'source.required' => 'Required',
            'destination.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // SAVING A NEW CHALLAN
            $challan->challan_date = $request->challan_date;
            $challan->challan_time = $request->challan_time;
            $challan->movement_type = $request->movement_type;

            $challan->source_id = $request->source['id'];
            $challan->destination_id = $request->destination['id'];
            $vehicle_id = $this->add_if_not_in_master('Vehicle', $request->vehicle, $request->current_year);
            $challan->vehicle_id = $vehicle_id;
            $challan->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
            $challan->vehicle_type = $request->vehicle_type;
            $challan->driver_id = $request->driver ? $request->driver['id'] : null;
            $challan->broker_id = $request->broker ? $request->broker['id'] : null;

            if($voucher_check2['status']==false){
                $challan->lorry_hire_from = $request->lorry_hire_from ? $request->lorry_hire_from['id'] : null;
            }
            
            $challan->remarks = $request->remarks;
            $challan->receiver_branch = $request->receiver_branch ? $request->receiver_branch['id'] :null;

            $challan->dr_on_account = $request->dr_on_account ? $request->dr_on_account['id'] : null;

            $challan->total_freight = $request->total_freight;

            $challan->lh_total_pkg = $request->lh_total_pkg;
            $challan->lh_total_wt = $request->lh_total_wt;
            $challan->lh_unit = $request->lh_unit;
            $challan->lh_rate = $request->lh_rate;
            $challan->lh_freight = $request->lh_freight;
            $challan->lh_extra_point = $request->lh_extra_point;
            $challan->lh_net_freight = $request->lh_net_freight;
            $challan->lh_munshiana = $request->lh_munshiana;
            $challan->lh_advance = $request->lh_advance;
            $challan->lh_shortage = $request->lh_shortage;
            $challan->material_cost = $request->material_cost;
            $challan->lh_payment = $request->lh_payment;
            $challan->lh_tds_percent = $request->lh_tds_percent;
            $challan->lh_tds = $request->lh_tds;

            if($challan->paid_status){
                if($challan->lh_balance < $request->lh_balance){
                    $challan->paid_status = 0;
                }
            }

            
            $challan->lh_balance = $request->lh_balance;
            $challan->paid_using = $request->paid_using ? $request->paid_using['id'] :null;

            $challan->material_cost = $request->material_cost;
            $challan->diesel_amt1 = $request->diesel_amt1;
            $challan->diesel_date1 = $request->diesel_date1;
            $challan->diesel_party1_id = $request->diesel_party1 ? $request->diesel_party1['id'] :null;

            $challan->diesel_amt2 = $request->diesel_amt2;
            $challan->diesel_date2 = $request->diesel_date2;
            $challan->diesel_party2_id = $request->diesel_party2 ? $request->diesel_party2['id'] :null;

            $challan->diesel_amt3 = $request->diesel_amt3;
            $challan->diesel_date3 = $request->diesel_date3;
            $challan->diesel_party3_id = $request->diesel_party3 ? $request->diesel_party3['id'] :null;

            $challan->load_date = $request->load_date;
            $challan->loading_labour = $request->loading_labour;
            $challan->loading_detention = $request->loading_detention;
            $challan->unload_date = $request->unload_date;
            $challan->unloading_labour = $request->unloading_labour;
            $challan->unloading_detention = $request->unloading_detention;
            $challan->collection_chg = $request->collection_chg;
            $challan->other_add_label = $request->other_add_label;
            $challan->other_add = $request->other_add;
            $challan->other_minus_label = $request->other_minus_label;
            $challan->other_minus = $request->other_minus;

            $challan->dl_comm_branch = $request->dl_comm_branch ? $request->dl_comm_branch['id'] : null;
            $challan->commission_account = $request->commission_account ? $request->commission_account['id'] : null;
            $challan->dl_comm_unit = $request->dl_comm_unit;
            $challan->dl_comm_total_pkg = $request->dl_comm_total_pkg;
            $challan->dl_comm_total_wt = $request->dl_comm_total_wt;
            $challan->dl_comm_rate = $request->dl_comm_rate;
            $challan->dl_comm_to_pay = $request->dl_comm_to_pay;
            $challan->dl_comm_dynamic = $request->dl_comm_dynamic;
            $challan->dl_comm_branch_balance = $request->dl_comm_branch_balance;

            $challan->updated_by = $request->user->id;

            $challan->save();
            // NEW CHALLAN SAVED
            
            // SAVING PACKAGE DETAILS
            $newly_added_challan_desc = $request->newly_added_challan_desc;
            foreach ($newly_added_challan_desc as $nacd) {
                if($nacd['loaded_pkgs'] < 0)
                continue;
                
                $ch_dc = new ChallanDesc();
                $ch_dc->challan_id = $challan->id;
                $ch_dc->challan_number = $challan->challan_number;
                $ch_dc->gr_id = $nacd['gr_id'];
                $ch_dc->gr_number = $nacd['gr_number'];
                $ch_dc->gr_desc_id = $nacd['gr_desc_id'];
                $ch_dc->loaded_pkgs = $nacd['loaded_pkgs'];
                $ch_dc->fin_year = $request->current_year;
                $ch_dc->save();

                $gr_desc = GrDesc::find($nacd['gr_desc_id']);
                $gr_desc->free_pkgs = $gr_desc->free_pkgs - $nacd['loaded_pkgs'];
                $gr_desc->save();

                // SAVING PACKAGE POSITIONS
                // SAVING PACKAGE POSITIONS
                $pkg_position = PackagePosition::where('gr_desc_id', $nacd['gr_desc_id'])->where('position' , $challan->branch_id)->first();
                
                if($pkg_position->pkg == $nacd['loaded_pkgs']){
                    $pkg_position->position = 'IN TRANSIT';    
                    $pkg_position->challan_desc_id =  $ch_dc->id; 
                    $pkg_position->save();
                }
                else{
                    $pkg_position->pkg = $pkg_position->pkg - $nacd['loaded_pkgs'];
                    $pkg_position->save();

                    $pkg_pos = new PackagePosition();
                    $pkg_pos->gr_desc_id = $nacd['gr_desc_id'];
                    $pkg_pos->gr_id = $nacd['gr_id'];
                    $pkg_pos->gr_number = $nacd['gr_number'];
                    $pkg_pos->position = 'IN TRANSIT';
                    $pkg_pos->pkg = $nacd['loaded_pkgs'];
                    $pkg_pos->challan_desc_id =  $ch_dc->id;
                    $pkg_pos->origin = $pkg_position->origin;
                    $pkg_pos->save();
                }
                // PACKAGE POSITIONS SAVED
                // PACKAGE POSITIONS SAVED
            }
            // PACKAGE DETAILS SAVED

            // SAVING UPDATES ON EXISTING  PACKAGE DETAILS
            // $updated_challan_desc = $request->updated_challan_desc;
            // if( !empty($updated_challan_desc) ){
            //     foreach ($updated_challan_desc as $ucd) {
            //         if(!$ucd['loaded_pkgs'] > 0)
            //         continue;
                       
            //             $ch_dc = ChallanDesc::find( $ucd['challan_desc_id'] );
            //             $diff =  $ch_dc->loaded_pkgs - $ucd['loaded_pkgs']; 
            //             $ch_dc->loaded_pkgs = $ucd['loaded_pkgs'];
            //             $ch_dc->save();
                        
            //             $gr_desc = GrDesc::find($ucd['gr_desc_id']);
            //             $gr_desc->free_pkgs = $gr_desc->free_pkgs + $diff;
            //             $gr_desc->save();

            //             // UPDATING PACKAGE POSITIONS


            //             $pkg_position = PackagePosition::where('gr_desc_id', $ucd['gr_desc_id'])->where('position',$ucd['branch_id'])->first();
            //             if($pkg_position){
            //                 $pkg_position->pkg = $pkg_position->pkg + $diff;
            //                 $pkg_position->save();
            //             }
                        

            //             $pkg_position = PackagePosition::where('gr_desc_id', $ucd['gr_desc_id'])->where('position','IN TRANSIT')->first();
            //             $pkg_position->pkg = $ucd['loaded_pkgs'];
            //             $pkg_position->save();
            //             // UPDATING POSITIONS SAVED
            //     }
            // }
            // UPDATES SAVED ON EXISTING PACKAGE DETAILS

            // DELETING EXISTING PACKAGE DETAILS
            $deleted_challan_desc = $request->deleted_challan_desc;
            if( !empty($deleted_challan_desc) ){
                foreach ($deleted_challan_desc as $dcd) {
                        // Deleted POSITIONS SAVED

                        $ch_dc = ChallanDesc::find( $dcd );
                        $ch_dc->delete();

                        $gr_desc = GrDesc::find( $ch_dc->gr_desc_id );
                        $gr_desc->free_pkgs = $gr_desc->free_pkgs + $ch_dc->loaded_pkgs;
                        $gr_desc->save();


                        // Deleted PACKAGE POSITIONS
                        $pkg_position = PackagePosition::where('gr_desc_id', $ch_dc->gr_desc_id )->where('position' ,$challan->branch_id)->first();
                        
                        if($pkg_position){
                            $pkg_position->pkg = $pkg_position->pkg + $ch_dc->loaded_pkgs;
                            $pkg_position->save();

                            $pkg_positio = PackagePosition::where('gr_desc_id', $ch_dc->gr_desc_id )->where('challan_desc_id',$ch_dc->id)->where('position' ,'IN TRANSIT')->first();
                            $pkg_positio->delete();
                        }
                        else{
                            $pkg_positio = PackagePosition::where('gr_desc_id', $ch_dc->gr_desc_id )->where('challan_desc_id',$ch_dc->id)->where('position' ,'IN TRANSIT')->first();
                            $pkg_positio->position = $challan->branch_id;
                            $pkg_positio->challan_desc_id = null;
                            $pkg_positio->save();
                        }
                }
                    
            }
            //DELETED EXISTING PACKAGE DETAILS


            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request,$challan);
            // TRANSACTION RECORDED IN ACCOUNTS
            


            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($challan->id);
            $this->createEditLog('updated', 'challans', $challan->id, $challan->challan_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'Challan');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $challan = Challan::find($pk);

        $challan_rec_check = $this->check_in_challan_receipt_desc('challan_rec_number','challan_rec_number', 'challan_id', $pk);
        $voucher_check  = $this->check_in_voucher_desc('voucher_number','voucher_number','CHALLAN',$challan->challan_number);
        $lorry_hire_check = $this->check_in_lorry_hire_desc('lh_number','lh_number','Challan',$challan->challan_number);
        $diesel_receipt_check = $this->check_in_diesel_receipt('dr_number','dr_number','CHALLAN',$challan->challan_number);
        $driver_settlement_desc_check = $this->check_in_driver_stl_desc('ds_number','ds_number','CHALLAN',$challan->challan_number);
        $lorry_hire_bill_check = $this->check_in_lorry_hire_bill_desc('lhb_number','lhb_number','Lorry Hire Bill',$challan->challan_number);

        if($challan_rec_check['status'] || $voucher_check['status'] || $lorry_hire_check['status'] || $diesel_receipt_check['status'] || $driver_settlement_desc_check['status'] || $lorry_hire_bill_check['status']){
           $status= $challan_rec_check['status'] ? $challan_rec_check : ( $voucher_check['status'] ? $voucher_check : ( $lorry_hire_check['status'] ? $lorry_hire_check : ( $diesel_receipt_check['status'] ? $diesel_receipt_check : ($driver_settlement_desc_check['status'] ? $driver_settlement_desc_check : $lorry_hire_bill_check )  ) ) );
           return response()->json(['error'=>'Document in use', 'data'=>$status],500);
        };

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($challan->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
        $challan_desc_ids = ChallanDesc::select('id')->where('challan_id', $pk)->pluck('id')->toArray();
        // DELETING CHALLAN DESCRIPTIONS
        foreach ($challan_desc_ids as $cdi) {
            $challan_desc = ChallanDesc::find($cdi);

            $gr_desc = GrDesc::find($challan_desc->gr_desc_id);
            $gr_desc->free_pkgs = $gr_desc->free_pkgs + $challan_desc->loaded_pkgs;
            $gr_desc->save();
            


            // Deleted PACKAGE POSITIONS
            $pkg_position = PackagePosition::where('gr_desc_id', $challan_desc->gr_desc_id )->where('position' ,$challan->branch_id)->first();
                        
            if($pkg_position){
                $pkg_position->pkg = $pkg_position->pkg + $challan_desc->loaded_pkgs;
                $pkg_position->save();

                $pkg_positio = PackagePosition::where('gr_desc_id', $challan_desc->gr_desc_id )->where('challan_desc_id',$challan_desc->id)->where('position' ,'IN TRANSIT')->first();
                $pkg_positio->delete();
            }
            else{
                $pkg_positio = PackagePosition::where('gr_desc_id', $challan_desc->gr_desc_id )->where('challan_desc_id',$challan_desc->id)->where('position' ,'IN TRANSIT')->first();
                $pkg_positio->position = $challan->branch_id;
                $pkg_positio->challan_desc_id = null;
                $pkg_positio->save();
            }
           
            $challan_desc->delete();
        }



        //FIND AND DELETE THE BRANCH
        $challan->delete();
        // COMMISSIONS WILL BE CASCADED AUTOMATICALLY

        //DELETE THE TRANSACTION VOUCHER
        $t_data = [];
        $t_data['t_against'] = 'CHALLAN';
        $t_data['t_against_id'] = $challan->id;
        $t_data['t_against_no'] = $challan->challan_number;
        $t_data['fin_year'] = $request->current_year;


        $this->delete_transaction_voucher($t_data);


        $this->createEditLog('deleted', 'challans', $challan->id, $challan->challan_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }

    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = Challan::select('id', 'challan_number','vehicle_number')->where('fin_year',$request->current_year);;

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }

        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('challan_number', 'like', '%'. $searchVariable . '%')
            ->orWhere('vehicle_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $challans = $query->get();
        return response(['challans' => $challans, 'draw' => $request->draw]);
    }

    public function challan_for_challan_receipt(Request $request){
        // Initialize the query builder
        $query = Challan::select('id', 'challan_number')->whereDoesntHave('challan_rec_desc');

        $query->where('movement_type',$request->movement_type)->where('branch_id',$request->arrival_from)->where('receiver_branch',$request->receiver_branch);


        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('challan_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $challans = $query->get();
        return response(['challans' => $challans, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Challan');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Challan::select('id', 'challan_number', 'challan_date','challan_time' , 'branch_id','source_id', 'destination_id')
                ->with(['branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name']) 
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('challan_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Challan::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $challansQuery = Challan::select('id', 'challan_number')->where('fin_year',$request->current_year);;
        
        if (!empty($request->user_branch_ids)) {
            $challansQuery->whereIn('branch_id', $request->user_branch_ids);
        }
        
        $challans = $challansQuery->orderBy('id', 'desc')->take(10)->get();
        
        return response()->json(['challans' => $challans]);
    }
    


    public function get_next_challan_number(Request $request){
        $type=$request->type;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        if($type=='SYSTEM'){
            $max = Challan::where('branch_id',$branch_id)->where('challan_type', $type)->max('max_number');
            $challan_number = $branch_code.'-'.sprintf('%06d', $max+1);
            return response()->json(['challan_number' => strtoupper($challan_number), 'max_number' => $max+1]);
        }
        else if($type=='PRINTED'){
            // GET THE NEXT NUMBER THAT HAS BEEN ISSUE TO THE BRANCH WITH THE CERTAIN PREFIX
            $max = Challan::where('prefix',$prefix)->where('challan_type', $type)->where('branch_id', $branch_id)->max('max_number');
            
            // IF MAX IS PRESENT, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            if( $max ){
                $check = IssueBooks::select('prefix','party_id')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','Challan')->where('issue_from', '<=', $max+1)->where('issue_to', '>=', $max+1)->first();
                if($max && $check){
                    $challan_number = $prefix ? $prefix.'-'.($max+1) : ($max+1);
                    return response()->json(['challan_number' => strtoupper($challan_number), 'max_number' => $max+1, 'party' => $check->party]);
                }
            }

            // IF MAX IS NOT PRESENT OR NOT VALID, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            $check = IssueBooks::select('prefix','party_id','issue_from')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','Challan')->where('issue_from', '>=', $max+1)->first();
            if( $check ){
                $challan_number = $prefix ? $prefix.'-'.$check->issue_from : $check->issue_from;
                return response()->json(['challan_number' => strtoupper($challan_number), 'max_number' => $check->issue_from, 'party' => $check->party]);
            }
            else{
                return response()->json(['errors' => ['challan_number' => ['No Valid Challan Number']]],422);
            }
        }
    }

    protected function get_data_for_obj($pk){
        // FETCH AND SEND
        $challan =Challan::with([
            'branch:id,branch_name,branch_code',
            'source:id,station_name',
            'destination:id,station_name',
            'vehicle:id,vehicle_number',
            'driver:id,emp_name',
            'challan_desc',
            'receiver_branch:id,branch_name,branch_code',
            'challan_desc.gr_desc:id,gr_id,gr_number,pkg,free_pkgs,material_id,material_weight',
            'challan_desc.gr_desc.material:id,material_name',
            'challan_desc.gr_desc.bookGr:id,gr_number,consignor_id,consignee_id,destination_id,destination_name,consignor_name,consignee_name,pay_mode,grand_total',
            'challan_desc.gr_desc.bookGr.consignor:id,party_name',
            'challan_desc.gr_desc.bookGr.consignee:id,party_name',
            'challan_desc.gr_desc.bookGr.destination:id,station_name',
            'broker:id,party_name',
            'lorry_hire_from:id,account_name',
            'dl_comm_branch:id,account_name',
            'commission_account:id,account_name',
            'dr_on_account:id,account_name',
            'paid_using:id,account_name',
            'diesel_party1:id,account_name',
            'diesel_party2:id,account_name',
            'diesel_party3:id,account_name',
        ])->find($pk);

        
            $challan->challan_desc->transform(function ($desc) {
                return [
                    'challan_desc_id' => $desc->id,
                    'gr_id' => $desc->gr_desc->gr_id,
                    'gr_number' => $desc->gr_desc->gr_number,
                    'challan_id' => $desc->challan_id,
                    'challan_number' => $desc->challan_number,
                    'gr_desc_id' => $desc->gr_desc_id,
                    'pkg' => $desc->gr_desc->pkg,
                    'wt' => $desc->gr_desc->material_weight,
                    'free_pkgs' => $desc->gr_desc->free_pkgs,
                    'loaded_pkgs' => $desc->loaded_pkgs,
                    'material' => $desc->gr_desc->material['material_name'],
                    'consignor' => $desc->gr_desc->bookGr->consignor_id ? $desc->gr_desc->bookGr->consignor['party_name'] : $desc->gr_desc->bookGr->consignor_name,
                    'consignee' => $desc->gr_desc->bookGr->consignee_id ? $desc->gr_desc->bookGr->consignee['party_name'] : $desc->gr_desc->bookGr->consignee_name,
                    'destination' => $desc->gr_desc->bookGr->destination_id ? $desc->gr_desc->bookGr->destination['station_name'] : $desc->gr_desc->bookGr->destination_name ,
                    'grand_total' => $desc->gr_desc->bookGr->grand_total,
                    'pay_mode' => $desc->gr_desc->bookGr->pay_mode,
                ];
            });

        return ['challan'=>$challan];
    }

    public function fetch_challan_desc_for_challan_receipt(Request $request){
        $challan_descs = ChallanDesc::with(
                        'gr_desc:id,gr_id,gr_number,pkg,free_pkgs,material_id,material_weight',
                        'gr_desc.material:id,material_name',
                        'gr_desc.bookGr:id,gr_number,consignor_id,consignee_id,destination_id,consignor_name,consignee_name,destination_name,pay_mode,grand_total',
                        'gr_desc.bookGr.consignor:id,party_name',
                        'gr_desc.bookGr.consignee:id,party_name',
                        'gr_desc.bookGr.destination:id,station_name'
                        )
                        ->where('challan_id', $request->challan_id)->get();

        $challan_descs->transform(function ($desc) {

            $in_transit_pkgs = PackagePosition::select('id','pkg')->where('gr_desc_id',$desc->gr_desc_id)->where('position','IN TRANSIT')->where('challan_desc_id',$desc->id)->where('pkg','!=',0)->first(); 
            
            if($in_transit_pkgs){
                return [
                    'pkg_pos_id' => $in_transit_pkgs->id,
                    'challan_rec_desc_id' => null,
                    'challan_rec_id' => null,
                    'challan_rec_number' => null,
                    'challan_desc_id' => $desc->id,
                    'gr_id' => $desc->gr_desc->bookGr->id,
                    'gr_number' => $desc->gr_desc->bookGr->gr_number,
                    'challan_id' => $desc->challan_id,
                    'challan_number' => $desc->challan_number,
                    'gr_desc_id' => $desc->gr_desc_id,
                    'pkg' => $desc->gr_desc->pkg,
                    'wt' => $desc->gr_desc->material_weight,
                    'free_pkgs' => $desc->gr_desc->free_pkgs,
                    'loaded_pkgs' => $in_transit_pkgs->pkg,
                    'unloaded_pkgs' => $in_transit_pkgs->pkg,
                    'material' => $desc->gr_desc->material['material_name'],
                    'consignor' => $desc->gr_desc->bookGr->consignor_id ?  $desc->gr_desc->bookGr->consignor['party_name'] : $desc->gr_desc->bookGr->consignor_name,
                    'consignee' => $desc->gr_desc->bookGr->consignee_id ? $desc->gr_desc->bookGr->consignee['party_name'] : $desc->gr_desc->bookGr->consignee_name,
                    'destination' => $desc->gr_desc->bookGr->destination_id ? $desc->gr_desc->bookGr->destination['station_name'] : $desc->gr_desc->bookGr->destination_name,
                    'grand_total' => $desc->gr_desc->bookGr->grand_total,
                    'pay_mode' => $desc->gr_desc->bookGr->pay_mode,
    
                    'checked' => true,
                ];

            }
            
        });
        return response()->json(['challan_descs' => $challan_descs]);
    }

    public function get_challan_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = Challan::with(['branch:id,branch_name,branch_code','receiver_branch:id,branch_code','source:id,station_name', 'vehicle:id,vehicle_number,owner_id','vehicle.party:id,party_name', 'driver:id,emp_name','destination:id,station_name','challan_desc','challan_desc.gr_desc:id,gr_id,gr_number,material_id','challan_desc.gr_desc.material:id,material_name','lorry_hire_from:id,account_name','broker:id,party_name','dr_on_account:id,account_name','paid_using:id,account_name',
            'voucher_desc'=>function($query){
                $query->where('against','Challan')->where('voucher_name','Payment Voucher');
            },
            'lhb_desc',
            'lhb_desc.lh_bill',
            'lhb_desc.lh_bill.voucher_desc',
            'lhb_desc.lh_bill.voucher_desc.voucher',
            'voucher_desc.voucher:id,voucher_number,narration,voucher_date',
            'diesel_party1:id,account_name',
            'diesel_party2:id,account_name',
            'diesel_party3:id,account_name',
            ])->where('fin_year',$request->current_year);

       
        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['challan_num_from']) && isset($report_data['challan_num_to'])){
            $query->where('challan_number','>=',$report_data['challan_num_from']['challan_number'])->where('challan_number','<=',$report_data['challan_num_to']['challan_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('challan_date','>=',$report_data['from_date'])->where('challan_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['movement_type']) ){
            $query->where('movement_type', $report_data['movement_type']);
        }

        if(isset($report_data['vehicle']) ){
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if (isset($report_data['owner'])) {
            $query->whereHas('vehicle.party', function ($query) use ($report_data) {
                $query->where('id', $report_data['owner']['id']);
            });
        }

        if (isset($report_data['gr_number'])) {
            $query->whereHas('challan_desc', function ($query) use ($report_data) {
                $query->where('gr_id', $report_data['gr_number']['id']);
            });
        }
        

        if(isset($report_data['driver']) ){
            $query->where('driver_id', $report_data['driver']['id']);
        }

        if(isset($report_data['broker']) ){
            $query->where('broker_id', $report_data['broker']['id']);
        }

        if(isset($report_data['lorry_hire_from']) ){
            $query->where('lorry_hire_from', $report_data['lorry_hire_from']['id']);
        }

        if(isset($report_data['source']) ){
            $query->where('source_id', $report_data['source']['id']);
        }

        if(isset($report_data['destination']) ){
            $query->where('destination_id', $report_data['destination']['id']);
        }

        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }


        if(isset($report_data['rcpt_status']) ){
            if($report_data['rcpt_status'] == 'Received'){
              $query->whereHas('challan_rec_desc');  
            }
            else if($report_data['rcpt_status'] == 'Not Received'){
                $query->whereDoesntHave('challan_rec_desc');
            }
        }

        if(isset($report_data['paid_status']) ){
            if($report_data['paid_status'] == 'Outstanding'){
              $query->where('paid_status', 0)->where('lh_net_freight','>',0);  
            }
            else if($report_data['paid_status'] == 'Paid'){
                $query->where(function($query){
                    $query->where('paid_status', 1)->orWhere('lh_net_freight',0);
                });
            }

            if($report_data['paid_status'] == 'Outstanding'){
                $column = 'lorry_hire_from';
            }
            
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('challan_number', 'like','%'. $searchValue . '%')->orWhere('vehicle_number', 'like','%'. $searchValue . '%');
            });
        }

        
        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_custom_fields(Request $request){
        $challan = Challan::select($request->custom_fields)->with($request->with_relations)->find($request->challan_id);
        return response()->json(['challan' => $challan]);
    }

    protected function get_max_number_from_challan_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected  function  record_transaction_in_accounts($request,$challan){
        $t_data = [];
        $t_data['t_against'] = 'CHALLAN';
        $t_data['t_against_id'] = $challan->id;
        $t_data['t_against_no'] = $challan->challan_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);

        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->challan_date;
        $t_data['voucher_time'] = $request->challan_time;
        
        $t_data['t_against_narration'] = $challan->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];
        
        $t_data['cancellable'] = 1;
        if($challan->dr_on_account && $challan->lorry_hire_from && $challan->lh_net_freight>0){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->dr_on_account, 'nature'=>'dr' , 'dr_amount'=>$challan->lh_net_freight, 'cr_amount'=>0 ];
            

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->lorry_hire_from, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$challan->lh_net_freight ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }

            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr;


            $this->create_transaction_voucher( $t_data );

        }
        
        $t_data['cancellable'] = 1;
        if($challan->lorry_hire_from && $challan->lh_tds>0){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$challan->lh_tds, 'cr_amount'=>0 ];
            
            // TDS PAYABLE
            $t_data['voucher_accounting_details'][] = ['account_id'=>4, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$challan->lh_tds ];
            
            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }

            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr;


            $this->create_transaction_voucher( $t_data );

        }
        $t_data['cancellable'] = 1;
        if($challan->lorry_hire_from && $challan->paid_using && $challan->lh_advance>0){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$challan->lh_advance, 'cr_amount'=>0 ];
            

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->paid_using, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$challan->lh_advance ];

           
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }

            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr;


            $this->create_transaction_voucher( $t_data );

        }

        $shortage_id = $this->get_acc_id('Shortage and Excess');
        if(!$shortage_id){
            dd('Shortage and Excess A/C not Found');
        }
        
        if($challan->lorry_hire_from && $shortage_id && $request->lh_shortage>0){
            
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->lh_shortage, 'cr_amount'=>0 ];
            

            // CREDIT SIDE
            $t_data['voucher_accounting_details'][] = ['account_id'=>$shortage_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->lh_shortage ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

           // Round both $total_dr and $total_cr to two decimal places
           $total_dr_rounded = round(floatval($total_dr), 2);
           $total_cr_rounded = round(floatval($total_cr), 2);

           // Check if they are not equal
           if ($total_dr_rounded != $total_cr_rounded) {
               // Output the values for debugging
               dd($total_dr_rounded, $total_cr_rounded);
               
               // Return response with an error message
               return response()->json(['message' => 'Error In Accounting Entries'], 422);
           }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }

        $material_cost_id = $this->get_acc_id('Material Cost');
        if(!$material_cost_id){
            dd('Material Cost A/C not Found');
        }
        
        if($challan->lorry_hire_from && $material_cost_id && $request->material_cost>0){
            
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->material_cost, 'cr_amount'=>0 ];
            

            // CREDIT SIDE
            $t_data['voucher_accounting_details'][] = ['account_id'=>$material_cost_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->material_cost ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

           // Round both $total_dr and $total_cr to two decimal places
           $total_dr_rounded = round(floatval($total_dr), 2);
           $total_cr_rounded = round(floatval($total_cr), 2);

           // Check if they are not equal
           if ($total_dr_rounded != $total_cr_rounded) {
               // Output the values for debugging
               dd($total_dr_rounded, $total_cr_rounded);
               
               // Return response with an error message
               return response()->json(['message' => 'Error In Accounting Entries'], 422);
           }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }

        $t_data['cancellable'] = 0;
        if($challan->lorry_hire_from && $challan->diesel_party1_id && $challan->diesel_amt1>0){
            $t_data['voucher_date'] = $challan->diesel_date1??$challan->challan_date;

            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$challan->diesel_amt1, 'cr_amount'=>0 ];
            

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->diesel_party1_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$challan->diesel_amt1 ];

           
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }

            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr;


            $this->create_transaction_voucher( $t_data );

        }

        $t_data['cancellable'] = 0;
        if($challan->lorry_hire_from && $challan->diesel_party2_id && $challan->diesel_amt2>0){
            $t_data['voucher_date'] = $challan->diesel_date2??$challan->challan_date;

            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$challan->diesel_amt2, 'cr_amount'=>0 ];
            

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->diesel_party2_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$challan->diesel_amt2 ];

           
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }

            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr;


            $this->create_transaction_voucher( $t_data );

        }

        $t_data['cancellable'] = 0;
        if($challan->lorry_hire_from && $challan->diesel_party3_id && $challan->diesel_amt3>0){
            $t_data['voucher_date'] = $challan->diesel_date3??$challan->challan_date;

            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$challan->diesel_amt3, 'cr_amount'=>0 ];
            

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->diesel_party3_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$challan->diesel_amt3 ];

           
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }

            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr;


            $this->create_transaction_voucher( $t_data );

        }



        // COMMISSION TRANSACTION
        // $t_data['cancellable'] = 1;
        // if($challan->dl_comm_branch && $challan->commission_account && $challan->dl_comm_branch_balance>0){
        //     $t_data['voucher_accounting_details']=[];
        //      // DEBIT PARTY ACCOUNT
        //      $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->commission_account, 'nature'=>'dr' , 'dr_amount'=>$challan->dl_comm_branch_balance, 'cr_amount'=>0 ];
            

        //      // CREDIT INCOME ACCOUNT
        //      $t_data['voucher_accounting_details'][] = ['account_id'=>$challan->dl_comm_branch, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$challan->dl_comm_branch_balance ];


        //     //CHECK TOTAL DR AND CR
        //     $total_dr = 0;
        //     $total_cr = 0;

        //     foreach ($t_data['voucher_accounting_details'] as $vad) {
        //         $total_dr += $vad['dr_amount'];
        //         $total_cr += $vad['cr_amount'];
        //     }
            

            // Round both $total_dr and $total_cr to two decimal places
            // $total_dr_rounded = round(floatval($total_dr), 2);
            // $total_cr_rounded = round(floatval($total_cr), 2);

            // // Check if they are not equal
            // if ($total_dr_rounded != $total_cr_rounded) {
            //     // Output the values for debugging
            //     dd($total_dr_rounded, $total_cr_rounded);
                
            //     // Return response with an error message
            //     return response()->json(['message' => 'Error In Accounting Entries'], 422);
            // }

        //     $t_data['total_dr_amount'] = $total_dr;
        //     $t_data['total_cr_amount'] = $total_cr;


        //     $this->create_transaction_voucher( $t_data );
        // }
    }

}
