<?php

namespace App\Http\Controllers\Inventory;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use Illuminate\Http\Request;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Bill\BillDesc;
use App\Models\Inventory\ProductSerial;
use App\Models\Inventory\Purchase;
use App\Models\Inventory\PurchaseDesc;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class PurchaseController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Bills');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $bill = Purchase::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'purchase_desc',
            'purchase_desc.product:id,product_name',
            'purchase_desc.vehicle:id,vehicle_number',
            'purchase_desc.dr_account:id,account_name',
            'purchase_desc.product_serial_desc',
            'purchase_desc.product_serial_desc.vehicle:id,vehicle_number',
        ])->where('fin_year', $request->current_year)->get();


        return response()->json(['bills'=>$bill]);
    }

    public function show(Request $request, $pk){
        //FETCH AND SEND ALL
        $bill = Purchase::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'purchase_desc',
            'purchase_desc.product:id,product_name',
            'purchase_desc.vehicle:id,vehicle_number',
            'purchase_desc.dr_account:id,account_name',
            'purchase_desc.product_serial_desc',
            'purchase_desc.product_serial_desc.vehicle:id,vehicle_number',
        ])->find($pk);


        return response()->json(['bills'=>$bill]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'Bills');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'bill_number' => 'required',
        ], [
            'branch.required' => 'Required',
            'bill_number.required' => 'Required',
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            $max_number = $this->get_max_number_from_bill_number($request->bill_number);

            
            // SAVING A NEW Bill
            $bill = new Purchase();
            $bill->bill_name = $request->bill_name;
            $bill->branch_id = $request->branch['id'];
            $bill->bill_type = $request->bill_type;
            $bill->prefix = $request->prefix;
            $bill->max_number = $max_number;
            $bill->bill_number = $request->bill_number;
            $bill->bill_date = $request->bill_date;
            $bill->acc_party_id = $request->acc_party?$request->acc_party['id']:null;
            $bill->invoice_number = $request->invoice_number;
            $bill->bill_attachment = $request->bill_attachment; 
            $bill->bill_attachment2 = $request->bill_attachment2;
            $bill->bill_attachment3 = $request->bill_attachment3;
            $bill->bill_attachment4 = $request->bill_attachment4;
            $bill->bill_attachment5 = $request->bill_attachment5;
            $bill->remarks = $request->remarks;
            $bill->total_discount = $request->total_discount;
            $bill->total_without_gst = $request->total_without_gst;
            $bill->total_igst_amount = $request->total_igst_amount;
            $bill->total_cgst_amount = $request->total_cgst_amount;
            $bill->total_sgst_amount = $request->total_sgst_amount;
            $bill->total_without_rounding = $request->total_without_rounding;
            $bill->rounded_off = $request->rounded_off;
            $bill->grand_total_amount = $request->grand_total_amount;
            
            $bill->fin_year = $request->current_year;

            $bill->created_by = $request->user->id;

            // Save the model to the database
            $bill->save();

             

            // SAVING BILL DESC
            $bill_desc = $request->bill_desc;

                foreach ($bill_desc as $bd) {
                    if($bill->bill_name == 'Stock Issue' || $bill->bill_name == 'Sale'){
                        $bal_data = [
                            'product_id'=>$bd['product']['id'],
                            'branch_id'=>$bill->branch_id,
                            'current_year'=>$request->current_year,
                        ];

                        $balance_of_stock = $this->calculate_balance_for_inventory($bal_data);

                        $current_bal = $balance_of_stock['balance']['current_balance'];

                        $future_bal = $current_bal - $bd['qty'];

                        if($future_bal<0){
                            return response()->json(['error'=>'Current Balance Exceeded', 'current_bal'=>$current_bal], 500);
                        }
                    }

                    $bill_desc = new PurchaseDesc();
                    $bill_desc->bill_id = $bill->id;
                    $bill_desc->bill_number = $bill->bill_number;
                    $bill_desc->product_id = $bd['product']['id'];
                    $bill_desc->dr_account_id = $bd['dr_account']?$bd['dr_account']['id']:null;
                    $bill_desc->vehicle_id = $bd['vehicle']?$bd['vehicle']['id']:null;
                    $bill_desc->reading = $bd['reading'];
                    $bill_desc->hsn = $bd['hsn'];
                    $bill_desc->qty = $bd['qty'];
                    $bill_desc->rate = $bd['rate'];
                    $bill_desc->discount = $bd['discount'];
                    $bill_desc->amount = $bd['amount'];
                    $bill_desc->cgst_percent = $bd['cgst_percent'];
                    $bill_desc->cgst_amount = $bd['cgst_amount'];
                    $bill_desc->sgst_percent = $bd['sgst_percent'];
                    $bill_desc->sgst_amount = $bd['sgst_amount'];
                    $bill_desc->igst_percent = $bd['igst_percent'];
                    $bill_desc->igst_amount = $bd['igst_amount'];
                    $bill_desc->net_amount = $bd['net_amount'];
                    $bill_desc->fin_year = $request->current_year;
                    $bill_desc->save();

                    $product_serial_desc = $bd['product_serial_desc'];

                    foreach ($product_serial_desc as $psd) {
                        $product_serial = new ProductSerial();
                        $product_serial->bill_id = $bill->id;
                        $product_serial->bill_number = $bill->bill_number;
                        $product_serial->product_id = $psd['product_id'];
                        $product_serial->product_name = $psd['product_name'];
                        $product_serial->ref_number = $psd['ref_number'] ?? null;
                        $product_serial->position = $psd['position'] ?? null;
                        $product_serial->vehicle_id = $psd['vehicle'] ? $psd['vehicle']['id'] : null;
                        $product_serial->bill_desc_id = $bill_desc->id;
                        $product_serial->save(); 
                    }


                   

                }

            // BILL DESC SAVED

            
            

            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request, $bill);
            // TRANSACTION RECORDED IN ACCOUNTS

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($bill->id);
            $this->createEditLog('created', 'purchases', $bill->id, $bill->bill_number,null, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            
        
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$bill->id, 'bill_number'=>$bill->bill_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'Bills');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $bill = Purchase::find($pk);

        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($bill->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $validatedData = $request->validate([
            'branch' => 'required',
            'bill_number' => 'required',
        ], [
            'branch.required' => 'Required',
            'bill_number.required' => 'Required',
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);
            
            // SAVING A NEW BILL
            $bill->bill_date = $request->bill_date;
            $bill->acc_party_id = $request->acc_party?$request->acc_party['id']:null;
            $bill->invoice_number = $request->invoice_number;
            $bill->remarks = $request->remarks;
            $bill->bill_attachment = $request->bill_attachment;
            $bill->bill_attachment2 = $request->bill_attachment2;
            $bill->bill_attachment3 = $request->bill_attachment3;
            $bill->bill_attachment4 = $request->bill_attachment4;
            $bill->bill_attachment5 = $request->bill_attachment5;
            $bill->total_discount = $request->total_discount;
            $bill->total_without_gst = $request->total_without_gst;
            $bill->total_igst_amount = $request->total_igst_amount;
            $bill->total_cgst_amount = $request->total_cgst_amount;
            $bill->total_sgst_amount = $request->total_sgst_amount;
            $bill->total_without_rounding = $request->total_without_rounding;
            $bill->rounded_off = $request->rounded_off;
            $bill->grand_total_amount = $request->grand_total_amount;

            $bill->updated_by = $request->user->id;
            $bill->save();
            // NEW BILL SAVED
            
            // SAVING Bill DESC
            $newly_added_bill_desc = $request->newly_added_bill_desc;
            foreach ($newly_added_bill_desc as $nabd) { 
                if($bill->bill_name == 'Stock Issue' || $bill->bill_name == 'Sale'){
                    $bal_data = [
                        'product_id'=>$nabd['product']['id'],
                        'branch_id'=>$bill->branch_id,
                        'current_year'=>$request->current_year,
                    ];

                    $balance_of_stock = $this->calculate_balance_for_inventory($bal_data);

                    $current_bal = $balance_of_stock['balance']['current_balance'];

                    $future_bal = $current_bal - $nabd['qty'];

                    if($future_bal<0){
                        return response()->json(['error'=>'Current Balance Exceeded', 'current_bal'=>$current_bal], 500);
                    }
                }

                $bill_desc = new PurchaseDesc();
                $bill_desc->bill_id = $bill->id;
                $bill_desc->bill_number = $bill->bill_number;
                $bill_desc->product_id = $nabd['product']['id'];
                $bill_desc->dr_account_id = $nabd['dr_account']?$nabd['dr_account']['id']:null;
                $bill_desc->vehicle_id = $nabd['vehicle']?$nabd['vehicle']['id']:null;
                $bill_desc->reading = $nabd['reading'];
                $bill_desc->hsn = $nabd['hsn'];
                $bill_desc->qty = $nabd['qty'];
                $bill_desc->rate = $nabd['rate'];
                $bill_desc->discount = $nabd['discount'];
                $bill_desc->amount = $nabd['amount'];
                $bill_desc->cgst_percent = $nabd['cgst_percent'];
                $bill_desc->cgst_amount = $nabd['cgst_amount'];
                $bill_desc->sgst_percent = $nabd['sgst_percent'];
                $bill_desc->sgst_amount = $nabd['sgst_amount'];
                $bill_desc->igst_percent = $nabd['igst_percent'];
                $bill_desc->igst_amount = $nabd['igst_amount'];
                $bill_desc->net_amount = $nabd['net_amount'];
                $bill_desc->fin_year = $request->current_year;
                $bill_desc->save();

                $product_serial_desc = $nabd['product_serial_desc'];

                foreach ($product_serial_desc as $psd) {
                    $product_serial = new ProductSerial();
                    $product_serial->bill_id = $bill->id;
                    $product_serial->bill_number = $bill->bill_number;
                    $product_serial->product_id = $psd['product_id'];
                    $product_serial->product_name = $psd['product_name'];
                    $product_serial->ref_number = $psd['ref_number'] ?? null;
                    $product_serial->position = $psd['position'] ?? null;
                    $product_serial->vehicle_id = $psd['vehicle'] ? $psd['vehicle']['id'] : null;
                    $product_serial->bill_desc_id = $bill_desc->id;
                    $product_serial->save(); 
                }
                
            }
            // Bill DESC SAVED

            $updated_bill_desc = $request->updated_bill_desc;
            foreach ($updated_bill_desc as $ubd) {    
                $bill_desc = PurchaseDesc::find($ubd['id']);

                $bal_data = [
                    'product_id'=>$ubd['product']['id'],
                    'branch_id'=>$bill->branch_id,
                    'current_year'=>$request->current_year,
                ];

                $balance_of_stock = $this->calculate_balance_for_inventory($bal_data);

                $current_bal = $balance_of_stock['balance']['current_balance'];

                if($bill->bill_name == 'Purchase')
                $future_bal = $current_bal - $bill_desc['qty'] + $ubd['qty'];
                else if($bill->bill_name == 'Stock Issue' || $bill->bill_name == 'Sale')
                $future_bal = $current_bal + $bill_desc['qty'] - $ubd['qty'];


                if($future_bal<0){
                    return response()->json(['error'=>'Current Balance Exceeded', 'current_bal'=>$current_bal], 500);
                }

                $bill_desc->bill_id = $bill->id;
                $bill_desc->bill_number = $bill->bill_number;
                $bill_desc->qty = $ubd['qty'];
                $bill_desc->rate = $ubd['rate'];
                $bill_desc->discount = $ubd['discount'];
                $bill_desc->amount = $ubd['amount'];
                $bill_desc->cgst_percent = $ubd['cgst_percent'];
                $bill_desc->cgst_amount = $ubd['cgst_amount'];
                $bill_desc->sgst_percent = $ubd['sgst_percent'];
                $bill_desc->sgst_amount = $ubd['sgst_amount'];
                $bill_desc->igst_percent = $ubd['igst_percent'];
                $bill_desc->igst_amount = $ubd['igst_amount'];
                $bill_desc->net_amount = $ubd['net_amount'];
                $bill_desc->save();

                $product_serial = ProductSerial::where('bill_desc_id', $bill_desc->id)->delete();
                 
                $product_serial_desc = $ubd['product_serial_desc'];
                foreach ($product_serial_desc as $psd) {
                    $product_serial = new ProductSerial();
                    $product_serial->bill_id = $bill->id;
                    $product_serial->bill_number = $bill->bill_number;
                    $product_serial->product_id = $psd['product_id'];
                    $product_serial->product_name = $psd['product_name'];
                    $product_serial->ref_number = $psd['ref_number'] ?? null;
                    $product_serial->position = $psd['position'] ?? null;
                    $product_serial->vehicle_id = $psd['vehicle'] ? $psd['vehicle']['id'] : null;
                    $product_serial->bill_desc_id = $bill_desc->id;
                    $product_serial->save(); 
                }

            }



            // $updated_product_serial = $request->updated_product_serial;
            // foreach ($updated_product_serial as $psd) {
            //     $product_serial = ProductSerial::find($psd['id']);
            //     $product_serial->ref_number = $psd['ref_number'] ?? null;
            //     $product_serial->position = $psd['position'] ?? null;
            //     $product_serial->save(); 
            // }
                

            // DELETING EXISTING POD DESC
            $deleted_bill_desc = $request->deleted_bill_desc;
            if( !empty($deleted_bill_desc) ){
                foreach ($deleted_bill_desc as $dbd) {
                    $d_bil_d = PurchaseDesc::find($dbd);

                    if($bill->bill_name == 'Purchase'){
                        $bal_data = [
                            'product_id'=>$d_bil_d['product_id'],
                            'branch_id'=>$bill->branch_id,
                            'current_year'=>$request->current_year,
                        ];

                        $balance_of_stock = $this->calculate_balance_for_inventory($bal_data);

                        $current_bal = $balance_of_stock['balance']['current_balance'];

                        $future_bal = $current_bal - $d_bil_d['qty'];

                        if($future_bal<0){
                            return response()->json(['error'=>'Current Balance Falls Below 0', 'current_bal'=>$current_bal], 500);
                        }
                    }

                    $product_serial = ProductSerial::where('bill_desc_id', $d_bil_d->id)->delete();

                    $d_bil_d->delete();
                }      
            }
            //DELETED EXISTING POD DESC

            // RECORD THE TRANSACTION IN ACCOUNTS
             $this->record_transaction_in_accounts($request, $bill);
            // TRANSACTION RECORDED IN ACCOUNTS
            
            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($bill->id);
            $this->createEditLog('updated', 'purchases', $bill->id, $bill->bill_number,$oldValue, $newVal, $request->user->id,$request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'Bills');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
            
        
        $bill = Purchase::find($pk);
                
        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($bill->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        // LOOP THROUGH BILL DESC, CHECK selection_type, If 'GR' send gr_number  , If 'Loading Slip' send ls_number
        // $bill_desc_ids = PurchaseDesc::select('id')->where('bill_id', $bill->id)->pluck('id')->toArray();
        
        // DELETING BILL DESCRIPTIONS

        $product_serial = ProductSerial::where('bill_id',$pk)->delete();

        $bill_desc = PurchaseDesc::select('id')->where('bill_id', $pk)->pluck('id')->toArray();

        // DELETING EXISTING POD DESC
        if( !empty($bill_desc) ){
            foreach ($bill_desc as $dbd) {
                $d_bil_d = PurchaseDesc::find($dbd);
                  
                  
                if($bill->bill_name == 'Purchase'){
                    $bal_data = [
                        'product_id'=>$d_bil_d['product_id'],
                        'branch_id'=>$bill->branch_id,
                        'current_year'=>$request->current_year,
                    ];



                    $balance_of_stock = $this->calculate_balance_for_inventory($bal_data);

                    $current_bal = $balance_of_stock['balance']['current_balance'];

                    $future_bal = $current_bal - $d_bil_d['qty'];

                    if($future_bal<0){
                        return response()->json(['error'=>'Current Balance Falls Below 0', 'current_bal'=>$current_bal], 500);
                    }
                }

                $d_bil_d->delete();
            }      
        }
        //DELETED EXISTING POD DESC


        //FIND AND DELETE THE BILL
        $bill->delete();


        // DELETE THE TRANSACTION VOUCHER
        $t_data = [];
        $t_data['t_against'] = 'Purchase Bills';
        $t_data['t_against_id'] = $bill->id;
        $t_data['t_against_no'] = $bill->bill_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);

        $this->createEditLog('deleted', 'purchases', $bill->id, $bill->bill_number, $oldValue, null, $request->user->id,$request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = Purchase::select('id', 'bill_number')->where('bill_name',$request->bill_name)->where('fin_year', $request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

            // Loop through the whereFilters array
            foreach ($whereFilters as $whereFilter) {
                // Apply the filter to the query
                $query->where($whereFilter['key'], $whereFilter['value']);
            }

            // Loop through the orWhereFilters array
            foreach ($orWhereFilters as $orWhereFilter) {
                // Apply the filter to the query
                $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
            }
        
        }


        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('bill_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $bill = $query->get();

        return response(['bills' => $bill, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Bills');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Purchase::with(['branch:id,branch_name,branch_code',
        'acc_party:id,account_name',
        'purchase_desc',
        'purchase_desc.product:id,product_name',
        'purchase_desc.vehicle:id,vehicle_number',
        'purchase_desc.dr_account:id,account_name',
        'purchase_desc.product_serial_desc',
            'purchase_desc.product_serial_desc.vehicle:id,vehicle_number'])->where('fin_year', $request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('bill_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Purchase::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $billQuery = Purchase::select('id', 'bill_number')->where('fin_year', $request->current_year);
    
        if (!empty($request->user_branch_ids)) {
            $billQuery->whereIn('branch_id', $request->user_branch_ids);
        }
    
        $bills = $billQuery->orderBy('id', 'desc')->take(10)->get();
    
        return response()->json(['bills' => $bills]);
    }

    public function get_unused_ref_numbers(Request $request){
        $product_id = $request->product_id;
        
        // Pluck ref_numbers from product_serials
        $query = ProductSerial::select('id', 'product_id', 'ref_number')
                ->where('product_id', $product_id)
                ->whereNull('vehicle_id'); // Select records where vehicle_id is null

            // Exclude records with ref_number having associated vehicle_id
            $query->whereNotExists(function ($subquery) {
                $subquery->select(DB::raw(1))
                    ->from('product_serials as psd')
                    ->whereRaw('psd.ref_number = product_serials.ref_number')
                    ->whereNotNull('vehicle_id');
            });

            // Check if searchVariable is provided and apply the filter
            if ($request->has('searchVariable')) {
                $searchVariable = $request->searchVariable;
                $query->where('ref_number', 'like', '%' . $searchVariable . '%');
            }

           $results = $query->get();


        return response()->json(['product_serials' => $results]);
    }
    


    public function get_next_bill_number(Request $request){
        $type=$request->type;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;
        $bill_name = $request->bill_name;

        $year_stamp = $this->get_short_name($request->current_year);

        if($bill_name=='Purchase'){
            $prefix = 'PUR';
        }
        elseif($bill_name=='Service'){
            $prefix = 'SER';
        }
        elseif($bill_name=='Stock Issue'){
            $prefix = 'SI';
        }
        elseif($bill_name=='Sale'){
            $prefix = 'SAL';
        }
        

        if($type=='SYSTEM'){
            $max = Purchase::where('branch_id',$branch_id)->where('bill_type', $type)->where('bill_name',$bill_name)->where('created_by',$request->user->id)->max('max_number');
           
            // First 4 characters of the user;
            $user_tag = str_replace(' ', '', substr($request->user->username, 0, 4));
            $bill_number = $prefix.'-'.$branch_code.'-'.$user_tag.'-'.sprintf('%06d', $max+1);        

            return response()->json(['bill_number' => strtoupper($bill_number), 'max_number' => $max+1]);
        }
    }

    public function get_inventory_bill_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = Purchase::with(['branch:id,branch_name,branch_code', 
            'acc_party:id,account_name',
            'purchase_desc',
            'purchase_desc.product:id,product_name',
            'purchase_desc.product_serial_desc',
            'purchase_desc.product_serial_desc.vehicle:id,vehicle_number',
            'voucher_desc'=>function($query){
            $query->where('against','Purchase')->where('voucher_name','Payment Voucher');
        }])->where('fin_year', $request->current_year);

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }

        if(isset($report_data['bill_num_from']) && isset($report_data['bill_num_to'])){
            $query->where('bill_number','>=',$report_data['bill_num_from']['bill_number'])->where('bill_number','<=',$report_data['bill_num_to']['bill_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('bill_date','>=',$report_data['from_date'])->where('bill_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['bill_name']) ){
            $query->where('bill_name', $report_data['bill_name']);
        }

        if (isset($report_data['product'])) {
            $query->whereHas('purchase_desc', function ($qr) use ($report_data) {
                $qr->where('product_id', $report_data['product']['id']);
            });
        }
        
        


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('bill_number', 'like', '%'. $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function get_inventory_issue_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = PurchaseDesc::where('vehicle_id','!=', null)
                ->with(['product:id,product_name','vehicle:id,vehicle_number','purchase:id,bill_number,bill_date'])
                ->whereHas('purchase', function($qr) use ($report_data) {
                    $qr->where('bill_name', 'Stock Issue')
                    ->when($report_data['from_date'], function($sub) use ($report_data) {
                        $sub->where('bill_date', '>=', $report_data['from_date'])
                            ->where('bill_date', '<=', $report_data['to_date']);
                    });
                })
                ->where('fin_year', $request->current_year);

        
        if (isset($report_data['product'])) {
            $query->where('product_id', $report_data['product']['id']);
        }
        
        if (isset($report_data['vehicle'])) {
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('bill_number', 'like', '%'. $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }



    protected function check_for_balance($against,$default_balance,$selection){
            $filteredVoucherDesc = $against->voucher_desc->where('against', $selection);
            
            if ($filteredVoucherDesc->isEmpty()) {
                // No voucher_desc, return grand_total_amount as balance
                return $default_balance;
            }
            
            $recievedAmtSum = $filteredVoucherDesc->sum('receipt_amt');
            $deductionSum = $filteredVoucherDesc->sum('deduction');
            
            $balance = $default_balance - $recievedAmtSum - $deductionSum ;
            return $balance;
    }





    protected function get_data_for_obj($pk){
        $bill = Purchase::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'purchase_desc',
            'purchase_desc.product:id,product_name',
            'purchase_desc.vehicle:id,vehicle_number',
            'purchase_desc.dr_account:id,account_name',
            'purchase_desc.product_serial_desc',
            'purchase_desc.product_serial_desc.vehicle:id,vehicle_number',
        ])->find($pk);


        return ['bills'=>$bill];
    }

    protected function get_max_number_from_bill_number($input){
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected function calculate_balance_for_inventory($request){
        $pk = $request['product_id'];
        $branch_id = $request['branch_id'];
        $current_year = $request['current_year'];

        $min_max_date = $this->get_min_max_date($current_year);
        $min_date = $min_max_date->min_date;
        $from_date = $min_date;

        // Today's date
        $to_date = Carbon::now()->toDateString();

        $balance_details = $this->calculate_balance_for_inventory_stock($pk,$min_date,$from_date,$to_date,$current_year,$branch_id);
        return ['balance' => $balance_details];
    }

    


    protected function record_transaction_in_accounts($request, $bill){
        $t_data = [];
        $t_data['t_against'] = 'Purchase Bills';
        $t_data['t_against_id'] = $bill->id;
        $t_data['t_against_no'] = $bill->bill_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->bill_date;
        $t_data['voucher_time'] = $request->bill_time;
        $t_data['t_against_narration'] = $bill->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];


        if($bill->bill_name=='Purchase' || $bill->bill_name=='Service'){
            $purchaseDescQuery = PurchaseDesc::where('bill_id', $bill->id);

            $bill_desc = $purchaseDescQuery->get();

            $sums = $purchaseDescQuery->selectRaw('SUM(cgst_amount) as total_cgst, SUM(sgst_amount) as total_sgst, SUM(igst_amount) as total_igst, SUM(amount) as total_amount')
                ->first();

            
            $total_cgst = $sums->total_cgst ?? 0;
            $total_sgst = $sums->total_sgst ?? 0;
            $total_igst = $sums->total_igst ?? 0;
            $total_amount = $sums->total_amount ?? 0;

            
            $t_data['cancellable'] = 1;
            
            if($bill->acc_party_id && $total_amount>0 ){
                foreach ($bill_desc as $key) {
                    // DEBIT ALL THE ACCOUNTS IN DESC
                    if($key['dr_account_id'] && $key['amount']){
                        $t_data['voucher_accounting_details'][] = ['account_id'=>$key['dr_account_id'], 'nature'=>'dr' , 'dr_amount'=>$key['amount'], 'cr_amount'=>0 ];
                    }
                }

                // CREDIT PARTY AND GST
                // CREDIT SIDE
                if($total_cgst > 0){ 
                    // CREDIT CGST ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>9, 'nature'=>'dr' , 'dr_amount'=>$total_cgst, 'cr_amount'=>0 ];
                }
                if($total_sgst > 0){
                    // CREDIT SGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>10, 'nature'=>'dr' , 'dr_amount'=>$total_sgst, 'cr_amount'=>0 ];
                }
                if($total_igst > 0){
                    // CREDIT IGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>11, 'nature'=>'dr' , 'dr_amount'=>$total_igst, 'cr_amount'=>0 ];
                }

                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$bill->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$bill->total_without_rounding ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }


                // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


                


                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 

                

                $this->create_transaction_voucher( $t_data );
            }
        }
        else if($bill->bill_name=='Sale'){
            $purchaseDescQuery = PurchaseDesc::where('bill_id', $bill->id);

            $bill_desc = $purchaseDescQuery->get();

            $sums = $purchaseDescQuery->selectRaw('SUM(cgst_amount) as total_cgst, SUM(sgst_amount) as total_sgst, SUM(igst_amount) as total_igst, SUM(amount) as total_amount')
                ->first();

            
            $total_cgst = $sums->total_cgst ?? 0;
            $total_sgst = $sums->total_sgst ?? 0;
            $total_igst = $sums->total_igst ?? 0;
            $total_amount = $sums->total_amount ?? 0;

            
            $t_data['cancellable'] = 1;
            
            if($bill->acc_party_id && $total_amount>0 ){
                foreach ($bill_desc as $key) {
                    // DEBIT ALL THE ACCOUNTS IN DESC
                    if($key['dr_account_id'] && $key['amount']){
                        $t_data['voucher_accounting_details'][] = ['account_id'=>$key['dr_account_id'], 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$key['amount'] ];
                    }
                }

                // CREDIT PARTY AND GST
                // CREDIT SIDE
                if($total_cgst > 0){ 
                    // CREDIT CGST ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>12, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$total_cgst ];
                }
                if($total_sgst > 0){
                    // CREDIT SGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>13, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$total_sgst ];
                }
                if($total_igst > 0){
                    // CREDIT IGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>14, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$total_sgst ];
                }

                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$bill->acc_party_id, 'nature'=>'dr' , 'dr_amount'=>$bill->total_without_rounding, 'cr_amount'=>0 ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }


                // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


                


                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 

                

                $this->create_transaction_voucher( $t_data );
            }
        }
        elseif($bill->bill_name=='Stock Issue'){
        }

        

    }

    
    
}

