<?php

namespace App\Http\Controllers\Master;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Http\Traits\CommonTrait;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Account\VoucherDesc;
use App\Models\Booking\BookGr;
use Illuminate\Support\Facades\DB;
use App\Models\Master\FuelAllocationMaster;

use Carbon\Carbon;

class FuelAllocationMasterController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        
    }

    public function show(Request $request, $pk){
      $test = $this->is_read_permitted($request->user->id, 'FuelAllocationMaster');
      if(!$test){
        return response()->json(['Permission Denied For The Action'],401);
      }
      $FuelAllocationMaster = FuelAllocationMaster::find($pk);
      return response()->json(['FuelAllocationMaster'=>$FuelAllocationMaster]);
    }

    public function store(Request $request){
        $test = $this->is_new_permitted($request->user->id, 'FuelAllocationMaster');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        $validatedData = $request->validate([
            'fuel_allocation_no' => 'required|unique:fuel_allocation_masters,fuel_allocation_no',
            'source' => 'required',
            'destination' => 'required',
            'fuel_type' => 'required',
            'fuel_qty' => 'required',
        ]);

        try{
            $source = $request->source;
            $source_name  = $source['station_name'];
            $source_id = $source['id'];

            $destination = $request->destination;
            $destination_name  = $destination['station_name'];
            $destination_id = $destination['id'];

            $fuel_type = $request->fuel_type;
            $fuel_qty = $request->fuel_qty;
            $driver_advance = $request->driver_advance;

             // TRANSACTION BEGINS
            DB::beginTransaction();  
            $fuel = new FuelAllocationMaster();
            #return response()->json($request->fuel_allocation_no);
            $fuel->fuel_allocation_no = $request->fuel_allocation_no;
            $fuel->source_name = $source_name;
            $fuel->source_id = $source_id;
            $fuel->destination_name = $destination_name;
            $fuel->destination_id = $destination_id;
            $fuel->fuel_type = $fuel_type;
            $fuel->fuel_qty = $fuel_qty;
            $fuel->driver_advance = $driver_advance;

            $fuel->created_by = $request->user->id;

            $cheack = $fuel->save();
            ## die($cheack);
            if(!$cheack){
                return response()->json(['status' => 'error', 'message' => 'Failed to save the fuel allocation record'],500);
            }
            $newVal = $this->get_data_for_obj($fuel->id,$request->current_year);

            $this->createEditLog('created', 'fuel_allocation_masters', $fuel->id, $fuel->fuel_allocation_no , null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG

            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE
            
        }catch(\Exception $e){
            DB::rollBack();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $test = $this->is_edit_permitted($request->user->id, 'FuelAllocationMaster');
        $test_edit = $test->edit;
        $edit_valid_till_hours = $test->edit_valid_till;

        if( !$test_edit ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        $validatedData = $request->validate([
            'fuel_allocation_no' => 'required',
            'source' => 'required',
            'destination' => 'required',
            'fuel_type' => 'required',
            'fuel_qty' => 'required',
        ]);
        $fuel = FuelAllocationMaster::find($pk);

                // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk,$request->current_year);

        
        if( $edit_valid_till_hours!=null ){
            $validity = Carbon::parse($fuel->created_at)->addHours($edit_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }
        try {
            $source = $request->source;
            $source_name  = $source['station_name'];
            $source_id = $source['id'];

            $destination = $request->destination;
            $destination_name  = $destination['station_name'];
            $destination_id = $destination['id'];

            $fuel_type = $request->fuel_type;
            $fuel_qty = $request->fuel_qty;
            $driver_advance = $request->driver_advance;

             // TRANSACTION BEGINS
            DB::beginTransaction();  
            
            #return response()->json($request->fuel_allocation_no);
            $fuel->source_name = $source_name;
            $fuel->source_id = $source_id;
            $fuel->destination_name = $destination_name;
            $fuel->destination_id = $destination_id;
            $fuel->fuel_type = $fuel_type;
            $fuel->fuel_qty = $fuel_qty;
            $fuel->driver_advance = $driver_advance;

            $fuel->updated_by = $request->user->id;
            $fuel->save();

            $newVal = $this->get_data_for_obj($fuel->id,$request->current_year);

            $this->createEditLog('updated', 'fuel_allocation_masters', $fuel->id, $fuel->fuel_allocation_no , $oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();


            return response()->json("Success", 200); // RETURNED RESPONSE
        }catch(\Exception $e){
            DB::rollBack();
            return response()->json(['error' => $e->getMessage()], 500);
        }
        
    }

    public function destroy(Request $request, $pk){
        $test = $this->is_delete_permitted($request->user->id, 'FuelAllocationMaster');
        $test_delete = $test->delete;
        $delete_valid_till_hours = $test->delete_valid_till;

        if( !$test_delete ){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $fuel = FuelAllocationMaster::find($pk);

        if( $delete_valid_till_hours!=null ){
            $validity = Carbon::parse($fuel->created_at)->addHours($delete_valid_till_hours);
            if( !$validity->isFuture() ){
                return response()->json(['Permission Timed Out'],500);
            }
        }

        $oldValue = $this->get_data_for_obj($pk,$request->current_year);
        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();
           //FIND AND DELETE THE STATION

            $fuel->delete();

            $this->createEditLog('deleted', 'fuel_allocation_masters', $fuel->id, $fuel->fuel_allocation_no , $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG

            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json("Success", 200); // RETURNED RESPONSE
        }catch(\Exception $e){
            DB::rollBack();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function get_next_fuel_allocation_no(Request $request){
        $test = $this->is_new_permitted($request->user->id, 'FuelAllocationMaster');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        // Assuming FuelAllocationMaster is a model representing the fuel allocation table
        $FuelAllocationMaster = new FuelAllocationMaster();
    
        // Get the maximum fuel allocation number from the table
        $maxFuelAllocationNo = $FuelAllocationMaster::max('fuel_allocation_no');
    
        // Generate the next fuel allocation number by incrementing the max number
        $nextFuelAllocationNo = $maxFuelAllocationNo ? $maxFuelAllocationNo + 1 : 1;
    
        if($nextFuelAllocationNo){
            return response()->json([
                'status' => 'success',
                'max_number' => $nextFuelAllocationNo
            ]);
        }else{
            return response()->json([
                'status' => 'error',
                'message' => 'Failed to generate the next fuel allocation number'
            ]);
        }

    }
    
    public function autocomplete(Request $request){
        $query = FuelAllocationMaster::select('id','fuel_allocation_no','source_name','destination_name');
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('fuel_allocation_no', 'like', '%'.$searchVariable.'%')->orwhere('source_name', 'like', '%'.$searchVariable.'%')->orwhere('destination_name', 'like', '%'.$searchVariable.'%');
        }
        $query->orderBy('fuel_allocation_no', 'asc');
        return response(['fuel_allocation_masters' => $query->get(),'draw' => $request->draw]);
    }

    public function fuel_usage_report(Request $request){
        $report_data = $request->report_data;
                                                                                                                 
        $query = BookGr::with(
                            [
                                'source:id,station_name','destination:id,station_name','vehicle:id,vehicle_number',
                                'diesel_receipt:id,dr_number,booking_id,against,qty,rate,amount,cash_amount'
                            ]
                        )
                        ->where('fin_year',$request->current_year);

                        $query->withSum('diesel_receipt', 'qty')->withSum('diesel_receipt', 'amount')->withSum('diesel_receipt', 'cash_amount');

       


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('gr_date','>=',$report_data['from_date'])->where('gr_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['vehicle']) ){
            $query->where('vehicle_id', $report_data['vehicle']['id']);
        }

        if(isset($report_data['source']) ){
            $query->where('source_id', $report_data['source']['id']);
        }

        if(isset($report_data['destination']) ){
            $query->where('destination_id', $report_data['destination']['id']);
        }        

        $data = $query->get();

        // Map data 
        $result_data = [];
        foreach ($data as $key) {

            // Check in FuelAllocationMaster, to know the approved_fuel
            $fuel_allocation = FuelAllocationMaster::where('source_id',$key['source_id'])->where('destination_id',$key['destination_id'])->first();
            $approved_fuel =  $fuel_allocation->fuel_qty??0;
            $diesel_receipt_fuel_sum = $key['diesel_receipt_sum_qty']??0;
            $diff1 = $diesel_receipt_fuel_sum - $approved_fuel;

            $approved_advance =  $fuel_allocation->driver_advance??0;


            $driver_account_id = $this->get_acc_id($key->driver_name);
            $v_advance = 0;

            if($driver_account_id){
                $v_advance = VoucherDesc::where('against', 'GR_COST_CENTER') 
                ->where('against_id', $key->id) 
                ->whereHas('voucher_acc_details', function ($query) use ($driver_account_id) {
                    $query->where('account_id', $driver_account_id);
                })
                ->sum('received_amt'); 

            }   
            $alloted_advance = $key['driver_payment_amt'] + $key['diesel_receipt_sum_cash_amount'] + $v_advance;


            $diff2 =  $alloted_advance - $approved_advance ;

            $result_data[] = [
                  'id' => $key['id'],
                  'gr_number' => $key['gr_number'],
                  'gr_date' => $key['gr_date'],
                  'source_id' => $key['source_id'],
                  'source_name' => $key['source_name'],
                  'destination_id' => $key['destination_id'],
                  'destination_name' => $key['destination_name'],
                  'vehicle_id' => $key['vehicle_id'],
                  'vehicle_number' => $key['vehicle_number'],
                  'approved_fuel' => $approved_fuel,
                  'alloted_fuel' => $diesel_receipt_fuel_sum,
                  'excess_fuel' => $diff1,
                  'approved_advance' => $approved_advance,
                  'alloted_advance' => $alloted_advance,
                  'excess_advance' => $diff2,
            ];

        }

          
        return response()->json(['data' => $result_data, 'draw' => $request->draw]);
    }

    public function fuel_usage_agg_report(Request $request)
{
    $report_data = $request->report_data;

    // Prepare the initial query with necessary relationships and aggregation
    $query = BookGr::with([
        'source:id,station_name',
        'destination:id,station_name',
        'vehicle:id,vehicle_number',
        'diesel_receipt:id,booking_id,qty,amount'
    ])
    ->where('fin_year', $request->current_year)
    ->withSum('diesel_receipt', 'qty')  // Summing the fuel quantity
    ->withSum('diesel_receipt', 'amount'); // Summing the fuel amount

    // Apply filters based on the request data
    if (isset($report_data['from_date']) && isset($report_data['to_date'])) {
        $query->whereBetween('gr_date', [$report_data['from_date'], $report_data['to_date']]);
    }

    if (isset($report_data['vehicle'])) {
        $query->where('vehicle_id', $report_data['vehicle']['id']);
    }

    if (isset($report_data['source'])) {
        $query->where('source_id', $report_data['source']['id']);
    }

    if (isset($report_data['destination'])) {
        $query->where('destination_id', $report_data['destination']['id']);
    }

    // Execute the query and get the results
    $data = $query->get();

    // Initialize an array to hold the aggregated data grouped by each vehicle
    $vehicle_data = [];

    // Iterate through each record to calculate sums grouped by vehicle
    foreach ($data as $key) {
        $vehicle_id = $key['vehicle_id'];
        $vehicle_number = $key->vehicle->vehicle_number ?? '';

        // Fetch approved fuel and advance details from FuelAllocationMaster
        $fuel_allocation = FuelAllocationMaster::where('source_id', $key['source_id'])
            ->where('destination_id', $key['destination_id'])
            ->first();

        // Calculate approved fuel and advance values
        $approved_fuel = $fuel_allocation->fuel_qty ?? 0;
        $diesel_receipt_fuel_sum = $key['diesel_receipt_sum_qty'] ?? 0;
        $diff1 = $diesel_receipt_fuel_sum - $approved_fuel;

        $approved_advance = $fuel_allocation->driver_advance ?? 0;

        $driver_account_id = $this->get_acc_id($key->driver_name);
            $v_advance = 0;
            if($driver_account_id){
                $v_advance = VoucherDesc::where('against', 'GR_COST_CENTER') 
                ->where('against_id', $key->id) 
                ->whereHas('voucher_acc_details', function ($query) use ($driver_account_id) {
                    $query->where('account_id', $driver_account_id);
                })
                ->sum('received_amt'); 
            }   
            $alloted_advance = $key['driver_payment_amt'] + $key['diesel_receipt_sum_cash_amount'] + $v_advance;

        $diff2 = $alloted_advance - $approved_advance;

        // Initialize vehicle data if not already set
        if (!isset($vehicle_data[$vehicle_id])) {
            $vehicle_data[$vehicle_id] = [
                'vehicle_id' => $vehicle_id,
                'vehicle_number' => $vehicle_number,
                'approved_fuel' => 0,
                'alloted_fuel' => 0,
                'excess_fuel' => 0,
                'approved_advance' => 0,
                'alloted_advance' => 0,
                'excess_advance' => 0,
            ];
        }

        // Aggregate sums per vehicle
        $vehicle_data[$vehicle_id]['approved_fuel'] += $approved_fuel;
        $vehicle_data[$vehicle_id]['alloted_fuel'] += $diesel_receipt_fuel_sum;
        $vehicle_data[$vehicle_id]['excess_fuel'] += $diff1;
        $vehicle_data[$vehicle_id]['approved_advance'] += $approved_advance;
        $vehicle_data[$vehicle_id]['alloted_advance'] += $alloted_advance;
        $vehicle_data[$vehicle_id]['excess_advance'] += $diff2;
    }

    // Convert the associative array to an indexed array for the response
    $result_data = array_values($vehicle_data);

    // Return the aggregated data per vehicle as JSON response
    return response()->json(['data' => $result_data, 'draw' => $request->draw]);
}

    

    protected function get_data_for_obj($pk, $current_year){
        $fuel = FuelAllocationMaster::find($pk);
        return  ['fuel_allocation_masters' => $fuel];
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'FuelAllocationMaster');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = FuelAllocationMaster::select('id', 'fuel_allocation_no', 'source_name', 'source_id', 'destination_name', 'destination_id', 'fuel_type', 'fuel_qty','driver_advance')->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('fuel_allocation_no', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }
    public function get_fuel_allocation_data(Request $request)
    {
        $source_id = $request->source_id;
        $destination_id = $request->destination_id;

        $fuel_allocation = FuelAllocationMaster::select('fuel_qty', 'driver_advance')
            ->where('source_id', $source_id)
            ->where('destination_id', $destination_id)
            ->first();

        if ($fuel_allocation) {
            $fuel_qty = $fuel_allocation->fuel_qty;
            $driver_advance = $fuel_allocation->driver_advance;
        } else {
            $fuel_qty = null;
            $driver_advance = null;
        }

        return response()->json(['fuel_qty' => $fuel_qty, 'driver_advance' => $driver_advance]);
    }
        
}
