<?php

namespace App\Http\Controllers\Pod;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use Illuminate\Http\Request;
use App\Http\Traits\EditLogTrait;
use App\Http\Traits\PermissionsTrait;
use App\Models\Booking\BookGr;
use App\Models\Booking\EwayGr;
use App\Models\Booking\PackagePosition;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\Master\IssueBooks;
use App\Models\Master\Party;
use App\Models\Pod\PodDesc;
use App\Models\Pod\Pod;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;

class PodController extends Controller
{
    use PermissionsTrait;
    use EditLogTrait;
    use CommonTrait;

    public function index(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Pod') || $this->is_read_permitted($request->user->id, 'MobilePod');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        //FETCH AND SEND ALL
        $pods = Pod::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'received_in_account:id,account_name',
            'received_in_account2:id,account_name',
            'received_in_account3:id,account_name',
            'deduction_account:id,account_name',

            'vehicle:id,vehicle_number,vehicle_type',
            'pod_desc:id,gr_id,pod_id,gr_number',
            'pod_desc.bookGr:id,gr_number,gr_date,pay_mode,acc_party_id,vehicle_id,vehicle_number,delivery_pt,source_id,destination_id,consignor_id,consignee_id,consignor_name,consignee_name,destination_name,total_pkg,total_material_weight,total_km,total_chg_qty,total_desc_value,total_invoice_value,freight,gr_chg,disc_percent,disc_amount,green_tax,kanta,detention,unload_rate,unload_amt,toll_amt,diversion_rate,diversion_amt,grand_total,payment_dynamic_fields',
            'pod_desc.bookGr.acc_party:id,account_name',
            'pod_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',
            'pod_desc.bookGr.consignor:id,party_name',
            'pod_desc.bookGr.consignee:id,party_name',
            'pod_desc.bookGr.source:id,station_name',
            'pod_desc.bookGr.destination:id,station_name',
            'pod_desc.bookGr.challan_desc:id,challan_id,gr_id,gr_number',
            'pod_desc.bookGr.challan_desc.challan:id,challan_number',
            'lorry_hire_from:id,account_name,account_type',       
        ])->where('fin_year', $request->current_year)->get();

        $pods->pod_desc->transform(function ($desc) {
            return [
                'pod_id'=> $desc->pod_id,
                'pod_desc_id' => $desc->id,
                'gr_id' => $desc->gr_id,
                'gr_number' => ['id'=> $desc->gr_id, 'gr_number'=>$desc->gr_number],
                'gr_date' => $desc->bookGr->gr_date,
                'branch' => $desc->bookGr->branch,
                'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name] ,
                'destination' => $desc->bookGrdestination_id ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name] ,
                'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name] ,
                'total_pkg' => $desc->bookGr->total_pkg,
                'total_material_weight' => $desc->bookGr->total_material_weight,
                'total_km' => $desc->bookGr->total_km,
                'total_chg_qty' => $desc->bookGr->total_chg_qty,
                'total_desc_value' => $desc->bookGr->total_desc_value,
                'total_invoice_value' => $desc->bookGr->total_invoice_value,
                'freight' => $desc->bookGr->freight,
                'gr_chg' => $desc->bookGr->gr_chg,
                'grand_total' => $desc->bookGr->grand_total,
                'disc_percent' => $desc->bookGr->disc_percent,
                'disc_amount' => $desc->bookGr->disc_amount,
                'green_tax' => $desc->bookGr->green_tax,
                'kanta' => $desc->bookGr->kanta,
                'detention' => $desc->bookGr->detention,
                'unload_rate' => $desc->bookGr->unload_rate,
                'unload_amt' => $desc->bookGr->unload_amt,
                'toll_amt' => $desc->bookGr->toll_amt,
                'diversion_rate' => $desc->bookGr->diversion_rate,
                'diversion_amt' => $desc->bookGr->diversion_amt,

                'vehicle' => $desc->bookGr->vehicle,
                'acc_party' => $desc->bookGr->acc_party,
                'received_in_account' => $desc->received_in_account,
                'pay_mode'=>$desc->bookGr->pay_mode,
                'payment_dynamic_fields'=>$desc->bookGr->payment_dynamic_fields,
                'challan'=> count($desc->bookGr->challan_desc)?$desc->bookGr->challan_desc[0]->challan:null
            ];
        });

        return response()->json($pods);
    }

    public function show(Request $request, $pk){
        //FETCH AND SEND ALL
        $pod = Pod::with([
            'branch:id,branch_name,branch_code',
            'lorry_hire_from:id,account_name,account_type', 
            'acc_party:id,account_name',
            'received_in_account:id,account_name',
            'received_in_account2:id,account_name',
            'received_in_account3:id,account_name',
            'deduction_account:id,account_name',
            'vehicle:id,vehicle_number,vehicle_type',
            'pod_desc:id,gr_id,pod_id,gr_number',
            'pod_desc.bookGr:id,gr_number,gr_date,pay_mode,po_no,shipment_number,acc_party_id,vehicle_id,vehicle_number,delivery_pt,source_id,destination_id,consignor_id,consignee_id,consignor_name,consignee_name,destination_name,total_pkg,total_material_weight,total_km,total_chg_qty,total_desc_value,total_invoice_value,freight,gr_chg,disc_percent,disc_amount,green_tax,unload_rate,unload_amt,toll_amt,diversion_rate,diversion_amt,kanta,detention,grand_total,payment_dynamic_fields',
            'pod_desc.bookGr.acc_party:id,account_name',
            'pod_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',
            'pod_desc.bookGr.consignor:id,party_name',
            'pod_desc.bookGr.consignee:id,party_name',
            'pod_desc.bookGr.source:id,station_name',
            'pod_desc.bookGr.destination:id,station_name',
            'pod_desc.bookGr.challan_desc:id,challan_id,gr_id,gr_number',
            'pod_desc.bookGr.challan_desc.challan:id,challan_number',       
        ])->find($pk);

        $pod->pod_desc->transform(function ($desc) {
            return [
                'pod_id'=> $desc->pod_id,
                'pod_desc_id' => $desc->id,
                'gr_id' => $desc->gr_id,
                'gr_number' => ['id'=> $desc->gr_id, 'gr_number'=>$desc->gr_number],
                'gr_date' => $desc->bookGr->gr_date,
                'branch' => $desc->bookGr->branch,
                'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name] ,
                'destination' => $desc->bookGrdestination_id ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name] ,
                'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name] ,
                'total_pkg' => $desc->bookGr->total_pkg,
                'total_material_weight' => $desc->bookGr->total_material_weight,
                'po_no' => $desc->bookGr->po_no,
                'shipment_number' => $desc->bookGr->shipment_number, 
                'total_km' => $desc->bookGr->total_km,
                'total_chg_qty' => $desc->bookGr->total_chg_qty,
                'total_desc_value' => $desc->bookGr->total_desc_value,
                'total_invoice_value' => $desc->bookGr->total_invoice_value,
                'freight' => $desc->bookGr->freight,
                'gr_chg'=>$desc->bookGr->gr_chg,
                'grand_total' => $desc->bookGr->grand_total,
                'disc_percent' => $desc->bookGr->disc_percent,
                'disc_amount' => $desc->bookGr->disc_amount,
                'green_tax' => $desc->bookGr->green_tax,
                'kanta' => $desc->bookGr->kanta,
                'detention' => $desc->bookGr->detention,
                'unload_rate' => $desc->bookGr->unload_rate,
                'unload_amt' => $desc->bookGr->unload_amt,
                'toll_amt' => $desc->bookGr->toll_amt,
                'diversion_rate' => $desc->bookGr->diversion_rate,
                'diversion_amt' => $desc->bookGr->diversion_amt,
                

                'vehicle' => $desc->bookGr->vehicle,
                'acc_party' => $desc->bookGr->acc_party,
                'received_in_account' => $desc->received_in_account,
                'pay_mode'=>$desc->bookGr->pay_mode,
                'payment_dynamic_fields'=>$desc->bookGr->payment_dynamic_fields,
                'challan'=> count($desc->bookGr->challan_desc)?$desc->bookGr->challan_desc[0]->challan:null
            ];
        });

        return response()->json(['pod'=>$pod]);
    }

    public function store(Request $request){

        $test = $this->is_new_permitted($request->user->id, 'Pod') || $this->is_new_permitted($request->user->id, 'MobilePod');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }
        
        $validatedData = $request->validate([
            'branch' => 'required',
            'pod_number' => 'required|unique:pods,pod_number',
            'acc_party' => 'required'
        ], [
            'branch.required' => 'Required',
            'pod_number.required' => 'Required',
            'pod_number.unique' => 'Already Exist !',
            'acc_party.required' => 'Required'
        ]);

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

            $max_number = $this->get_max_number_from_pod_number($request->pod_number);
            
            // SAVING A NEW POD
            $pod = new Pod();
            $pod->branch_id = $request->branch['id'];
            $pod->pod_type = $request->pod_type;
            $pod->prefix = $request->prefix;
            $pod->max_number = $max_number;
            $pod->pod_number = $request->pod_number;

            $pod->delivery_date = $request->delivery_date;
            $pod->delivery_time = $request->delivery_time;

            $pod->pay_mode = $request->pay_mode;
            $pod->bill_applicable = $request->bill_applicable;

            if($request->pay_mode == 'TO PAY'){
                $pod->income_account_id = 6;  // TO PAY
            }elseif($request->pay_mode == 'PAID'){
                $pod->income_account_id = 7;  // PAID
            }

            $pod->acc_party_id = $request->acc_party['id'];
            $pod->receiver_name = $request->receiver_name;
            $pod->receiver_contact = $request->receiver_contact;
            $pod->vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
            $pod->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;

            $pod->lorry_hire_from_id = $request->lorry_hire_from ? $request->lorry_hire_from['id'] : null;

            $pod->pod_pic = $request->pod_pic;
            $pod->pod_pic1 = $request->pod_pic1;
            $pod->remarks = $request->remarks;
            $pod->printer_type = $request->printer_type;
            $pod->soft_copy_received = $request->softcopyreceived ? 1 : 0;
            $pod->hard_copy_received = $request->hardcopyreceived ? 1 : 0;
            
        

            


            $pod->pod_payment_dynamic_fields = $request->pod_payment_dynamic_fields;
            $pod->pod_payment_dynamic_total = $request->pod_payment_dynamic_total;

            $pod->load_detention_days = $request->load_detention_days;
            $pod->load_detention_rate = $request->load_detention_rate;
            $pod->load_detention = $request->load_detention;
            $pod->unload_detention_days = $request->unload_detention_days;
            $pod->unload_detention_rate = $request->unload_detention_rate;
            $pod->unload_detention = $request->unload_detention;
            $pod->net_km = $request->net_km;
            $pod->reporting_date = $request->reporting_date;
            $pod->reporting_time = $request->reporting_time;
            $pod->unload_date = $request->unload_date;
            $pod->unload_time = $request->unload_time;
            $pod->detention_days = $request->detention_days;

            $pod->unloaded_wt = $request->unloaded_wt;
            $pod->shortage_qty = $request->shortage_qty;
            $pod->shortage_rate = $request->shortage_rate;
            $pod->shortage_amt = $request->shortage_amt;

            $pod->shortage_tolerance_percent = $request->shortage_tolerance_percent;
            $pod->shortage_tolerance_qty = $request->shortage_tolerance_qty;

            $pod->material_cost = $request->material_cost;
            $pod->total_lh_deduction = $request->total_lh_deduction;

            $pod->freight = $request->freight;
            $pod->gr_chg = $request->gr_chg;
            $pod->disc_percent = $request->disc_percent;
            $pod->disc_amount = $request->disc_amount;
            $pod->green_tax = $request->green_tax;
            $pod->kanta = $request->kanta;
            $pod->detention = $request->detention;
            $pod->unload_rate = $request->unload_rate;
            $pod->unload_amt = $request->unload_amt;
            $pod->toll_amt = $request->toll_amt;
            $pod->diversion_rate = $request->diversion_rate;
            $pod->diversion_amt = $request->diversion_amt;

            $pod->payment_dynamic_fields = $request->payment_dynamic_fields;
            $pod->total_without_gst = $request->total_without_gst;
            $pod->gst_paid_by = $request->gst_paid_by;
            $pod->igst_percent = $request->igst_percent;
            $pod->cgst_percent = $request->cgst_percent;
            $pod->sgst_percent = $request->sgst_percent;
            $pod->igst_amount = $request->igst_amount;
            $pod->cgst_amount = $request->cgst_amount;
            $pod->sgst_amount = $request->sgst_amount;
            $pod->total_without_rounding = $request->total_without_rounding;
            $pod->rounded_off = $request->rounded_off;
            $pod->grand_total = $request->grand_total;

            $pod->received_in_account_id = $request->received_in_account? $request->received_in_account['id'] : null;
            $pod->received_amt = $request->received_amt;
            $pod->received_date = $request->received_date;
            $pod->received_in_account2_id = $request->received_in_account2? $request->received_in_account2['id'] : null;
            $pod->received_amt2 = $request->received_amt2;
            $pod->received_date2 = $request->received_date2;
            $pod->received_in_account3_id = $request->received_in_account3? $request->received_in_account3['id'] : null;
            $pod->received_amt3 = $request->received_amt3;
            $pod->received_date3 = $request->received_date3;
            $pod->deduction_account_id = $request->deduction_account? $request->deduction_account['id'] : null;
            $pod->deduction_amt = $request->deduction_amt;
            $pod->balance = $request->balance;


            $pod->fin_year = $request->current_year;
           

            $pod->created_by = $request->user->id;

            // Save the model to the database
            $pod->save();

            // SAVING POD DESC
            $pod_desc = $request->pod_desc;
            foreach ($pod_desc as $pd) {
                if( !$pd['gr_number'] )
                continue;
                
                $unld_desc = new PodDesc();
                $unld_desc->pod_id = $pod->id;
                $unld_desc->pod_number = $pod->pod_number;
                $unld_desc->gr_id = $pd['gr_number']['id'];
                $unld_desc->gr_number = $pd['gr_number']['gr_number'];
                $unld_desc->fin_year = $request->current_year;
                $unld_desc->save();

                // Update isDelivered to 'true'
                $pkg_pos = PackagePosition::where('gr_id', $pd['gr_number']['id']);
                $pkg_pos->update(['is_delivered' => true]);

                // Update isDelivered to 'true'
                $eway_grs = EwayGr::where('gr_id', $pd['gr_number']['id']);
                $eway_grs->update(['is_delivered' => true]);
            }
            // POD DESC SAVED

            // RECORD THE TRANSACTION IN ACCOUNTS
            
            $this->record_transaction_in_accounts($request, $pod);
            // TRANSACTION RECORDED IN ACCOUNTS

            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($pod->id);
            $this->createEditLog('created', 'pods', $pod->id, $pod->pod_number,null, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            
        
            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json(['id'=>$pod->id, 'pod_number'=>$pod->pod_number], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function update(Request $request, $pk){   
        $testPod = $this->is_edit_permitted($request->user->id, 'Pod');
        $testMobilePod = $this->is_edit_permitted($request->user->id, 'MobilePod');

        $test_edit = $testPod->edit || $testMobilePod->edit;
        $edit_valid_till_hours = $testPod->edit_valid_till; // Or whatever logic applies to get the valid till hours

        if (!$test_edit) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        
        $pod = Pod::find($pk);

        $bill_check = $this->check_in_bill_desc('bill_id','bill_number', 'pod_id', $pk);
        if($bill_check['status']){
            
            if($request->pod_pic || $request->pod_pic1){
                $pod->pod_pic = $request->pod_pic;
                $pod->pod_pic1 = $request->pod_pic1;
                $pod->save();
                
            }
            
            $status = $bill_check['status'] ? $bill_check : null;
            return response()->json(['error'=>'Document in use', 'data'=>$status],500);
         };


        // if( $edit_valid_till_hours!=null ){
        //     $validity = Carbon::parse($pod->created_at)->addHours($edit_valid_till_hours);
        //     if( !$validity->isFuture() ){
        //         return response()->json(['Permission Timed Out'],500);
        //     }
        // }

        $validatedData = $request->validate([
            'branch' => 'required',
            'pod_number' => 'required',
            'acc_party' => 'required'
        ], [
            'branch.required' => 'Required',
            'pod_number.required' => 'Required',
            'acc_party.required' => 'Required'
        ]);
          

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();

            // FETCHING OLD VALUE FOR LOG
            $oldValue = $this->get_data_for_obj($pk);

            // SAVING A NEW POD
            $pod->delivery_date = $request->delivery_date;
            $pod->delivery_time = $request->delivery_time;

            $pod->pay_mode = $request->pay_mode;
            if($request->pay_mode == 'TO PAY'){
                $pod->income_account_id = 6;  // TO PAY
            }elseif($request->pay_mode == 'PAID'){
                $pod->income_account_id = 7;  // PAID
            }

            $pod->acc_party_id = $request->acc_party['id'];
            $pod->receiver_name = $request->receiver_name;
            $pod->receiver_contact = $request->receiver_contact;
            $pod->vehicle_id = $request->vehicle ? $request->vehicle['id'] : null;
            $pod->vehicle_number = $request->vehicle ? $request->vehicle['vehicle_number'] : null;
            $pod->lorry_hire_from_id = $request->lorry_hire_from ? $request->lorry_hire_from['id'] : null;
            $pod->pod_pic = $request->pod_pic;
            $pod->pod_pic1 = $request->pod_pic1;
            $pod->remarks = $request->remarks;
            $pod->printer_type = $request->printer_type;
            $pod->pod_payment_dynamic_fields = $request->pod_payment_dynamic_fields;
            $pod->pod_payment_dynamic_total = $request->pod_payment_dynamic_total;
            $pod->net_km = $request->net_km;
            $pod->reporting_date = $request->reporting_date;
            $pod->reporting_time = $request->reporting_time;
            $pod->unload_date = $request->unload_date;
            $pod->unload_time = $request->unload_time;
            $pod->detention_days = $request->detention_days;

            $pod->load_detention_days = $request->load_detention_days;
            $pod->load_detention_rate = $request->load_detention_rate;
            $pod->load_detention = $request->load_detention;
            $pod->unload_detention_days = $request->unload_detention_days;
            $pod->unload_detention_rate = $request->unload_detention_rate;
            $pod->unload_detention = $request->unload_detention;
            $pod->soft_copy_received = $request->softcopyreceived ? 1 : 0;
            $pod->hard_copy_received = $request->hardcopyreceived ? 1 : 0;
            $pod->unloaded_wt = $request->unloaded_wt;
            $pod->shortage_qty = $request->shortage_qty;
            $pod->shortage_rate = $request->shortage_rate;
            $pod->shortage_amt = $request->shortage_amt;
            $pod->bill_applicable = $request->bill_applicable;

            $pod->shortage_tolerance_percent = $request->shortage_tolerance_percent;
            $pod->shortage_tolerance_qty = $request->shortage_tolerance_qty;

            $pod->material_cost = $request->material_cost;
            $pod->total_lh_deduction = $request->total_lh_deduction;

            $pod->freight = $request->freight;
            $pod->gr_chg = $request->gr_chg;
            $pod->disc_percent = $request->disc_percent;
            $pod->disc_amount = $request->disc_amount;
            $pod->green_tax = $request->green_tax;
            $pod->kanta = $request->kanta;
            $pod->detention = $request->detention;

            $pod->unload_rate = $request->unload_rate;
            $pod->unload_amt = $request->unload_amt;
            $pod->toll_amt = $request->toll_amt;
            $pod->diversion_rate = $request->diversion_rate;
            $pod->diversion_amt = $request->diversion_amt;

            $pod->payment_dynamic_fields = $request->payment_dynamic_fields;
            $pod->total_without_gst = $request->total_without_gst;
            $pod->gst_paid_by = $request->gst_paid_by;
            $pod->igst_percent = $request->igst_percent;
            $pod->cgst_percent = $request->cgst_percent;
            $pod->sgst_percent = $request->sgst_percent;
            $pod->igst_amount = $request->igst_amount;
            $pod->cgst_amount = $request->cgst_amount;
            $pod->sgst_amount = $request->sgst_amount;
            $pod->total_without_rounding = $request->total_without_rounding;
            $pod->rounded_off = $request->rounded_off;
            $pod->grand_total = $request->grand_total;

            $pod->received_in_account_id = $request->received_in_account? $request->received_in_account['id'] : null;
            $pod->received_amt = $request->received_amt;
            $pod->received_date = $request->received_date;
            $pod->received_in_account2_id = $request->received_in_account2? $request->received_in_account2['id'] : null;
            $pod->received_amt2 = $request->received_amt2;
            $pod->received_date2 = $request->received_date2;
            $pod->received_in_account3_id = $request->received_in_account3? $request->received_in_account3['id'] : null;
            $pod->received_amt3 = $request->received_amt3;
            $pod->received_date3 = $request->received_date3;
            $pod->deduction_account_id = $request->deduction_account? $request->deduction_account['id'] : null;
            $pod->deduction_amt = $request->deduction_amt;
            $pod->balance = $request->balance;

            
            $pod->updated_by = $request->user->id;
            $pod->save();
            // NEW POD SAVED
            
            // SAVING POD DESC
            $newly_added_pod_desc = $request->newly_added_pod_desc;
            foreach ($newly_added_pod_desc as $napd) {
                if( !$napd['gr_number'] )
                continue;

                $unld_desc = new PodDesc();
                $unld_desc->pod_id = $pod->id;
                $unld_desc->pod_number = $pod->pod_number;
                $unld_desc->gr_id = $napd['gr_number']['id'];
                $unld_desc->gr_number = $napd['gr_number']['gr_number'];
                $unld_desc->fin_year = $request->current_year;
                $unld_desc->save();

                $pkg_pos = PackagePosition::where('gr_id', $napd['gr_number']['id']);
                $pkg_pos->update(['is_delivered' => true]);

                $eway_grs = EwayGr::where('gr_id', $napd['gr_number']['id']);
                $eway_grs->update(['is_delivered' => true]);

            }
            // POD DESC SAVED


            // DELETING EXISTING POD DESC
            $deleted_pod_desc = $request->deleted_pod_desc;
            if( !empty($deleted_pod_desc) ){
                foreach ($deleted_pod_desc as $dpd) {
                    $unld_desc = PodDesc::find($dpd);
                    $unld_desc->delete();

                    $gr_number = $unld_desc->gr_number;
                    $t_data2['t_against'] = 'GR';
                    $t_data2['t_against_no'] = $gr_number;
                    $t_data2['fin_year'] = $request->current_year;
                    $this->set_transaction_status($t_data2, true);

                    $pkg_pos = PackagePosition::where('gr_id', $unld_desc->gr_id);
                    $pkg_pos->update(['is_delivered' => false]);

                    $eway_grs = EwayGr::where('gr_id', $unld_desc->gr_id);
                    $eway_grs->update(['is_delivered' => false]);
                    
                }      
            }
            //DELETED EXISTING POD DESC

            // RECORD THE TRANSACTION IN ACCOUNTS
            $this->record_transaction_in_accounts($request, $pod);
            // TRANSACTION RECORDED IN ACCOUNTS


            // SAVING THE EDIT LOG
            $newVal = $this->get_data_for_obj($pod->id);
            $this->createEditLog('updated', 'pods', $pod->id, $pod->pod_number,$oldValue, $newVal, $request->user->id, $request->current_year); // CREATE EDIT LOG
            

            // TRANSACTIONS COMMITTED
            DB::commit();

            
            return response()->json("Success", 200); // RETURNED RESPONSE

        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function destroy(Request $request, $pk){
        $testPod = $this->is_delete_permitted($request->user->id, 'Pod');
        $testMobilePod = $this->is_delete_permitted($request->user->id, 'MobilePod');

        $test_delete = $testPod->delete || $testMobilePod->delete;
        $delete_valid_till_hours = $testPod->delete_valid_till; // Or whatever logic applies to get the valid till hours

        if (!$test_delete) {
            return response()->json(['Permission Denied For The Action'], 401);
        }

        // FETCHING OLD VALUE FOR LOG
        $oldValue = $this->get_data_for_obj($pk);
        
        $pod = Pod::find($pk);

        $bill_check = $this->check_in_bill_desc('bill_id','bill_number', 'pod_id', $pk);
        if($bill_check['status']){
            $status = $bill_check['status'] ? $bill_check : null;
            return response()->json(['error'=>'Document in use', 'data'=>$status],500);
         };


        // if( $delete_valid_till_hours!=null ){
        //     $validity = Carbon::parse($pod->created_at)->addHours($delete_valid_till_hours);
        //     if( !$validity->isFuture() ){
        //         return response()->json(['Permission Timed Out'],500);
        //     }
        // }

        try {
        // TRANSACTION BEGINS
        DB::beginTransaction();

        $pod_desc_ids = PodDesc::select('id')->where('pod_id', $pk)->pluck('id')->toArray();
        
        // DELETING CHALLAN DESCRIPTIONS
        foreach ($pod_desc_ids as $pdi) {
            // MAKE GR TRANSACTION INACTIVE
            $unld_desc = PodDesc::find($pdi);

            $gr_number = $unld_desc->gr_number;
            $t_data2['t_against'] = 'GR';
            $t_data2['t_against_no'] = $gr_number;
            $t_data2['fin_year'] = $request->current_year;
            $this->set_transaction_status($t_data2, true);

            $pkg_pos = PackagePosition::where('gr_id', $unld_desc->gr_id);
            $pkg_pos->update(['is_delivered' => false]);

            $eway_grs = EwayGr::where('gr_id', $unld_desc->gr_id);
            $eway_grs->update(['is_delivered' => false]);

            $unld_desc->delete();
        }

        
        //FIND AND DELETE THE BRANCH
        $pod->delete();

        // DELETE THE TRANSACTION VOUCHER
        $t_data = [];
        $t_data['t_against'] = 'POD';
        $t_data['t_against_id'] = $pod->id;
        $t_data['t_against_no'] = $pod->pod_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);


        $this->createEditLog('deleted', 'pods', $pod->id, $pod->pod_number, $oldValue, null, $request->user->id, $request->current_year); // CREATE EDIT LOG
        
        // TRANSACTIONS COMMITTED
        DB::commit();
          
        } catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }

        return response()->json("Success", 200); // RETURNED RESPONSE
    }

    public function autocomplete(Request $request){
        // Initialize the query builder
        $query = Pod::select('id', 'pod_number','vehicle_number')->where('fin_year', $request->current_year);

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        if( $request->user->party_id ){
            $prt = Party::select('account_id')->find($request->user->party_id);
            
            $query->where(function($query) use ($prt){
                $query->where('acc_party_id', $prt->account_id)->where('acc_party_id','!=', null);
            });
        }

        if ($request->has('filterby')) {
            $orWhereFilters = isset($request->filterby['orWhere']) ? $request->filterby['orWhere'] : [];
            $whereFilters = isset($request->filterby['where']) ? $request->filterby['where'] : [];

        // Loop through the whereFilters array
        foreach ($whereFilters as $whereFilter) {
            // Apply the filter to the query
            $query->where($whereFilter['key'], $whereFilter['value']);
        }

        // Loop through the orWhereFilters array
        foreach ($orWhereFilters as $orWhereFilter) {
            // Apply the filter to the query
            $query->orWhere($orWhereFilter['key'], $orWhereFilter['value']);
        }
        
        }


        // Check if searchVariable is provided and apply the filter
        if ($request->has('searchVariable')) {
            $searchVariable = $request->searchVariable;
            $query->where('pod_number', 'like', '%'. $searchVariable . '%')
            ->orWhere('vehicle_number', 'like', '%'. $searchVariable . '%');
        }

        // Limit the number of results
        $query->take($request->limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $pod = $query->get();
        return response(['pods' => $pod, 'draw' => $request->draw]);
    }

    public function datatable(Request $request){
        $test = $this->is_read_permitted($request->user->id, 'Pod');
        if(!$test){
            return response()->json(['Permission Denied For The Action'],401);
        }

        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $query = Pod::select('id', 'pod_number', 'vehicle_id','vehicle_number', 'delivery_date', 'delivery_time', 'pay_mode' ,'acc_party_id', 'branch_id')
                ->with(['branch:id,branch_name,branch_code', 'vehicle:id,vehicle_number', 'acc_party:id,party_name']) 
                ->where('fin_year', $request->current_year)
                ->orderBy($column, $dir);

        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('pod_number', 'like', $searchValue . '%');
            });
        }

        $data = $query->paginate($length);

        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    public function check_unique(Request $request){
        $key = $request->key;
        $value = $request->value;

        if(!$value){
            return response()->json(['exists' => false]);
        }

        $exists = Pod::where($key, $value)->exists();
        if($exists){
            return response()->json(['errors' => [ $key  => ['Already Exists !']]],422);
        }

        return response()->json(['exists' => false]);
    }

    public function get_recent(Request $request) {
        $podQuery = Pod::select('id', 'pod_number')->where('fin_year', $request->current_year);
    
        if (!empty($request->user_branch_ids)) {
            $podQuery->whereIn('branch_id', $request->user_branch_ids);
        }

        if( $request->user->party_id ){
            $prt = Party::select('account_id')->find($request->user->party_id);
            
            $podQuery->where(function($query) use ($prt){
                $query->where('acc_party_id', $prt->account_id)->where('acc_party_id','!=', null);
            });
        }

    
        $pods = $podQuery->orderBy('id', 'desc')->take(20)->get();
    
        return response()->json(['pods' => $pods]);
    }
    


    public function get_next_pod_number(Request $request){
        $type=$request->type;
        $prefix=$request->prefix;
        $branch_id = $request->branch_id;
        $branch_code = $request->branch_code;

        if($type=='SYSTEM'){
            $max = Pod::where('branch_id',$branch_id)->where('pod_type', $type)->max('max_number');
            $pod_number = $branch_code.'-'.sprintf('%06d', $max+1);
            return response()->json(['pod_number' => strtoupper($pod_number), 'max_number' => $max+1]);
        }
        else if($type=='PRINTED'){
            // GET THE NEXT NUMBER THAT HAS BEEN ISSUE TO THE BRANCH WITH THE CERTAIN PREFIX
            $max = Pod::where('prefix',$prefix)->where('pod_type', $type)->where('branch_id', $branch_id)->max('max_number');
            
            // IF MAX IS PRESENT, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            if( $max ){
                $check = IssueBooks::select('prefix','party_id')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','POD')->where('issue_from', '<=', $max+1)->where('issue_to', '>=', $max+1)->first();
                if($max && $check){
                    $pod_number = $prefix ? $prefix.'-'.($max+1) : ($max+1);
                    return response()->json(['pod_number' => strtoupper($pod_number), 'max_number' => $max+1, 'party' => $check->party]);
                }
            }

            // IF MAX IS NOT PRESENT OR NOT VALID, THEN CHECK IN ISSUE BOOKS IF IT IS VALID
            $check = IssueBooks::select('prefix','party_id','issue_from')->with(['party:id,party_name'])->where('prefix',$prefix)->where('branch_id', $branch_id)->where('book_type','POD')->where('issue_from', '>=', $max+1)->first();
            if( $check ){
                $pod_number = $prefix ? $prefix.'-'.$check->issue_from : $check->issue_from;
                return response()->json(['pod_number' => strtoupper($pod_number), 'max_number' => $check->issue_from, 'party' => $check->party]);
            }
            else{
                return response()->json(['errors' => ['pod_number' => ['No Valid POD Number']]],422);
            }
        }
    }

    protected function get_data_for_obj($pk){
       //FETCH AND SEND ALL
       $pod = Pod::with([
                'branch:id,branch_name,branch_code',
                'lorry_hire_from:id,account_name,account_type', 
                'acc_party:id,account_name',
               'received_in_account:id,account_name',
            'received_in_account2:id,account_name',
            'received_in_account3:id,account_name',
            'deduction_account:id,account_name',
                
                'vehicle:id,vehicle_number,vehicle_type',
                'pod_desc:id,gr_id,pod_id,gr_number',
                'pod_desc.bookGr:id,gr_number,gr_date,pay_mode,acc_party_id,vehicle_id,vehicle_number,delivery_pt,source_id,destination_id,consignor_id,consignee_id,consignor_name,consignee_name,destination_name,total_pkg,total_material_weight,total_chg_qty,total_desc_value,total_invoice_value,freight,gr_chg,disc_percent,disc_amount,green_tax,kanta,detention,unload_rate,unload_amt,toll_amt,diversion_rate,diversion_amt,grand_total,payment_dynamic_fields',
                'pod_desc.bookGr.acc_party:id,account_name',
                'pod_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',
                'pod_desc.bookGr.consignor:id,party_name',
                'pod_desc.bookGr.consignee:id,party_name',
                'pod_desc.bookGr.source:id,station_name',
                'pod_desc.bookGr.destination:id,station_name'       
            ])->find($pk);

        $pod->pod_desc->transform(function ($desc) {
            return [
                'pod_id'=> $desc->pod_id,
                'pod_desc_id' => $desc->id,
                'gr_id' => $desc->gr_id,
                'gr_number' => ['id'=> $desc->gr_id, 'gr_number'=>$desc->gr_number],
                'gr_date' => $desc->bookGr->gr_date,
                'branch' => $desc->bookGr->branch,
                'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name] ,
                'destination' => $desc->bookGrdestination_id ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name] ,
                'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name] ,
                'total_pkg' => $desc->bookGr->total_pkg,
                'total_material_weight' => $desc->bookGr->total_material_weight,
                'total_km' => $desc->bookGr->total_km,
                'total_chg_qty' => $desc->bookGr->total_chg_qty,
                'total_desc_value' => $desc->bookGr->total_desc_value,
                'total_invoice_value' => $desc->bookGr->total_invoice_value,
                'freight' => $desc->bookGr->freight,
                'gr_chg' => $desc->bookGr->gr_chg,
                'disc_percent' => $desc->bookGr->disc_percent,
                'disc_amount' => $desc->bookGr->disc_amount,
                'green_tax' => $desc->bookGr->green_tax,
                'kanta' => $desc->bookGr->kanta,
                'detention' => $desc->bookGr->detention,
                'unload_rate' => $desc->bookGr->unload_rate,
                'unload_amt' => $desc->bookGr->unload_amt,
                'toll_amt' => $desc->bookGr->toll_amt,
                'diversion_rate' => $desc->bookGr->diversion_rate,
                'diversion_amt' => $desc->bookGr->diversion_amt,
                

                'grand_total' => $desc->bookGr->grand_total,
                'vehicle' => $desc->bookGr->vehicle,
                'received_in_account' => $desc->received_in_account,
                'acc_party' => $desc->bookGr->acc_party,
                'pay_mode'=>$desc->bookGr->pay_mode,
                'payment_dynamic_fields'=>$desc->bookGr->payment_dynamic_fields,
            ];
        });

        return ['pod'=>$pod];
    }

    public function get_pod_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                                                                                                 
        $query = Pod::with(['branch:id,branch_name,branch_code','acc_party:id,account_name','pod_desc','pod_desc.bookGr.source:id,station_name','pod_desc.bookGr.destination:id,station_name','lorry_hire_from:id,account_name,account_type',])
        ->where('fin_year', $request->current_year);

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }

        if(isset($report_data['pod_num_from']) && isset($report_data['pod_num_to'])){
            $query->where('pod_number','>=',$report_data['pod_num_from']['pod_number'])->where('pod_number','<=',$report_data['pod_num_to']['pod_number']);
        }


        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('delivery_date','>=',$report_data['from_date'])->where('delivery_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['lorry_hire_from']) ){
            $query->where('lorry_hire_from', $report_data['lorry_hire_from']['id']);
        }

        if(isset($report_data['pod_attachment']) ){
            if($report_data['pod_attachment']=='Done')
            $query->whereNotNull('pod_pic');
            if($report_data['pod_attachment']=='Pending')
            $query->whereNull('pod_pic');
        }

        if (isset($report_data['softcopy']) && $report_data['softcopy'] !== 'all') {
            $query->where('soft_copy_received', $report_data['softcopy']);
        }
        
        if (isset($report_data['hardcopy']) && $report_data['hardcopy'] !== 'all') {
            $query->where('hard_copy_received', $report_data['hardcopy']);
        }


        if ( !empty($request->user_branch_ids) ) {
            $query->whereIn('branch_id', $request->user_branch_ids );
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('pod_number', 'like','%'. $searchValue . '%');
            });
        }

        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }


    public function get_pending_pod_report(Request $request){
        $length = $request->length;
        $column = $request->column;
        $dir = $request->dir;
        $searchValue = $request->search;

        $report_data = $request->report_data;
                                   
        if($report_data['against']=='GR'){
        $query = BookGr::select('id','gr_number as against_number','branch_id','source_id','destination_id','consignor_id','consignee_id','acc_party_id','grand_total as freight_to_show')
        ->with(['branch:id,branch_name,branch_code','source:id,station_name','destination:id,station_name','acc_party:id,account_name','challan_desc:id,challan_id,challan_number,gr_id,gr_number','challan_desc.challan:id,challan_number,lorry_hire_from','challan_desc.challan.lorry_hire_from:id,account_name','consignor:id,party_name','consignee:id,party_name'])
        ->where('fin_year', $request->current_year)
        ->whereDoesntHave('pod_desc');

        
        if(isset($report_data['branch']) ){
            $query->where('branch_id', $report_data['branch']['id']);
        }

        if(isset($report_data['from_date']) && isset($report_data['to_date'])){
            $query->where('gr_date','>=',$report_data['from_date'])->where('gr_date','<=',$report_data['to_date']);
        }

        if(isset($report_data['acc_party']) ){
            $query->where('acc_party_id', $report_data['acc_party']['id']);
        }

       // Filtering `lorry_hire_from`
        if (isset($report_data['lorry_hire_from'])) {
            $query->whereHas('challan_desc.challan', function ($q) use ($report_data) {
                $q->where('lorry_hire_from', $report_data['lorry_hire_from']['id']);
            });
        }


        if ($searchValue) {
            $query->where(function ($query) use ($searchValue) {
                $query->where('gr_number', 'like','%'. $searchValue . '%');
            });
        }

       }

        if($report_data['against']=='LOADING SLIP'){
            $query = LoadingSlip::select('id','ls_number as against_number','branch_id','source_id','destination_id','acc_party_id','ls_date','grand_total as freight_to_show')
                    ->with(['branch:id,branch_name,branch_code','source:id,station_name','destination:id,station_name','acc_party:id,account_name'])
                    ->where('fin_year', $request->current_year)
                    ->whereNull('pod_pic');
        
            if(isset($report_data['branch']) ){
                $query->where('branch_id', $report_data['branch']['id']);
            }

            if(isset($report_data['from_date']) && isset($report_data['to_date'])){
                $query->where('ls_date','>=',$report_data['from_date'])->where('ls_date','<=',$report_data['to_date']);
            }

            
            if(isset($report_data['acc_party']) ){
                $query->where('acc_party_id', $report_data['acc_party']['id']);
            }


            if ($searchValue) {
                $query->where(function ($query) use ($searchValue) {
                    $query->where('ls_number', 'like','%'. $searchValue . '%');
                });
            }
        }


        if ($length) {
            $data = $query->orderBy($column, $dir)->paginate($length);
        } else {
            $data = $query->orderBy($column, $dir)->paginate( $query->count() );
        }
          
        return response()->json(['data' => $data, 'draw' => $request->draw]);
    }

    protected function get_max_number_from_pod_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }

    protected function record_transaction_in_accounts($request, $pod){
        $t_data = [];
        $t_data['t_against'] = 'POD';
        $t_data['t_against_id'] = $pod->id;
        $t_data['t_against_no'] = $pod->pod_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = $request->branch;
        $t_data['voucher_date'] = $request->delivery_date;
        $t_data['voucher_time'] = $request->delivery_time;
        $t_data['t_against_narration'] = $pod->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];
        
        $t_data['cancellable'] = 1;
        if($request->pay_mode == 'TO PAY' || $request->pay_mode == 'PAID'){
            if($pod->income_account_id && $request->total_without_rounding>0){
                $AFTER_GST = $request->green_tax + $request->kanta + $request->detention; 

                $t_data['voucher_accounting_details']=[];
                // DEBIT PARTY ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$pod->acc_party_id, 'nature'=>'dr' , 'dr_amount'=>$request->total_without_rounding, 'cr_amount'=>0 ];
                

                // CREDIT SIDE
                if($request->cgst_amount > 0){
                    // CREDIT CGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>12, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->cgst_amount ];
                }
                if($request->sgst_amount > 0){
                    // CREDIT SGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>13, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->sgst_amount ];
                }
                if($request->igst_amount > 0){
                    // CREDIT IGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>14, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->igst_amount ];
                }

                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$pod->income_account_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->total_without_gst + $AFTER_GST ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }

                // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 


                $this->create_transaction_voucher( $t_data );
            }
            $pod_desc_ids = PodDesc::select('id')->where('pod_id', $pod->id)->pluck('id')->toArray();
        
            // DELETING CHALLAN DESCRIPTIONS
            foreach ($pod_desc_ids as $pdi) {
                // MAKE GR TRANSACTION INACTIVE
                $gr_number = PodDesc::select('gr_number')->where('pod_id',$pdi)->find($pdi)['gr_number'];
                $t_data2['t_against'] = 'GR';
                $t_data2['t_against_no'] = $gr_number;
                $t_data2['fin_year'] = $request->current_year;
                $this->set_transaction_status($t_data2, false);
            }
        }
        
        $t_data['cancellable'] = 0;
        if($pod->received_in_account_id && $pod->received_amt>0){
            $t_data['voucher_date'] = $pod->received_date ?? $request->delivery_date;
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$pod->received_in_account_id, 'nature'=>'dr' , 'dr_amount'=>$request->received_amt, 'cr_amount'=>0 ];
            

            // CREDIT SIDE
            $t_data['voucher_accounting_details'][] = ['account_id'=>$pod->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->received_amt ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }

        $t_data['cancellable'] = 0;
        
        if($pod->received_in_account2_id && $pod->received_amt2>0){
            $t_data['voucher_date'] = $pod->received_date2 ?? $request->delivery_date;
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$pod->received_in_account2_id, 'nature'=>'dr' , 'dr_amount'=>$pod->received_amt2, 'cr_amount'=>0 ];
            

            // CREDIT SIDE
            $t_data['voucher_accounting_details'][] = ['account_id'=>$pod->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$pod->received_amt2 ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }

            


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }

        $t_data['cancellable'] = 0;
        if($pod->received_in_account3_id && $pod->received_amt3>0){
            $t_data['voucher_date'] = $pod->received_date3 ?? $request->delivery_date;
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$pod->received_in_account3_id, 'nature'=>'dr' , 'dr_amount'=>$pod->received_amt3, 'cr_amount'=>0 ];
            

            // CREDIT SIDE
            $t_data['voucher_accounting_details'][] = ['account_id'=>$pod->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$pod->received_amt3 ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }

        $t_data['cancellable'] = 0;
        if($pod->deduction_account_id && $pod->deduction_amt>0){
            $t_data['voucher_date'] =  $request->delivery_date;
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$pod->deduction_account_id, 'nature'=>'dr' , 'dr_amount'=>$request->deduction_amt, 'cr_amount'=>0 ];
            

            // CREDIT SIDE
            $t_data['voucher_accounting_details'][] = ['account_id'=>$pod->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->deduction_amt ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 


            $this->create_transaction_voucher( $t_data );
        }

        // $t_data['cancellable'] = 0;
        // $shortage_id = $this->get_acc_id('Shortage and Excess');
        // if(!$shortage_id){
        //     dd('Shortage and Excess A/C not Found');
        // }
        
        // if($pod->lorry_hire_from_id && $shortage_id && $request->shortage_amt>0){
            
        //     $t_data['voucher_accounting_details']=[];
        //     // DEBIT PARTY ACCOUNT
        //     $t_data['voucher_accounting_details'][] = ['account_id'=>$pod->lorry_hire_from_id, 'nature'=>'dr' , 'dr_amount'=>$request->shortage_amt, 'cr_amount'=>0 ];
            

        //     // CREDIT SIDE
        //     $t_data['voucher_accounting_details'][] = ['account_id'=>$shortage_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->shortage_amt ];

            
        //     //CHECK TOTAL DR AND CR
        //     $total_dr = 0;
        //     $total_cr = 0;

        //     foreach ($t_data['voucher_accounting_details'] as $vad) {
        //         $total_dr += $vad['dr_amount'];
        //         $total_cr += $vad['cr_amount'];
        //     }

        //    // Round both $total_dr and $total_cr to two decimal places
        //    $total_dr_rounded = round(floatval($total_dr), 2);
        //    $total_cr_rounded = round(floatval($total_cr), 2);

        //    // Check if they are not equal
        //    if ($total_dr_rounded != $total_cr_rounded) {
        //        // Output the values for debugging
        //        dd($total_dr_rounded, $total_cr_rounded);
               
        //        // Return response with an error message
        //        return response()->json(['message' => 'Error In Accounting Entries'], 422);
        //    }


        //     $t_data['total_dr_amount'] = $total_dr;
        //     $t_data['total_cr_amount'] = $total_cr; 


        //     $this->create_transaction_voucher( $t_data );
        // }

        

    }
}



