<?php

namespace App\Http\Controllers\Prints;

use App\Http\Controllers\Controller;
use App\Models\Account\Voucher;
use App\Models\Bill\Bill;
use App\Models\Bill\CreditNote;
use App\Models\Booking\BookGr;
use App\Models\Booking\PackagePosition;
use Illuminate\Support\Facades\DB;
use App\Models\Business\Company;
use App\Models\FleetOwner\DriverSettlement;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\FleetOwner\LorryHire;
use App\Models\FleetOwner\LorryHireBill;
use App\Models\GoodsMove\Challan;
use App\Models\GoodsMove\ChallanReceipt;
use App\Models\Master\Vehicle;
use App\Models\Pod\Pod;
use App\Models\HR\Salary;
use App\Models\Inventory\Purchase;
use Carbon\Carbon;
use Illuminate\Http\Request;

class PrintController extends Controller
{


    protected $company;

    public function open_gr_print(Request $request){
            $this->initialize($request->cc);

            $query = BookGr::with([
                'branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name,city,state,tele_number,gst_number,pan_number,pincode,address1', 'consignee:id,party_name,city,state,tele_number,gst_number,pan_number,pincode,address1', 'vehicle:id,vehicle_number,vehicle_type','driver:id,emp_name',
                'gr_desc',
                'gr_desc.material' => function ($query) {
                    $query->select('id', 'material_name');
                },
                'eway_details',
                'acc_party' => function ($query) {
                    $query->select('id', 'account_name','account_type');
                },
                'acc_party.party:id,account_id,party_name,city,state,tele_number,gst_number,pan_number,pincode,address1,payment_term',
                'acc_party_branch:id,branch_name',
                
                'consignor_branch:id,branch_name',
                'consignee_branch:id,branch_name',
            ]);

            if($request->gr_no){
                $gr = $query->where('gr_number',$request->gr_no)->first();
            }
            else{
                $gr = $query->where('gr_number',$request->pk)->first();
            }
            

            return view('docs.gr_print')->with(['gr' => $gr,'company'=>$this->company]);
    }
    
    
     public function open_salary_print(Request $request){
        $this->initialize($request->cc);

        $ls = Salary::with([
                'branch:id,branch_name,branch_code',
                'expense:id,account_name',
                'ss_desc'
            ])->find($request->pk);
        

        return view('docs.salary_print')->with(['salary' => $ls,'company'=>$this->company]);
    }

    public function open_pod_print(Request $request){
        $this->initialize($request->cc);

        $pod = Pod::with([
            'branch:id,branch_name,branch_code,address1,address2,city,state,pincode',
            'lorry_hire_from:id,account_name,account_type', 
            'acc_party:id,account_name',
            'received_in_account:id,account_name',
            'vehicle:id,vehicle_number,vehicle_type',
            'pod_desc:id,gr_id,pod_id,gr_number',
            'pod_desc.bookGr:id,gr_number,gr_date,pay_mode,po_no,shipment_number,acc_party_id,vehicle_id,vehicle_number,delivery_pt,source_id,destination_id,consignor_id,consignee_id,consignor_name,consignee_name,destination_name,total_pkg,total_material_weight,total_km,total_chg_qty,total_desc_value,total_invoice_value,freight,gr_chg,disc_percent,disc_amount,green_tax,kanta,detention,grand_total,payment_dynamic_fields',
            'pod_desc.bookGr.acc_party:id,account_name',
            'pod_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',
            'pod_desc.bookGr.consignor:id,party_name',
            'pod_desc.bookGr.consignee:id,party_name,party_code,address1,city,state,tele_number,gst_number,pan_number,pincode',
            'pod_desc.bookGr.source:id,station_name',
            'pod_desc.bookGr.destination:id,station_name',
            'pod_desc.bookGr.challan_desc:id,challan_id,gr_id,gr_number',
            'pod_desc.bookGr.challan_desc.challan:id,challan_number', 
            ])->find($request->pk);
        

        return view('docs.pod_print')->with(['pod' => $pod,'company'=>$this->company]);
    }


public function open_purchase_print(Request $request){
        $this->initialize($request->cc);

        $bill = Purchase::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name,group_id',
            'acc_party.group:id,group_name',
            'acc_party.party',
            'purchase_desc',
            'purchase_desc.product:id,product_name,unit,maintain_serial',
            'purchase_desc.vehicle:id,vehicle_number',
            'purchase_desc.dr_account:id,account_name',
            'purchase_desc.product_serial_desc',
            'purchase_desc.product_serial_desc.vehicle:id,vehicle_number',
            'created_by_user:id,username',
        ])->find($request->pk);
        
        return view('docs.purchase_print')->with(['bill' => $bill,'company'=>$this->company]);
    }


    public function open_bill_print(Request $request){
        $this->initialize($request->cc);
        $query = Bill::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'acc_party_branch:id,branch_name',
            'acc_party.party:id,account_id,party_name,address1,address2,state,tele_number,gst_number,pan_number,payment_term',
            'income:id,account_name',

            'bill_desc',
            'bill_desc.bookGr:id,gr_number,po_no,shipment_number,gr_date,consignor_id,consignee_id,vehicle_id,source_id,destination_id,consignor_name,consignee_name,total_chg_qty,source_name,destination_name',
            'bill_desc.bookGr.consignor:id,party_name',
            'bill_desc.bookGr.consignee:id,party_name',
            'bill_desc.bookGr.source:id,station_name',
            'bill_desc.bookGr.destination:id,station_name',
            'bill_desc.pod:id,pod_number',
            'bill_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',
            'bill_desc.bookGr.gr_desc',
            'bill_desc.loading_slip',
            'bill_desc.bookGr.challan_desc.challan'
        ]);

        if($request->bill_no){
            $bill = $query->where('bill_number',$request->bill_no)->first();
        }
        else{
            $bill = $query->find($request->pk);
        }


        $bill->bill_desc->transform(function ($desc) {
            $data = [
                'bill_desc_id' => $desc->id,
                'bill_id' => $desc->bill_id,
                'total_without_gst' => $desc->total_without_gst,
                'igst_amount' => $desc->igst_amount,
                'cgst_amount' => $desc->cgst_amount,
                'sgst_amount' => $desc->sgst_amount,
                'rounded_off' => $desc->rounded_off,
                'grand_total' => $desc->grand_total,
            ];

            
        
            if ($desc->selection_type == 'GR') {
                $data += [
                    'gr_id' => $desc->gr_id,
                    'gr_number' => $desc->gr_number,
                    'gr_date' => $desc->bookGr->gr_date,
                    'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name],
                    'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name],
                    'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name],
                    'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                    'district' => $desc->bookGr->district ? $desc->bookGr->district : ['id'=> null, 'station_name'=>$desc->bookGr->district] ,
                    'total_pkg' => $desc->total_pkg,
                    'detention' => $desc->detention,
                    'total_material_weight' => $desc->total_material_weight,
                    'total_chg_qty' => $desc->bookGr->total_chg_qty,
                    'total_desc_value' => $desc->total_desc_value,
                    'total_invoice_value' => $desc->total_invoice_value,
                    'other1' => $desc->other1,
                    'other2' => $desc->other2,
                    'other3' => $desc->other3,
                    'other4' => $desc->other4,
                    'freight' => $desc->freight,
                    'po_no' => $desc->bookGr->po_no,
                    'shipment_number' => $desc->bookGr->shipment_number,
                    'rate'=> $desc->gr_desc ? json_decode($desc->gr_desc)[0]->rate : null,
                    'gr_chg' => $desc->gr_chg,
                    'bookGr' => $desc->bookGr,
                    'gr_desc' => json_decode($desc->gr_desc),
                    'pod_id' => $desc->pod ? $desc->pod_id : null,
                    'pod_number' =>  $desc->pod ? $desc->pod->pod_number:null,
                    
                    'pod_date' => $desc->pod ? $desc->pod->delivery_date:null,
                    'challan_date' => $desc->pod ? $desc->pod->challan_date:null,
                    'vehicle' => $desc->bookGr->vehicle_id ? $desc->bookGr->vehicle : ['id'=> null, 'vehicle_number'=>$desc->bookGr->vehicle_number],
                    'load_detention' => $desc->load_detention,
                    'unload_detention' => $desc->unload_detention,
                    'payment_dynamic_fields' => json_decode($desc->payment_dynamic_fields)??[],
                    'pod_payment_dynamic_fields' => json_decode($desc->pod_payment_dynamic_fields)??[],
                ];
            }
        
            if ($desc->selection_type == 'Loading Slip') {
                $data += [
                    'ls_id' => $desc->ls_id,
                    'ls_number' => $desc->ls_number,
                    'ls_date' => $desc->loading_slip->ls_date,
                    'branch' => $desc->loading_slip->branch,
                    'vehicle' => $desc->loading_slip->vehicle,
                    'source' => $desc->loading_slip->source,
                    'gr_number' => $desc->loading_slip->gr_number,
                    'material_name'=>$desc->loading_slip->material_name,
                    'material_weight'=>$desc->loading_slip->material_weight,
                    'rate'=>$desc->loading_slip->rate,

                    'destination' => $desc->loading_slip->destination,
                    'freight' => $desc->loading_slip->freight,
                    'munshiana' => $desc->loading_slip->munshiana,
                    'dalla' => $desc->loading_slip->dalla,
                    'advance_amt' => $desc->loading_slip->advance_amt,
                    'balance_received_in_amt' => $desc->loading_slip->balance_received_in_amt,
                    'diesel_amt' => $desc->loading_slip->diesel_amt,
                    'payment_and_collection' => $desc->loading_slip->payment_and_collection,
                    'labour' => $desc->loading_slip->labour,
                    'detention' => $desc->loading_slip->detention,
                    'unload_detention' => $desc->loading_slip->unload_detention,
                    'balance_receivable' => $desc->loading_slip->balance_receivable,
                ];
            }

            if ($desc->selection_type == 'Supplementary') {
                $data += [
                    'supplementary_desc' => json_decode($desc->supplementary_desc),
                ];
            }
        
            return $data;
        });        

        if($bill->selection_type == 'GR'){
            
            if(str_contains(strtolower($bill->acc_party->account_name), 'ultratech') ){
              return view('docs.ulta_tech_bill_print')->with(['bill' => $bill,'company'=>$this->company]); 
              
            }else if(str_contains(strtolower($bill->acc_party->account_name), 'jk cement works, prayagraj') ){
              return view('docs.jk_cement_new_bill_print')->with(['bill' => $bill,'company'=>$this->company]);  
              
            }else if(str_contains(strtolower($bill->acc_party->account_name), 'jk cement') ){
              return view('docs.jk_cement_bill_print')->with(['bill' => $bill,'company'=>$this->company]);  
            }
            else if(str_contains(strtolower($bill->acc_party->account_name), 'jk cement') ){
                return view('docs.jk_cement_bill_print')->with(['bill' => $bill,'company'=>$this->company]);  
            }
            else{
                if($request->format==1)
                return view('docs.bill_print')->with(['bill' => $bill,'company'=>$this->company]);
                else
                return view('docs.bill_print2')->with(['bill' => $bill,'company'=>$this->company]);
            }
            
        }
        elseif($bill->selection_type == 'Loading Slip'){
            return view('docs.ls_bill_print')->with(['bill' => $bill,'company'=>$this->company]);
        } elseif($bill->selection_type == 'Supplementary'){
            return view('docs.manual_bill_print')->with(['bill' => $bill,'company'=>$this->company]);
        } 

        //return view('docs.bill_print')->with(['bill' => $bill,'company'=>$this->company]);
    }

    public function open_multi_bill_print(Request $request){
        $this->initialize($request->cc);

        $query = Bill::select('id','bill_number');


        if( isset($request['acc_party']) && $request['acc_party']){
            $query->where('acc_party_id', $request['acc_party']);
        }


        if($request['bill_num_from'] && $request['bill_num_to']){
            
            $query->where('bill_number','>=',$request['bill_num_from'])->where('bill_number','<=',$request['bill_num_to']);
        }
    
        if($request['from_date'] && $request['to_date']){
            $query->where('bill_date','>=',$request['from_date'])->where('bill_date','<=',$request['to_date']);
        }

        if($request['selection_type'] ){
            $query->where('selection_type', $request['selection_type']);
        }

        $bills = $query->get();


        $all_bills = [];



        foreach ($bills as $key) {
            $bill = Bill::with([
                'branch:id,branch_name,branch_code',
                'acc_party:id,account_name',
                'acc_party_branch:id,branch_name',
                'acc_party.party:id,account_id,party_name,address1,address2,state,tele_number,gst_number,pan_number',
                'income:id,account_name',
    
                'bill_desc',
                'bill_desc.bookGr:id,gr_number,gr_date,consignor_id,consignee_id,vehicle_id,source_id,destination_id,consignor_name,consignee_name,total_chg_qty,source_name,destination_name',
                'bill_desc.bookGr.consignor:id,party_name',
                'bill_desc.bookGr.consignee:id,party_name',
                'bill_desc.bookGr.source:id,station_name',
                'bill_desc.bookGr.destination:id,station_name',
                'bill_desc.pod:id,pod_number',
                'bill_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',
                'bill_desc.bookGr.gr_desc',
                'bill_desc.loading_slip'
            ])->find($key['id']);


            $bill->bill_desc->transform(function ($desc) {
                $data = [
                    'bill_desc_id' => $desc->id,
                    'bill_id' => $desc->bill_id,
                    'total_without_gst' => $desc->total_without_gst,
                    'igst_amount' => $desc->igst_amount,
                    'cgst_amount' => $desc->cgst_amount,
                    'sgst_amount' => $desc->sgst_amount,
                    'rounded_off' => $desc->rounded_off,
                    'grand_total' => $desc->grand_total,
                ];
    
                
            
                if ($desc->selection_type == 'GR') {
                    $data += [
                        'gr_id' => $desc->gr_id,
                        'gr_number' => $desc->gr_number,
                        'gr_date' => $desc->bookGr->gr_date,
                        'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name],
                        'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name],
                        'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name],
                        'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                        'total_pkg' => $desc->total_pkg,
                        'detention' => $desc->detention,
                        'total_material_weight' => $desc->total_material_weight,
                        'total_chg_qty' => $desc->bookGr->total_chg_qty,
                        'total_desc_value' => $desc->total_desc_value,
                        'total_invoice_value' => $desc->total_invoice_value,
                        'freight' => $desc->freight,
                        'gr_chg' => $desc->gr_chg,
                        'bookGr' => $desc->bookGr,
                        'gr_desc' => json_decode($desc->gr_desc),
                        'pod_id' => $desc->pod ? $desc->pod_id : null,
                        'pod_number' =>  $desc->pod ? $desc->pod->pod_number:null,
                        'vehicle' => $desc->bookGr->vehicle_id ? $desc->bookGr->vehicle : ['id'=> null, 'vehicle_number'=>$desc->bookGr->vehicle_number],
                        'load_detention' => $desc->load_detention,
                        'unload_detention' => $desc->unload_detention,
                        'payment_dynamic_fields' => json_decode($desc->payment_dynamic_fields)??[],
                        'pod_payment_dynamic_fields' => json_decode($desc->pod_payment_dynamic_fields)??[],
                    ];
                }
            
                if ($desc->selection_type == 'Loading Slip') {
                    $data += [
                        'ls_id' => $desc->ls_id,
                        'ls_number' => $desc->ls_number,
                        'ls_date' => $desc->loading_slip->ls_date,
                        'branch' => $desc->loading_slip->branch,
                        'vehicle' => $desc->loading_slip->vehicle,
                        'source' => $desc->loading_slip->source,
                        'gr_number' => $desc->loading_slip->gr_number,
                        'material_name'=>$desc->loading_slip->material_name,
                        'material_weight'=>$desc->loading_slip->material_weight,
                        'rate'=>$desc->loading_slip->rate,
    
                        'destination' => $desc->loading_slip->destination,
                        'freight' => $desc->loading_slip->freight,
                        'munshiana' => $desc->loading_slip->munshiana,
                        'dalla' => $desc->loading_slip->dalla,
                        'advance_amt' => $desc->loading_slip->advance_amt,
                        'balance_received_in_amt' => $desc->loading_slip->balance_received_in_amt,
                        'diesel_amt' => $desc->loading_slip->diesel_amt,
                        'payment_and_collection' => $desc->loading_slip->payment_and_collection,
                        'labour' => $desc->loading_slip->labour,
                        'detention' => $desc->loading_slip->detention,
                        'unload_detention' => $desc->loading_slip->unload_detention,
                        'balance_receivable' => $desc->loading_slip->balance_receivable,
                    ];
                }
    
                if ($desc->selection_type == 'Supplementary') {
                    $data += [
                        'supplementary_desc' => json_decode($desc->supplementary_desc),
                    ];
                }
            
                return $data;
            });   

            $all_bills[] = $bill;
        }
        if($request['selection_type'] == 'GR'){
            return view('docs.bill_print')->with(['multi_print'=>1, 'multi_bills' => $all_bills,'company'=>$this->company]);
        }elseif($request['selection_type'] == 'Loading Slip'){
            return view('docs.ls_bill_print')->with(['multi_print'=>1, 'multi_bills' => $all_bills,'company'=>$this->company]);
        } elseif($request['selection_type'] == 'Supplementary'){
            return view('docs.manual_bill_print')->with(['multi_print'=>1, 'multi_bills' => $all_bills,'company'=>$this->company]);
        } 

        // return view('docs.bill_print')->with(['bill' => $bill,'company'=>$this->company]);
    }

    public function open_challan_print(Request $request){
        $this->initialize($request->cc);

        $challan = Challan::with([
            'branch:id,branch_name,branch_code',
            'source:id,station_name',
            'destination:id,station_name',
            'vehicle:id,vehicle_number,chasis_number,engine_number,owner_id',
            'vehicle.party',
            'driver:id,emp_name,address,pan_number,contact_number,license_number',
            'receiver_branch:id,branch_name,branch_code',
            'challan_desc',
            'challan_desc.gr_desc:id,gr_id,gr_number,pkg,free_pkgs,rate,material_id,material_weight',
            'challan_desc.gr_desc.material:id,material_name',
            'challan_desc.gr_desc.bookGr:id,branch_id,gr_number,gr_date,consignor_name,consignee_name,consignor_phone_number,consignee_phone_number,destination_name,consignor_id,acc_party_id,consignee_id,destination_id,pay_mode,grand_total',
            'challan_desc.gr_desc.bookGr.consignor:id,party_name',
            'challan_desc.gr_desc.bookGr.acc_party:id,account_name',
            'challan_desc.gr_desc.bookGr.consignee:id,party_name',
            'challan_desc.gr_desc.bookGr.destination:id,station_name',
            'challan_desc.gr_desc.bookGr.eway_details',
            'broker',
            'lorry_hire_from:id,account_name',
            'dl_comm_branch:id,account_name',
            'commission_account:id,account_name',
            'dr_on_account:id,account_name',
            'paid_using:id,account_name',
        ])->find($request->pk);

        $challan_branch = $challan->branch_id;

        $challan->challan_desc->transform(function ($desc) use ($challan_branch) {
            $free = PackagePosition::where('gr_desc_id', $desc->gr_desc_id)->where('position', $challan_branch)->sum('pkg');
            return [
                'branch_id' => $desc->gr_desc->bookGr->branch_id, 
                'challan_desc_id' => $desc->id,
                'gr_id' => $desc->gr_desc->gr_id,
                'gr_number' => $desc->gr_desc->gr_number,
                'gr_date' => $desc->gr_desc->bookGr->gr_date,
                'acc_party' => $desc->gr_desc->bookGr->acc_party,
                'challan_id' => $desc->challan_id,
                'challan_number' => $desc->challan_number,
                'consignor_phone_number' => $desc->gr_desc->bookGr->consignor_phone_number ,
                'consignee_phone_number' => $desc->gr_desc->bookGr->consignee_phone_number ,
                'gr_desc_id' => $desc->gr_desc_id,
                'pkg' => $desc->gr_desc->pkg,
                'wt' => $desc->gr_desc->material_weight,
                'rate' => $desc->gr_desc->rate,
                'free_pkgs' => $free,
                'loaded_pkgs' => $desc->loaded_pkgs,
                'material' => $desc->gr_desc->material['material_name'],
                'consignor' => $desc->gr_desc->bookGr->consignor_id ? $desc->gr_desc->bookGr->consignor['party_name'] : $desc->gr_desc->bookGr->consignor_name,
                'consignee' => $desc->gr_desc->bookGr->consignee_id ? $desc->gr_desc->bookGr->consignee['party_name'] : $desc->gr_desc->bookGr->consignee_name,
                'destination' => $desc->gr_desc->bookGr->destination_id ? $desc->gr_desc->bookGr->destination['station_name'] : $desc->gr_desc->bookGr->destination_name,
                'grand_total' => $desc->gr_desc->bookGr->grand_total,
                'pay_mode' => $desc->gr_desc->bookGr->pay_mode,
                'eway_details' => $desc->gr_desc->bookGr->eway_details 
            ];
        });
        

        return view('docs.challan_print')->with(['challan' => $challan,'company'=>$this->company]);
    }

    public function open_credit_note_print(Request $request){
        $this->initialize($request->cc);

        $credit_note = CreditNote::with([
                        'branch:id,branch_name,branch_code',
                        'acc_party:id,account_name',
                        'acc_party.party',
                        'cn_desc',
                        'created_by_user:id,username',
                        'income:id,account_name'
                    ])->find($request->pk);
        

        if($credit_note->note_name=='Credit Note'){
            return view('docs.credit_note')->with(['credit_note' => $credit_note,'company'=>$this->company]);
        }
        else if($credit_note->note_name=='Debit Note'){
            return view('docs.debit_note')->with(['credit_note' => $credit_note,'company'=>$this->company]);
        }
    }

    public function open_lorry_hire_bill_print(Request $request){
        $this->initialize($request->cc);

        $lorry_hire_bill = LorryHireBill::with([
            'branch:id,branch_name,branch_code',
            'lorry_hire_from:id,account_name',
            'lorry_hire_from.party',
            'vehicle:id,vehicle_number',
            'lhb_desc',
            'lhb_desc.challan',
            'lhb_desc.challan_desc',
            'lhb_desc.challan_desc.bookGr:id,gr_number,gr_date,consignor_name,source_name,consignee_name,destination_name,total_material_weight',
            'lhb_desc.challan_desc.bookGr.pod_desc',
            'lhb_desc.challan_desc.bookGr.pod_desc.pod',
            'lhb_desc.challan.voucher_desc'=>function($query){
                $query->where('against','Challan')->where('voucher_name','Payment Voucher');
            },
            'lhb_desc.challan.voucher_desc.voucher:id,voucher_number,narration,voucher_date',
        ])->find($request->pk);

        $lorry_hire_bill->lhb_desc->transform(function ($desc) use ($lorry_hire_bill)  {  
            return [
                'lhb_desc_id' => $desc->id,
                'lhb_id' => $lorry_hire_bill->id,
                'challan_id' => $desc->challan_id,
                'challan_number' => $desc->challan_number,
                'challan_date' => $desc->challan->challan_date,
                'lh_net_freight' => $desc->lh_net_freight,
                'balance' => $desc->balance,
                'lh_freight' => $desc->lh_freight,
                'total_lh_deduction' => $desc->total_lh_deduction,
                'total_without_gst'=> $desc->total_without_gst,
                'gr_number' => $desc->challan_desc ? $desc->challan_desc->bookGr->gr_number : '',
                'gr_date' => $desc->challan_desc ? $desc->challan_desc->bookGr->gr_date : '',
                'vehicle_number' => $desc->challan ? $desc->challan->vehicle_number : '',
                'consignor' => $desc->challan_desc ? $desc->challan_desc->bookGr->consignor_name : '',
                'source' => $desc->challan_desc ? $desc->challan_desc->bookGr->source_name : '',
                'consignee' => $desc->challan_desc ? $desc->challan_desc->bookGr->consignee_name : '',
                'destination' => $desc->challan_desc ? $desc->challan_desc->bookGr->destination_name : '',
                'load_wt' => $desc->challan_desc ? $desc->challan_desc->bookGr->total_material_weight : '',
                'unload_wt' => $desc->challan_desc ? $desc->challan_desc->bookGr->pod_desc?$desc->challan_desc->bookGr->pod_desc->pod->unloaded_wt:0 : 0,
                'hire_rate' => $desc->challan ? $desc->challan->lh_rate : '',
                'lh_freight' => $desc->challan ? $desc->challan->lh_freight : '',
                'lh_advance' => $desc->challan ? $desc->challan->lh_advance : '',
                'lh_net_freight' => $desc->challan ? $desc->challan->lh_net_freight : '',
                'shortage_qty' => $desc->challan_desc ? $desc->challan_desc->bookGr->pod_desc?$desc->challan_desc->bookGr->pod_desc->pod->shortage_qty:0 : 0,
                'shortage_amt' => $desc->challan_desc ? $desc->challan_desc->bookGr->pod_desc?$desc->challan_desc->bookGr->pod_desc->pod->shortage_amt:0 : 0,
                'voucher_desc' => $desc->challan->voucher_desc ? $desc->challan->voucher_desc : '',
                'challan'=> $desc->challan,
                'pod'=> $desc->challan_desc ? $desc->challan_desc->bookGr->pod_desc? $desc->challan_desc->bookGr->pod_desc->pod:null : '',

            ];
        });

        

        return view('docs.lorry_hire_bill')->with(['lorry_hire_bill' => $lorry_hire_bill,'company'=>$this->company]);
    }

    public function open_challan_receipt_print(Request $request){
        $this->initialize($request->cc);

        $challan_rec = ChallanReceipt::with([
            'branch:id,branch_name,branch_code',
            'arrival_from:id,branch_name,branch_code',
            'vehicle:id,vehicle_number',
            'driver:id,emp_name',
            'challan:id,challan_number',
            'challan_rec_desc',
            'challan_rec_desc.challan_desc:id,loaded_pkgs',
            'challan_rec_desc.gr_desc:id,gr_id,gr_number,pkg,free_pkgs,material_id,material_weight',
            'challan_rec_desc.gr_desc.material:id,material_name',
            'challan_rec_desc.gr_desc.bookGr:id,gr_number,consignor_id,consignee_id,destination_id,consignor_name,consignee_name,destination_name,pay_mode,grand_total',
            'challan_rec_desc.gr_desc.bookGr.consignor:id,party_name',
            'challan_rec_desc.gr_desc.bookGr.consignee:id,party_name',
            'challan_rec_desc.gr_desc.bookGr.destination:id,station_name',
        ])->find($request->pk);

        $challan_rec->challan_rec_desc->transform(function ($desc) {
            return [

                'challan_rec_id' => $desc->id,
                'challan_rec_number' => $desc->challan_rec_number,
                'challan_rec_desc_id' => $desc->id,
                'gr_id' => $desc->gr_desc->gr_id,
                'gr_number' => $desc->gr_desc->gr_number,
                'gr_desc_id' => $desc->gr_desc_id,
                'challan_id' => $desc->challan_id,
                'challan_number' => $desc->challan_number,
                'challan_desc_id' => $desc->challan_desc_id,
                'pkg' => $desc->gr_desc->pkg,
                'wt' => $desc->gr_desc->material_weight,
                'free_pkgs' => $desc->gr_desc->free_pkgs,
                'loaded_pkgs' => $desc->challan_desc->loaded_pkgs,
                'unloaded_pkgs' => $desc->unloaded_pkgs,
                'material' => $desc->gr_desc->material['material_name'],
                'consignor' => $desc->gr_desc->bookGr->consignor_id ? $desc->gr_desc->bookGr->consignor['party_name'] : $desc->gr_desc->bookGr->consignor_name,
                'consignee' => $desc->gr_desc->bookGr->consignee_id ? $desc->gr_desc->bookGr->consignee['party_name'] : $desc->gr_desc->bookGr->consignee_name,
                'destination' => $desc->gr_desc->bookGr->destination_id ? $desc->gr_desc->bookGr->destination['station_name'] : $desc->gr_desc->bookGr->destination_name,
                'grand_total' => $desc->gr_desc->bookGr->grand_total,
                'pay_mode' => $desc->gr_desc->bookGr->pay_mode,
            ];
        });

        $challan_rec['total_loaded_pkg'] = $challan_rec->challan_rec_desc->sum('unloaded_pkgs');
        $challan_rec['total_unloaded_pkg'] = $challan_rec->challan_rec_desc->sum('unloaded_pkgs');
        

        $copy = json_decode(json_encode($challan_rec));
        
        $challan_rec['arrival_from_branch'] = $copy->arrival_from->branch_name;

        return view('docs.challan_receipt_print')->with(['challan_receipt' => $challan_rec,'company'=>$this->company]);
    }

    public function open_vehicle_alert_print(Request $request){
            $this->initialize($request->cc);

        $days = 15;
        // Calculate the date 15 days from now
        $allowedTillDate = Carbon::now()->addDays($days);

        // Define the expiry date fields and their corresponding document names
        $expiryDateFields = [
            'permit_exp_date' => 'Permit',
            'five_years_permit_due_date' => '5 Years Permit',
            'insurance_due_date' => 'Insurance',
            'road_tax_due_date' => 'Road Tax',
            'fitness_due_date' => 'Fitness Certificate',
            'reg_due_date' => 'Registration',
            'pollution_due_date' => 'Pollution Certificate',
        ];

        $expiryFields = [
            'permit_exp_date' => 'permit_number',
            'five_years_permit_due_date' => 'five_years_permit_number',
            'insurance_due_date' => 'insurance',
            'road_tax_due_date' => 'road_tax',
            'fitness_due_date' => 'fitness_certificate_number',
            'reg_due_date' => 'reg_number',
            'pollution_due_date' => 'pollution',
        ];

        // Retrieve vehicles with expiry dates within the next 15 days
        $reminders = Vehicle::select(
            'vehicle_number',
            'id',
            ...array_keys($expiryDateFields), // Include all expiry date fields
            ...array_values($expiryFields) // Include all expiry fields
        )
        ->where(function ($query) use ($allowedTillDate, $expiryDateFields) {
            foreach ($expiryDateFields as $field => $documentName) {
                $query->orWhereDate($field, '<=', $allowedTillDate);
            }
        })
        ->get();

        // Group the data by expiry date fields
        $groupedData = [];

        foreach ($expiryDateFields as $field => $documentName) {
            $groupedData[$documentName] = [];

            $reminders->each(function ($reminder) use (&$groupedData, $field,$days, $documentName,$expiryFields, $allowedTillDate) {
                if ($reminder->{$field}) {
                    $dateValue = Carbon::parse($reminder->{$field});
                    $formattedDate = $dateValue->format('d M Y');

                    $currentDate = Carbon::now();
                    $difference = $dateValue->isPast() ? -$currentDate->diffInDays($dateValue) : $currentDate->diffInDays($dateValue) + 1;

                    if ($difference <= $days) {
                        $groupedData[$documentName][] = [
                            'vehicle_id' => $reminder->id,
                            'vehicle_number' => $reminder->vehicle_number,
                            'expiry_date' => $formattedDate,
                            'against' => $reminder->{$expiryFields[$field]},
                            'field_difference' => $difference,
                        ];
                    }
                }
            });
        }
            

            return view('docs.vehicle_alert')->with(['vehicle_alerts' => $groupedData,'company'=>$this->company]);
    }

    public function open_ls_bill_print(Request $request){
        $this->initialize($request->cc);
        $bill = Bill::with([
            'branch:id,branch_name,branch_code',
            'acc_party:id,account_name',
            'acc_party_branch:id,branch_name',
            'acc_party.party:id,account_id,party_name,address1,address2,state,tele_number,gst_number',
            'income:id,account_name',

            'bill_desc',
            'bill_desc.bookGr:id,gr_number,gr_date,consignor_id,consignee_id,source_id,destination_id,consignor_name,consignee_name,source_name,destination_name',
            'bill_desc.bookGr.consignor:id,party_name',
            'bill_desc.bookGr.consignee:id,party_name',
            'bill_desc.bookGr.source:id,station_name',
            'bill_desc.bookGr.destination:id,station_name',
            'bill_desc.pod:id,pod_number',
            
            'bill_desc.bookGr.vehicle:id,vehicle_number,vehicle_type',

            'bill_desc.loading_slip'
        ])->find($request->pk);


        $bill->bill_desc->transform(function ($desc) {
            $data = [
                'bill_desc_id' => $desc->id,
                'bill_id' => $desc->bill_id,
                'total_without_gst' => $desc->total_without_gst,
                'igst_amount' => $desc->igst_amount,
                'cgst_amount' => $desc->cgst_amount,
                'sgst_amount' => $desc->sgst_amount,
                'rounded_off' => $desc->rounded_off,
                'grand_total' => $desc->grand_total,
            ];

            
        
            if ($desc->selection_type == 'GR') {
                $data += [
                    'gr_id' => $desc->gr_id,
                    'gr_number' => $desc->gr_number,
                    'gr_date' => $desc->bookGr->gr_date,
                    'consignor' => $desc->bookGr->consignor_id ? $desc->bookGr->consignor : ['id'=> null, 'party_name'=>$desc->bookGr->consignor_name],
                    'consignee' => $desc->bookGr->consignee_id ? $desc->bookGr->consignee : ['id'=> null, 'party_name'=>$desc->bookGr->consignee_name],
                    'source' => $desc->bookGr->source_id ? $desc->bookGr->source : ['id'=> null, 'station_name'=>$desc->bookGr->source_name],
                    'destination' => $desc->bookGr->destination ? $desc->bookGr->destination : ['id'=> null, 'station_name'=>$desc->bookGr->destination_name] ,
                    'total_pkg' => $desc->total_pkg,
                    'total_material_weight' => $desc->total_material_weight,
                    'total_chg_qty' => $desc->total_chg_qty,
                    'total_desc_value' => $desc->total_desc_value,
                    'total_invoice_value' => $desc->total_invoice_value,
                    'freight' => $desc->freight,
                    'gr_chg' => $desc->gr_chg,
                    'gr_desc' => json_decode($desc->gr_desc),
                    'pod_id' => $desc->pod ? $desc->pod_id : null,
                    'pod_number' =>  $desc->pod ? $desc->pod->pod_number:null,
                    'load_detention' => $desc->load_detention,
                    'unload_detention' => $desc->unload_detention,
                    'payment_dynamic_fields' => json_decode($desc->payment_dynamic_fields)??[],
                    'pod_payment_dynamic_fields' => json_decode($desc->pod_payment_dynamic_fields)??[],
                ];
            }
        
            if ($desc->selection_type == 'Loading Slip') {
                $data += [
                    'ls_id' => $desc->ls_id,
                    'ls_number' => $desc->ls_number,
                    'total_pkg' => $desc->loading_slip->pkg,
                    'rate' => $desc->loading_slip->rate,
                    'ls_date' => $desc->loading_slip->ls_date,
                    'branch' => $desc->loading_slip->branch,
                    'vehicle' => $desc->loading_slip->vehicle,
                    'source' => $desc->loading_slip->source,
                    'destination' => $desc->loading_slip->destination,
                    'freight' => $desc->loading_slip->freight,
                    'munshiana' => $desc->loading_slip->munshiana,
                    'dalla' => $desc->loading_slip->dalla,
                    'advance_amt' => $desc->loading_slip->advance_amt,
                    'balance_received_in_amt' => $desc->loading_slip->balance_received_in_amt,
                    'diesel_amt' => $desc->loading_slip->diesel_amt,
                    'payment_and_collection' => $desc->loading_slip->payment_and_collection,
                    'labour' => $desc->loading_slip->labour,
                    'detention' => $desc->loading_slip->detention,
                    'unload_detention' => $desc->loading_slip->unload_detention,
                    'balance_receivable' => $desc->loading_slip->balance_receivable,
                ];
            }

            if ($desc->selection_type == 'Supplementary') {
                $data += [
                    'supplementary_desc' => json_decode($desc->supplementary_desc),
                ];
            }
        
            return $data;
        });        

        return view('docs.ls_bill_print')->with(['bill' => $bill,'company'=>$this->company]);
    }

    public function open_voucher_print(Request $request){
        $this->initialize($request->cc);
        $voucher = Voucher::with(['branch:id,branch_name','voucher_acc_details:id,voucher_id,account_id,nature,against,dr_amount,cr_amount', 'voucher_descs:id,voucher_id,v_acc_det_id,against_id,against,against_number,date,due_amt,receipt_amt,received_amt,tds_percent,tds,deduction,balance_amt,from_opening','voucher_acc_details.account:id,account_name','voucher_acc_details.account.party:id,account_id,party_name,pan_number,account_number,bank_name,ifsc','voucher_descs.challan:id,challan_number,challan_date,vehicle_id,source_id,destination_id,lh_shortage,lh_total_wt,lh_rate,lh_extra_point,lh_tds,lh_advance,lh_net_freight,lh_balance','voucher_descs.challan.vehicle:id,vehicle_number','voucher_descs.challan.source:id,station_name','voucher_descs.challan.destination:id,station_name','voucher_descs.challan.voucher_desc','voucher_descs.challan.voucher_desc.voucher'])->find($request->pk);
        
        if($request->format==1){
           return view('docs.voucher_print')->with(['voucher' => $voucher,'company'=>$this->company]);
        }
        else if($request->format==2){
            return view('docs.voucher_challan_print')->with(['voucher' => $voucher,'company'=>$this->company]);
        }
    }

    public function open_ls_print(Request $request){
        $this->initialize($request->cc);

        $ls = LoadingSlip::with([
            'branch:id,branch_name,branch_code',
            'source:id,station_name',
            'destination:id,station_name',
            'vehicle:id,vehicle_number',
            'acc_party:id,account_name',
            'acc_party.party:id,party_name,account_id,mob_number,address1,state,city,gst_number',
            'driver:id,account_name',
            'driver.employee:id,account_id,emp_name,contact_number',
            'driver_payment_from:id,account_name',
            'advance_received_in:id,account_name',
            'balance_received_in:id,account_name',
            'income:id,account_name'
            ])->find($request->pk);
        

        return view('docs.ls_print')->with(['ls' => $ls,'company'=>$this->company]);
    }

    public function open_driver_settlement_print(Request $request){
        $this->initialize($request->cc);

        $query = DriverSettlement::with([
            'branch:id,branch_name,branch_code',
            'vehicle:id,vehicle_number,account_id,vehicle_type',
            'driver:id,emp_name,account_id,contact_number,pan_number,license_number',
            'd_settlement_desc',
        ])->find($request->pk);

        $driver_account_id = $query->driver->account_id;

        $unmarked_voucher = Voucher::with(['voucher_acc_details'=>function($sub){
                                    $sub->where('nature','cr');
                            },
                            'voucher_acc_details.account:id,account_name,account_type'
                         ])
                        ->whereBetween('voucher_date',[$query->from_date, $query->till_date])
                        ->whereHas('voucher_acc_details', function ($query) use ($driver_account_id) {
                            $query->where('account_id', $driver_account_id)->where('nature','dr'); 
                        })
                        ->whereIn('voucher_name',['Payment Voucher','Journal'])
                        ->whereDoesntHave('voucher_descs')
                        ->get();


        $query->d_settlement_desc->transform(function ($desc) {
            $diesel_receipt = [];
            $payment_voucher = [];

            if($desc->against == 'GR'){
                $book_gr = BookGr::select('id','gr_date','acc_party_id','vehicle_id','total_chg_qty','source_id','destination_id','driver_payment_amt','freight','grand_total')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name','acc_party:id,account_name','payment_voucher','diesel_receipt','diesel_receipt.filling_station:id,account_name',
                    'payment_voucher.voucher:id,voucher_number,voucher_date,narration',
                    'gr_desc',
                    'gr_desc.material:id,material_name',
                    'payment_voucher.voucher.voucher_acc_details'=>function($sub){
                                $sub->where('nature','cr');
                    },
                    'payment_voucher.voucher.voucher_acc_details.account:id,account_name,account_type'
                ])->find($desc->against_id);
                $date = $book_gr->gr_date;
                $vehicle = $book_gr->vehicle ? $book_gr->vehicle->vehicle_number : null;
                $source = $book_gr->source ? $book_gr->source->station_name : null;
                $destination = $book_gr->destination ? $book_gr->destination->station_name : null;
                $acc_party = $book_gr->acc_party['account_name'];

                $material = $book_gr->gr_desc ? $book_gr->gr_desc[0]->material['material_name'] : null;
                $weight = $book_gr->total_chg_qty;
                $rate = $book_gr->gr_desc ? $book_gr->gr_desc[0]->rate : 0;
                $freight = $book_gr->freight;
                $grand_total = $book_gr->grand_total;

                $driver_payment_amt = $book_gr->driver_payment_amt;

                $start_reading = 0;
                $end_reading = 0;

                $diesel_receipt = $book_gr->diesel_receipt;
                $payment_voucher = $book_gr->payment_voucher;

            }
            elseif($desc->against == 'Loading Slip'){
                $l_slip = LoadingSlip::select('id','ls_date','acc_party_id','vehicle_id','source_id','destination_id','start_reading','end_reading','driver_payment_amt')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name','acc_party:id,account_name','diesel_receipt','diesel_receipt.filling_station:id,account_name',
                    'payment_voucher',
                    'payment_voucher.voucher:id,voucher_number,voucher_date,narration',
                    'payment_voucher.voucher.voucher_acc_details'=>function($sub){
                                $sub->where('nature','cr');
                    },
                    'payment_voucher.voucher.voucher_acc_details.account:id,account_name,account_type'
                ])->find($desc->against_id);
                $date = $l_slip->ls_date;
                $vehicle = $l_slip->vehicle ? $l_slip->vehicle->vehicle_number : null;
                $source = $l_slip->source ? $l_slip->source->station_name : null;
                $destination = $l_slip->destination ? $l_slip->destination->station_name : null;
                $acc_party = $l_slip->acc_party['account_name'];

                $driver_payment_amt = $l_slip->driver_payment_amt;

                $material = $l_slip->material_name;
                $weight = $l_slip->material_weight;
                $rate = $l_slip->rate;
                $freight = $l_slip->freight;
                $grand_total = $l_slip->grand_total;


                $start_reading = 0;
                $end_reading = 0;

                $diesel_receipt = $l_slip->diesel_receipt;
                $payment_voucher = $l_slip->payment_voucher;

            }
            elseif($desc->against == 'Challan'){
                $challan = Challan::select('id','challan_date','vehicle_id','source_id','destination_id')->with(['vehicle:id,vehicle_number','source:id,station_name','destination:id,station_name','diesel_receipt','diesel_receipt.filling_station:id,account_name','payment_voucher','payment_voucher.voucher:id,voucher_number,voucher_date,narration'])->find($desc->against_id);
                $date = $challan->challan_date;
                $vehicle = $challan->vehicle ?  $challan->vehicle->vehicle_number : null;
                $source = $challan->source ? $challan->source->station_name : null;
                $destination = $challan->destination ? $challan->destination->station_name : null;
                $acc_party = '';

                $material = '';
                $weight = 0;
                $rate = 0;
                $freight = 0;
                $grand_total = 0;

                $driver_payment_amt = 0;
                
                $start_reading = 0;
                $end_reading = 0;

                $diesel_receipt = $challan->diesel_receipt;
                $payment_voucher = $challan->payment_voucher;
            }
                
                return [
                    'd_settlement_desc_id' => $desc->id,
                    'against' => $desc->against,
                    'against_number' => $desc->against_number,
                    'against_id' => $desc->against_id,
                    'vehicle_number' => $vehicle,
                    'source' => $source,
                    'destination' => $destination,
                    'date' => $date,
                    'acc_party' => $acc_party,
                    
                    'material' => $material,
                    'weight' => $weight,
                    'rate' => $rate,
                    'freight' => $freight,
                    'grand_total' => $grand_total,

                    'driver_payment_amt' => $driver_payment_amt,
                    
                    'diesel_receipt_amt' => $desc->diesel_receipt_amt,
                    'diesel_receipt_qty' => $desc->diesel_receipt_qty,
                    'alloted_dsl' => $desc->alloted_dsl,
                    'dsl_diff' => $desc->dsl_diff,
                    'dsl_rate' => $desc->dsl_rate,
                    'dsl_diff_amount' => $desc->dsl_diff_amount,
                    'freight' => $desc->freight,
                    'advance' => $desc->advance,
                    'reading' => $desc->reading,
                    'return_freight' => $desc->return_freight,
                    'green_tax' => $desc->green_tax,
                    'diesel_amt' => $desc->diesel_amt,
                    'toll' => $desc->toll,
                    'state_entry' => $desc->state_entry,
                    'legal_exp' => $desc->legal_exp,
                    'food' => $desc->food,
                    'maintainance' => $desc->maintainance,
                    'parking' => $desc->parking,
                    'dalla' => $desc->dalla,
                    'other1' => $desc->other1,
                    'other2' => $desc->other2,
                    'other3' => $desc->other3,
                    'total' => $desc->total,
                    'diesel_receipt' => $diesel_receipt,
                    'payment_voucher' => $payment_voucher,
                ];
        });

        $d_settlement = $query;     

        return view('docs.driver_settlement_print')->with(['ds' =>  $d_settlement, 'unmarked_voucher'=>$unmarked_voucher,'company'=>$this->company]);
}

public function open_lh_print(Request $request){
    $this->initialize($request->cc);

    $gr = BookGr::with([
        'branch:id,branch_name,branch_code','source:id,station_name', 'destination:id,station_name', 'consignor:id,party_name,mob_number', 'consignee:id,party_name,mob_number', 'vehicle:id,vehicle_number','driver:id,emp_name,contact_number',
        'gr_desc',
        'gr_desc.material' => function ($query) {
            $query->select('id', 'material_name');
        },
        'eway_details',
        'acc_party' => function ($query) {
            $query->select('id', 'account_name','account_type');
        },
        'acc_party.party:id,account_id,mob_number',
        'acc_party_branch:id,branch_name,tele_number',
        'lorry_hire_from:id,account_name',
        'lorry_hire_from.party',
        
        'consignor_branch:id,branch_name,tele_number',
        'consignee_branch:id,branch_name,tele_number',
    ])->find($request->pk);
    

    return view('docs.lh_print')->with(['gr' => $gr,'company'=>$this->company]);
}

    public function open_lorry_hire_print(Request $request){
        $this->initialize($request->cc);

        $lorry_hire = LorryHire::with([
            'branch:id,branch_name,branch_code',
            'source:id,station_name',
            'destination:id,station_name',
            'vehicle:id,vehicle_number,vehicle_type',
            'driver:id,emp_name',
            'broker:id,party_name',
            'lorry_hire_desc',
            'lorry_hire_from:id,account_name',
            'dr_on_account:id,account_name',
            'paid_using:id,account_name',
            'voucher_desc',
        ])->find($request->pk);

        $lorry_hire->lorry_hire_desc->transform(function ($desc) {
            if($desc->against == 'GR'){
                $book_gr = BookGr::select('gr_date','acc_party_id','vehicle_id', 'vehicle_number')->with(['vehicle:id,vehicle_number,vehicle_type','acc_party:id,account_name'])->find($desc->against_id);
                $date = $book_gr->gr_date;
                $vehicle = $book_gr->vehicle ? $book_gr->vehicle->vehicle_number : $book_gr->vehicle_number;
                $vehicle_type = $book_gr->vehicle ? $book_gr->vehicle->vehicle_type : $book_gr->vehicle_type;
                $party = $book_gr->acc_party->account_name;
            }
            elseif($desc->against == 'Loading Slip'){
                $l_slip = LoadingSlip::select('ls_date','acc_party_id','vehicle_id', 'vehicle_number')->with(['vehicle:id,vehicle_number,vehicle_type','acc_party:id,account_name'])->find($desc->against_id);
                $date = $l_slip->ls_date;
                $vehicle = $l_slip->vehicle ? $l_slip->vehicle->vehicle_number : $l_slip->vehicle_number;
                $vehicle_type = $l_slip->vehicle ? $l_slip->vehicle->vehicle_type : $l_slip->vehicle_type;
                $party = $l_slip->acc_party->account_name;
            }
            elseif($desc->against == 'Challan'){
                $challan = Challan::select('challan_date','vehicle_id', 'vehicle_number')->with(['vehicle:id,vehicle_number,vehicle_type'])->find($desc->against_id);
                $date = $challan->challan_date;
                $vehicle = $challan->vehicle ?  $challan->vehicle->vehicle_number : $challan->vehicle_number;
                $vehicle_type = $challan->vehicle ? $challan->vehicle->vehicle_type : $challan->vehicle_type;
                $party = null;
            }

            
            return [
                'lorry_hire_desc_id' => $desc->id,
                'against' => $desc->against,
                'against_number' => $desc->against_number,
                'against_id' => $desc->against_id,
                'vehicle' => $vehicle,
                'vehicle_type' => $vehicle_type,
                'party' => $party,
                'date' => $date,
                'loaded_wt' => $desc->loaded_wt,
                'loaded_pkg' => $desc->loaded_pkg,
                'voucher' => $desc->voucher ? $desc->voucher->voucher_number : null,
                'voucher_desc' => $desc->voucher_desc ? $desc->voucher_desc->description : null
            ];
        });
        

        return view('docs.lorry_hire_print')->with(['lorry_hire' =>  $lorry_hire,'company'=>$this->company]);
    }

    protected function initialize($companyCode)
    {
        $this->connectToTheDB($companyCode);

        $this->company = Company::where('company_code', $companyCode)->first();
    }
    
    protected function connectToTheDB($company){
        try {
            // Update the 'database' config in the 'dynamic' connection to point to the current company's database
            config(['database.connections.dynamic.database' => config('app.SERVER_PREFIX') .strtolower($company)]);
        
            // Reconnect to the updated database connection
            DB::connection('dynamic')->reconnect();
        
            DB::setDefaultConnection('dynamic');

        } catch (\Exception $e) {
            // Handle the exception here, for example:
            return response()->json(['message' => 'Failed to update dynamic database connection'], 500);
        }
        
    }
}
