<?php

namespace App\Http\Controllers\Z_Core;

use App\Http\Controllers\Controller;
use App\Models\Account\Account;
use App\Models\Apis\ApiCounter;
use App\Models\Booking\BookGr;
use App\Models\Booking\EwayGr;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\GoodsMove\Challan;
use App\Models\Account\Voucher;
use App\Models\Account\VoucherDesc;
use App\Models\Bill\Bill;
use App\Models\FleetOwner\LorryHire;
use App\Models\Inventory\Purchase;
use App\Models\Master\Employee;
use App\Models\Master\Party;
use App\Models\Master\Vehicle;
use App\Models\Z_Core\EditLog;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;

class DashboardController extends Controller
{
    public function index(Request $request)
    {
        // Get total records for today
        $totalRecordsToday = BookGr::whereDate('gr_date', today())->where('fin_year',$request->current_year)->count();

        // Get total records for the current month
        $totalRecordsThisMonth = BookGr::whereMonth('gr_date', now())->where('fin_year',$request->current_year)->count();

        $totalRecordsLastMonth = BookGr::whereMonth('gr_date', now()->subMonth())->where('fin_year',$request->current_year)->count();


        $totalUpdatesToday = EditLog::whereDate('created_at', today())->where('operation', 'updated')->where('fin_year',$request->current_year)->count();
        $totalDeletesToday = EditLog::whereDate('created_at', today())->where('operation', 'deleted')->where('fin_year',$request->current_year)->count();

        $pendingBillsGr = BookGr::where('pay_mode','TBB')->where('freight_bill',0)->orWhere('way_exp_bill',0)->where('fin_year',$request->current_year)->count();
        $pendingBillsLs = LoadingSlip::where('bill_status',0)->where('fin_year',$request->current_year)->count();



        // I WANT THEM FOR DISTINCT API_NAMES
        $api_data = ApiCounter::select(
            'api_name',
            \DB::raw('SUM(CASE WHEN DATE(date) = CURDATE() THEN api_count ELSE 0 END) as today'),
            \DB::raw('SUM(api_count) as total')
        )
        ->groupBy('api_name')
        ->get();

        // Your other logic for the index method...

        return response()->json([
            'totalRecordsToday' => $totalRecordsToday,
            'totalRecordsThisMonth' => $totalRecordsThisMonth,
            'totalRecordsLastMonth' => $totalRecordsLastMonth,
            'totalUpdatesToday' => $totalUpdatesToday,
            'totalDeletesToday' => $totalDeletesToday,
            'pendingBillsGr' => $pendingBillsGr,
            'pendingBillsLs' => $pendingBillsLs,
            'api_data' => $api_data,


        ]);
    }

    public function get_eway_reminders(Request $request)
    {
        $days = $request->days;
        $startDate = Carbon::now()->startOfDay(); 
        $endDate = Carbon::now()->addDays($days-1)->endOfDay(); 
    
       
    
        $eway = EwayGr::select('id','gr_id','gr_number','eway_number','eway_exp_date')
            ->where('eway_exp_date', '<=',$endDate)
            ->with(['bookGr:id,gr_number,truck_status'])
             ->whereHas('bookGr', function ($query) {
                $query->where('truck_status', '!=', 'UNLOADED');
             })
            ->where('is_delivered', 0)
            ->where('fin_year', $request->current_year)
            ->get();
        
        //dd($eway);    
   
        // Add `field_difference` to each record
        foreach ($eway as $key => $value) {
            $now = Carbon::today();
            $ewayDate = Carbon::parse($value->eway_exp_date)->startOfDay();
            $value->field_difference = $now->diffInDays($ewayDate, false);
            // $value->field_difference = Carbon::now()->diffInDays(Carbon::parse($value->eway_exp_date), false);
        }
    
        return response()->json($eway);
    }


    public function get_gr_reminders(Request $request)
    {
        $grlr = $request->grlr;
        $days = $request->days;

        $today = Carbon::now()->toDateString();
        $yesterday = Carbon::yesterday()->toDateString();

        $dateMinusDays = Carbon::now()->subDays($days)->toDateString();

        if ($grlr == 'Gr Number') {
            $query = BookGr::select('id', 'gr_number as agnst_no', 'gr_date as agnst_date', 'source_id', 'destination_id', 'vehicle_number','total_material_weight')
                ->with(['source:id,station_name', 'destination:id,station_name','gr_desc:id,gr_id,pkg,chg_qty,rate','challan_desc:id,challan_id,challan_number,gr_id,gr_number','challan_desc.challan:id,challan_number,lorry_hire_from,lh_rate,lh_total_wt','challan_desc.challan.lorry_hire_from:id,account_name','challan_desc.challan.voucher_desc'])
                ->whereBetween('gr_date', [$dateMinusDays, $today]);
        } elseif ($grlr == 'Loading Number') {
            $query = LoadingSlip::select('id', 'ls_number as agnst_no', 'ls_date as agnst_date', 'source_id', 'destination_id', 'vehicle_number')
                ->with(['source:id,station_name', 'destination:id,station_name', 'lorry_hire_desc:id,lorry_hire_id,lh_number,against_id,against,against_number'])
                ->whereBetween('ls_date', [$dateMinusDays, $today]);
        } elseif ($grlr == 'Challan Number') {
            $query = Challan::select('id', 'challan_number as agnst_no', 'challan_date as agnst_date', 'source_id', 'destination_id', 'vehicle_number','lorry_hire_from')
                ->with(['source:id,station_name', 'destination:id,station_name','lorry_hire_from:id,account_name'])
                ->whereBetween('challan_date', [$dateMinusDays, $today]);
        } else {
            return response()->json(['error' => 'Invalid grlr value'], 400);
        }

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        $reminders = $query->orderBy('id','desc')->get();

        return response()->json($reminders);
    }
    

    public function get_voucher_reminders(Request $request)
    {
        $voucherselect = $request->voucherselect;
        $allowedTillDate = Carbon::now()->toDateString();

        $days = $request->days;

        $dateMinusDays = Carbon::now()->subDays($days)->toDateString();

        $today = Carbon::now()->toDateString();
        $yesterday = Carbon::yesterday()->toDateString();

        if ($voucherselect == 'Receipt Voucher') {
            $query = Voucher::select('id', 'voucher_number', 'voucher_date','bank_date', 'total_dr_amount')
                    ->with(['voucher_acc_details' => function($query) {
                        $query->where('nature', 'cr');
                    }, 'voucher_acc_details.account:id,account_name'])
                ->where('voucher_name', 'Receipt Voucher')
                ->whereBetween('voucher_date', [$dateMinusDays, $today]);
        } elseif ($voucherselect == 'Payment Voucher') {
            $query = Voucher::select('id',  'voucher_number', 'voucher_date','bank_date', 'total_dr_amount')
            ->with(['voucher_acc_details' => function($query) {
                $query->where('nature', 'dr');
            },'voucher_acc_details.account:id,account_name'])
                ->where('voucher_name', 'Payment Voucher')
                ->whereBetween('voucher_date', [$dateMinusDays, $today]);
        }elseif ($voucherselect == 'Journal') {
            $query = Voucher::select('id', 'voucher_number', 'voucher_date','bank_date', 'total_dr_amount')
            ->with(['voucher_acc_details' => function($query) {
                $query->where('nature', 'dr');
            },'voucher_acc_details.account:id,account_name'])
                ->where('voucher_name', 'Journal')
                ->whereBetween('voucher_date', [$dateMinusDays, $today]);
        }elseif ($voucherselect == 'Contra') {
            $query = Voucher::select('id','voucher_number', 'voucher_date','bank_date', 'total_dr_amount')
            ->with(['voucher_acc_details' => function($query) {
                $query->where('nature', 'dr');
            },'voucher_acc_details.account:id,account_name'])
                ->where('voucher_name', 'Contra')
                ->whereBetween('voucher_date', [$dateMinusDays, $today]);
        }else {
            return response()->json(['error' => 'Invalid grlr value'], 400);
        }

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        // dd($query);
        $reminders = $query->orderBy('voucher_date','desc')->get();


        return response()->json($reminders);
    }


    public function autocomplete_for_merge_against(Request $request){
        // Initialize the query builder
        $type_of_merge = $request->type_of_merge;
        $searchVariable = $request->searchVariable;

        if($type_of_merge=='Party'){
            $againsts = Party::select('id','party_name as against_name','account_id')->where('party_name', 'like', '%'. $searchVariable . '%');
        }
        elseif ($type_of_merge=='Driver') {
            $againsts = Employee::select('id','emp_name as against_name','account_id')->where('category','Driver')->where('emp_name', 'like', '%'. $searchVariable . '%');
        }
        elseif ($type_of_merge=='Account') {
            $againsts = Account::select('id','account_name as against_name')->where('account_name', 'like', '%'. $searchVariable . '%');
        }
        elseif ($type_of_merge=='Vehicle') {
            $againsts = Vehicle::select('id','vehicle_number as against_name','account_id')->where('vehicle_number', 'like', '%'. $searchVariable . '%');
        }

    
        // Limit the number of results
        $limit = 25;

        $againsts->take($limit)->orderBy('id', 'desc');

        // Retrieve the filtered 
        $againsts = $againsts->get();
        return response(['againsts' => $againsts, 'draw' => $request->draw]);

    }

    public function merge_data_upload(Request $request)
    {
        $type_of_merge = $request->type_of_merge;

        foreach ($request->merged_list as $merged_item) {
            $replaced_data = $merged_item['replaced_data'];
            $kept_data = $merged_item['kept_data'];
            $changed_name = $merged_item['changed_name'] ?? $kept_data['against_name'];

            if ($type_of_merge == 'Party') {
                $replaced_party_id = $replaced_data['id'];
                $kept_party_id = $kept_data['id'];

                $replaced_account_id = $replaced_data['account_id'];
                $kept_account_id = $kept_data['account_id'];

                // Generate and execute update SQL for Party references
                $updatePartySQLs = DB::select("
                    SELECT CONCAT('UPDATE ', TABLE_NAME, ' SET ', COLUMN_NAME, ' = ', :kept_party_id, 
                        ' WHERE ', COLUMN_NAME, ' = ', :replaced_party_id, ';') AS sql_stmt
                    FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
                    WHERE REFERENCED_TABLE_NAME = 'parties'
                    AND REFERENCED_COLUMN_NAME = 'id'
                    AND TABLE_SCHEMA = DATABASE()
                ", ['kept_party_id' => $kept_party_id, 'replaced_party_id' => $replaced_party_id]);

                // Execute the generated SQL statements
                foreach ($updatePartySQLs as $sql) {
                    DB::statement($sql->sql_stmt);
                }

                // Generate and execute update SQL for Account references
                $updateAccountSQLs = DB::select("
                    SELECT CONCAT('UPDATE ', TABLE_NAME, ' SET ', COLUMN_NAME, ' = ', :kept_account_id, 
                        ' WHERE ', COLUMN_NAME, ' = ', :replaced_account_id, ';') AS sql_stmt
                    FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
                    WHERE REFERENCED_TABLE_NAME = 'accounts'
                    AND REFERENCED_COLUMN_NAME = 'id'
                    AND TABLE_SCHEMA = DATABASE()
                ", ['kept_account_id' => $kept_account_id, 'replaced_account_id' => $replaced_account_id]);

                // Execute the generated SQL statements
                foreach ($updateAccountSQLs as $sql) {
                    DB::statement($sql->sql_stmt);
                }

                // Update kept party details
                $keptParty = Party::find($kept_party_id);
                $keptParty->party_name = $changed_name;
                $keptParty->save();

                $keptAccount = Account::find($kept_account_id);
                $keptAccount->account_name = $changed_name;
                $keptAccount->save();

                $bookgr = BookGr::where('consignor_id',$kept_party_id)->update(['consignor_name'=>$changed_name]);
                $bookgr = BookGr::where('consignee_id',$kept_party_id)->update(['consignee_name'=>$changed_name]);

                // Delete the replaced records
                Party::destroy($replaced_party_id);
                Account::destroy($replaced_account_id);
            }
            elseif ($type_of_merge == 'Vehicle') {
                $replaced_vehicle_id = $replaced_data['id'];
                $kept_vehicle_id = $kept_data['id'];

                $replaced_account_id = $replaced_data['account_id'];
                $kept_account_id = $kept_data['account_id'];

                // Generate and execute update SQL for Party references
                $updatePartySQLs = DB::select("
                    SELECT CONCAT('UPDATE ', TABLE_NAME, ' SET ', COLUMN_NAME, ' = ', :kept_vehicle_id, 
                        ' WHERE ', COLUMN_NAME, ' = ', :replaced_vehicle_id, ';') AS sql_stmt
                    FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
                    WHERE REFERENCED_TABLE_NAME = 'vehicles'
                    AND REFERENCED_COLUMN_NAME = 'id'
                    AND TABLE_SCHEMA = DATABASE()
                ", ['kept_vehicle_id' => $kept_vehicle_id, 'replaced_vehicle_id' => $replaced_vehicle_id]);

                // Execute the generated SQL statements
                foreach ($updatePartySQLs as $sql) {
                    DB::statement($sql->sql_stmt);
                }

                // Generate and execute update SQL for Account references
                $updateAccountSQLs = DB::select("
                    SELECT CONCAT('UPDATE ', TABLE_NAME, ' SET ', COLUMN_NAME, ' = ', :kept_account_id, 
                        ' WHERE ', COLUMN_NAME, ' = ', :replaced_account_id, ';') AS sql_stmt
                    FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
                    WHERE REFERENCED_TABLE_NAME = 'accounts'
                    AND REFERENCED_COLUMN_NAME = 'id'
                    AND TABLE_SCHEMA = DATABASE()
                ", ['kept_account_id' => $kept_account_id, 'replaced_account_id' => $replaced_account_id]);

                // Execute the generated SQL statements
                foreach ($updateAccountSQLs as $sql) {
                    DB::statement($sql->sql_stmt);
                }

                // Update kept party details
                $keptParty = Vehicle::find($kept_vehicle_id);
                $keptParty->vehicle_number = $changed_name;
                $keptParty->save();

                $keptAccount = Account::find($kept_account_id);
                $keptAccount->account_name = $changed_name;
                $keptAccount->save();

                $bookgr = BookGr::where('vehicle_id',$kept_vehicle_id)->update(['vehicle_number'=>$changed_name]);
                $challan = Challan::where('vehicle_id',$kept_vehicle_id)->update(['vehicle_number'=>$changed_name]);
                $ls = LoadingSlip::where('vehicle_id',$kept_vehicle_id)->update(['vehicle_number'=>$changed_name]);
                $lh = LorryHire::where('vehicle_id',$kept_vehicle_id)->update(['vehicle_number'=>$changed_name]);

                $voucher_desc = VoucherDesc::where('against_id',$replaced_vehicle_id)->where('against','VEHICLE_COST_CENTER')->update(['against_id'=>$kept_vehicle_id,'against_number'=>$changed_name]);

                // Delete the replaced records
                Vehicle::destroy($replaced_vehicle_id);
                Account::destroy($replaced_account_id);
            }
            elseif ($type_of_merge == 'Driver') {
                $replaced_driver_id = $replaced_data['id'];
                $kept_driver_id = $kept_data['id'];

                $replaced_account_id = $replaced_data['account_id'];
                $kept_account_id = $kept_data['account_id'];

                // Generate and execute update SQL for Party references
                $updatePartySQLs = DB::select("
                    SELECT CONCAT('UPDATE ', TABLE_NAME, ' SET ', COLUMN_NAME, ' = ', :kept_driver_id, 
                        ' WHERE ', COLUMN_NAME, ' = ', :replaced_driver_id, ';') AS sql_stmt
                    FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
                    WHERE REFERENCED_TABLE_NAME = 'employees'
                    AND REFERENCED_COLUMN_NAME = 'id'
                    AND TABLE_SCHEMA = DATABASE()
                ", ['kept_driver_id' => $kept_driver_id, 'replaced_driver_id' => $replaced_driver_id]);

                // Execute the generated SQL statements
                foreach ($updatePartySQLs as $sql) {
                    DB::statement($sql->sql_stmt);
                }

                // Generate and execute update SQL for Account references
                $updateAccountSQLs = DB::select("
                    SELECT CONCAT('UPDATE ', TABLE_NAME, ' SET ', COLUMN_NAME, ' = ', :kept_account_id, 
                        ' WHERE ', COLUMN_NAME, ' = ', :replaced_account_id, ';') AS sql_stmt
                    FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
                    WHERE REFERENCED_TABLE_NAME = 'accounts'
                    AND REFERENCED_COLUMN_NAME = 'id'
                    AND TABLE_SCHEMA = DATABASE()
                ", ['kept_account_id' => $kept_account_id, 'replaced_account_id' => $replaced_account_id]);

                // Execute the generated SQL statements
                foreach ($updateAccountSQLs as $sql) {
                    DB::statement($sql->sql_stmt);
                }

                // Update kept party details
                $keptParty = Employee::find($kept_driver_id);
                $keptParty->emp_name = $changed_name;
                $keptParty->save();

                $keptAccount = Account::find($kept_account_id);
                $keptAccount->account_name = $changed_name;
                $keptAccount->save();

                $bookgr = BookGr::where('driver_id',$kept_driver_id)->update(['driver_name'=>$changed_name]);

                // Delete the replaced records
                Employee::destroy($replaced_driver_id);
                Account::destroy($replaced_account_id);
            }
            elseif ($type_of_merge == 'Account') {
                $replaced_account_id = $replaced_data['id'];
                $kept_account_id = $kept_data['id'];

                // Generate and execute update SQL for Account references
                $updateAccountSQLs = DB::select("
                    SELECT CONCAT('UPDATE ', TABLE_NAME, ' SET ', COLUMN_NAME, ' = ', :kept_account_id, 
                        ' WHERE ', COLUMN_NAME, ' = ', :replaced_account_id, ';') AS sql_stmt
                    FROM INFORMATION_SCHEMA.KEY_COLUMN_USAGE
                    WHERE REFERENCED_TABLE_NAME = 'accounts'
                    AND REFERENCED_COLUMN_NAME = 'id'
                    AND TABLE_SCHEMA = DATABASE()
                ", ['kept_account_id' => $kept_account_id, 'replaced_account_id' => $replaced_account_id]);

                // Execute the generated SQL statements
                foreach ($updateAccountSQLs as $sql) {
                    DB::statement($sql->sql_stmt);
                }


                $keptAccount = Account::find($kept_account_id);
                $keptAccount->account_name = $changed_name;
                $keptAccount->save();

                Account::destroy($replaced_account_id);
            }

            
        }
    }

    public function get_pending_bill_reminders(Request $request){
        $grlr = $request->grlr;

        if ($grlr == 'GR') {
            $query = BookGr::select('acc_party_id', \DB::raw('COUNT(*) as document_count'), \DB::raw('SUM(grand_total) as document_amount'))
                ->with('acc_party:id,account_name')
                ->where('pay_mode', 'TBB')
                ->whereDoesntHave('bill_desc')
                ->groupBy('acc_party_id');
        } elseif ($grlr == 'Loading Slip') {
            $query = LoadingSlip::select('acc_party_id', \DB::raw('COUNT(*) as document_count'), \DB::raw('SUM(total_without_gst) as document_amount'))
                ->with('acc_party:id,account_name')
                ->where('bill_status', 0)
                ->whereDoesntHave('bill_desc')
                ->groupBy('acc_party_id');
        } else {
            return response()->json([
                'message' => 'Invalid grlr parameter.',
            ], 400);
        }

        // Execute query and format response
        $results = $query->get()->map(function ($item) {
            return [
                'party_id' => $item->acc_party_id,
                'party_name' => $item->acc_party->account_name ?? 'Unknown Party',
                'document_count' => $item->document_count,
                'document_amount' => $item->document_amount,
            ];
        })->sortBy('party_name')->values();

        return response()->json($results);
}
public function get_pending_pod_reminders(Request $request){
    $grlr = $request->grlr;

    if($grlr == 'GR') {
        $query = BookGr::select(
            'acc_party_id',
            \DB::raw('COUNT(*) as document_count'),
            \DB::raw('SUM(grand_total) as document_amount'),
            \DB::raw("SUM(CASE WHEN DATEDIFF(CURDATE(), gr_date) BETWEEN 0 AND 7 THEN 1 ELSE 0 END) as count_0_7"),
            \DB::raw("SUM(CASE WHEN DATEDIFF(CURDATE(), gr_date) BETWEEN 8 AND 15 THEN 1 ELSE 0 END) as count_8_15"),
            \DB::raw("SUM(CASE WHEN DATEDIFF(CURDATE(), gr_date) BETWEEN 16 AND 30 THEN 1 ELSE 0 END) as count_16_30"),
            \DB::raw("SUM(CASE WHEN DATEDIFF(CURDATE(), gr_date) > 30 THEN 1 ELSE 0 END) as count_30_plus")
        )
            ->where('fin_year', $request->current_year)
            ->whereDoesntHave('pod_desc')
            ->groupBy('acc_party_id');
    } elseif ($grlr == 'Loading Slip') {
        $query = LoadingSlip::select(
            'acc_party_id',
            \DB::raw('COUNT(*) as document_count'),
            \DB::raw('SUM(total_without_gst) as document_amount'),
            \DB::raw("SUM(CASE WHEN DATEDIFF(CURDATE(), ls_date) BETWEEN 0 AND 7 THEN 1 ELSE 0 END) as count_0_7"),
            \DB::raw("SUM(CASE WHEN DATEDIFF(CURDATE(), ls_date) BETWEEN 8 AND 15 THEN 1 ELSE 0 END) as count_8_15"),
            \DB::raw("SUM(CASE WHEN DATEDIFF(CURDATE(), ls_date) BETWEEN 16 AND 30 THEN 1 ELSE 0 END) as count_16_30"),
            \DB::raw("SUM(CASE WHEN DATEDIFF(CURDATE(), ls_date) > 30 THEN 1 ELSE 0 END) as count_30_plus")
        )
            ->where('fin_year', $request->current_year)
            ->whereNull('pod_pic')
            ->groupBy('acc_party_id');
    } else {
        return response()->json([
            'message' => 'Invalid grlr parameter.',
        ], 400);
    }
    
    // Execute query and format response
    $results = $query->get()->map(function ($item) {
        return [
            'party_id' => $item->acc_party_id,
            'party_name' => $item->acc_party->account_name ?? 'Unknown Party',
            'document_count' => $item->document_count,
            'document_amount' => $item->document_amount,
            '0_7' => $item->count_0_7,
            '8_15' => $item->count_8_15,
            '16_30' => $item->count_16_30,
            '30_plus' => $item->count_30_plus,
        ];
    })->sortBy('party_name')->values();
    
    return response()->json($results);
}

public function get_outstanding_bill_reminders(Request $request){
        $selectionType = $request->grlr;
    
        // Initialize result storage
        $results = [];
    
        // Determine query type
        if ($selectionType === 'Lorry Hire') {
            $lorryHireQuery = LorryHire::select('id', 'lorry_hire_from', 'balance', 'lh_number', 'lh_date')
                ->where('paid_status', false)
                ->with('lorry_hire_from:id,account_name');
    
            $challanQuery = Challan::select('id', 'lorry_hire_from', 'lh_balance as balance', 'challan_number', 'challan_date as date')
                ->where('paid_status', false)
                ->with('lorry_hire_from:id,account_name');
    
            // Combine queries
            $query = $lorryHireQuery->unionAll($challanQuery);
        } elseif ($selectionType === 'Purchase') {
            $query = Purchase::select('id', 'acc_party_id', 'total_without_rounding as balance', 'bill_number', 'bill_date as date')
                ->where('bill_name', '!=', 'Stock Issue')
                ->where('paid_status', false)
                ->with('acc_party:id,account_name');
    
        } elseif ($selectionType === 'GR' || $selectionType === 'Loading Slip' || $selectionType === 'Supplementary') {
            $query = Bill::select('id', 'acc_party_id', 'grand_total_amount as balance', 'bill_number', 'bill_date')
                ->with('acc_party:id,account_name')
                ->where('paid_status', false)
                ->where('selection_type', $selectionType);
        } else {
            return response()->json(['message' => 'Invalid selection type.'], 400);
        }
    
        // Fetch and process results
        $data = $query->get();
    
        foreach ($data as $item) {
            $balance = $this->check_for_balance($item, $item->balance, [$selectionType]);
    
            if ($balance > 1) {
                if($selectionType === 'Lorry Hire' || $selectionType === 'Purchase'){

                    $copy = json_decode(json_encode($item), true);
                    $partyId = $copy['lorry_hire_from']['id'];

                    $partyName = $copy['lorry_hire_from']['account_name'] ?? 'Unknown Party';

                }else{
                    $partyId = $item->acc_party_id;
                    $partyName = $item->acc_party->account_name ?? 'Unknown Party';
                }
                // Accumulate balances and counts per party
                if (!isset($results[$partyId])) {
                    $results[$partyId] = [
                        'party_id' => $partyId,
                        'party_name' => $partyName,
                        'document_amount' => 0,
                        'document_count' => 0,
                    ];
                }
    
                $results[$partyId]['document_amount'] += $balance;
                $results[$partyId]['document_count']++;
            }
        }
    
        $sortedResults = collect($results)->sortBy('party_name')->values();
        
            // Format response
            return response()->json($sortedResults);
    }

    public function get_rate_difference_reminders(Request $request){
        $grlr = $request->grlr;
        $days = $request->days;
        $from_date = $request->from_date;
        $to_date = $request->to_date;
        $percentage = $request->percentage;


        $query = BookGr::select('id', 'gr_number','total_without_gst')
            ->with(['challan_desc:id,challan_id,challan_number,gr_id,gr_number','challan_desc.challan:id,challan_number,lorry_hire_from,lh_net_freight,lh_rate,lh_total_wt'])
            ->whereBetween('gr_date', [$from_date, $to_date])
            ->where('total_without_gst','>',0)
            ->whereHas('challan_desc');
        

        $user_branch_ids = $request->user_branch_ids;
        if ( $user_branch_ids ) {
            $query->whereIn('branch_id', $user_branch_ids );
        }

        $reminders = $query->orderBy('id','desc')->get();


        $formattedReminders = $reminders->map(function ($rem) use ($percentage) {
            $percent = count($rem->challan_desc) ? round((($rem->total_without_gst - $rem->challan_desc[0]->challan->lh_net_freight) / $rem->total_without_gst) * 100, 2) : 0;

            if($percent <= $percentage){
                return [
                    'id' => $rem->id,
                    'gr_number' => $rem->gr_number,
                    'freight' => $rem->total_without_gst,
                    'challan_id' => count($rem->challan_desc) ? $rem->challan_desc[0]->challan->id : null, // Safely access related data
                    'challan_number' => count($rem->challan_desc) ? $rem->challan_desc[0]->challan->challan_number : null, 
                    'lh_net_freight' =>count($rem->challan_desc) ? $rem->challan_desc[0]->challan->lh_net_freight : null,
                    'percentage' => $percent,
                ];
            }
            
        });

        

        return response()->json($formattedReminders);
    }


    protected function check_for_balance($against,$default_balance,$selection){
        $filteredVoucherDesc = $against->voucher_desc->whereIn('against', $selection);
        
        if ($filteredVoucherDesc->isEmpty()) {
            // No voucher_desc, return grand_total_amount as balance
            return $default_balance;
        }
        
        $recievedAmtSum = $filteredVoucherDesc->sum('receipt_amt');
        $deductionSum = $filteredVoucherDesc->sum('deduction');
        
        $balance = $default_balance - $recievedAmtSum ;
        return $balance;
}



}
