<?php

namespace App\Http\Controllers\Z_Core;

use App\Http\Controllers\Controller;
use App\Http\Traits\CommonTrait;
use App\Models\Account\Account;
use App\Models\Account\Balances;
use App\Models\Account\Group;
use App\Models\Account\Voucher;
use App\Models\Account\VoucherAccDetails;
use App\Models\Account\VoucherDesc;
use App\Models\Booking\BookGr;
use App\Models\Booking\EwayGr;
use App\Models\Booking\GrDesc;
use App\Models\Booking\PackagePosition;
use App\Models\Business\Company;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\FleetOwner\LoadingSlip;
use App\Models\GoodsMove\Challan;
use App\Models\Inventory\Product;
use App\Models\Inventory\Purchase;
use App\Models\Inventory\PurchaseDesc;
use App\Models\Master\Branch;
use App\Models\Master\Employee;
use App\Models\Master\Material;
use App\Models\Master\Party;
use App\Models\Master\PriceList;
use App\Models\Master\Station;
use App\Models\Master\Vehicle;
use App\Models\Master\VehicleType;
use App\Models\Settings\GeneralSettings;
use Carbon\Carbon;
use DateTime;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Date;
use Illuminate\Support\Facades\DB;

class ExcelUploadController extends Controller
{
    use CommonTrait;

    public function upload_fastag(Request $request){

        $required_fields = ['date','vehicle_number','amount'];



        $FASTAG_EXPENSE = $this->get_acc_id('FASTAG EXPENSE');
        $PAYMENT_FROM = $this->get_acc_id('KOTAK MAHINDERA FAST TAG');

        if(!$FASTAG_EXPENSE || !$PAYMENT_FROM){
            return response('The Mandatory FASTAG or Payment Account not found',500);
        }

        $excelData = $request->excelData;

        // Check if the first row of the Excel data array contains all the required fields
        foreach ($required_fields as $field) {
            // Check if the required field is missing in the first row
            if (!in_array($field, $excelData[0])) {
                // Handle the case where a required field is missing
                return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.'],500);
            }
        }


        $branch_to_use = 'JHR';

        $check_branch = Branch::where('branch_code', $branch_to_use)->first();
        if(!$check_branch){
            return response(['message'=>'Contact Admin To Set A Branch To import data']);
        }
        
        $fin_yr_wise = GeneralSettings::where('setting_name','voucher_number_fin_yr_wise')->where('value',1)->exists();

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

                $rand = rand();
                for ($i = 1; $i < count($excelData); $i++) {
                    $item = $excelData[$i];

                    

                    $required_err = false;
                    foreach ($required_fields as $field) {
                        // Check if the required field is missing in the first row
                        
                        if (!isset($item[$field])) {
                            // Handle the case where a required field is missing
                           $required_err = true;
                           // return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.', 'item'=>$item],500);
                        }
                    }
                    
                    if($required_err==true || !$item['amount']){
                        continue;
                    }


                    $max = Voucher::where('branch_id',$check_branch->id)->where('voucher_type', 'SYSTEM')->where('voucher_name','Journal')->where('fin_yr_wise',$fin_yr_wise)->max('max_number');
                
                    if($fin_yr_wise){
                        $year_stamp = $this->get_short_name($request->current_year);
                        $voucher_number = $check_branch->branch_code.'/'.$year_stamp.'/'.sprintf('%06d', $max+1);
                    }
                    else{
                        $voucher_number = $check_branch->branch_code.'-'.sprintf('%06d', $max+1);
                    }

                    
                    // SAVING A new Voucher
                    $voucher = new Voucher;
                    $voucher->branch_id = $check_branch['id'];
                    $voucher->voucher_type = 'SYSTEM';
                    $voucher->fin_yr_wise = $fin_yr_wise;
                    $voucher->max_number = $max+1;
                    $voucher->voucher_name = 'Journal';

                    $voucher->voucher_number = $voucher_number;
                    
                   // $item['date'] COMES IN THIS FORMAT 14/05/2024 10:45:25

                   $voucher->voucher_date = DateTime::createFromFormat('d/m/Y H:i:s', $item['date'])->format('Y-m-d');

                   $voucher->bank_date = DateTime::createFromFormat('d/m/Y H:i:s', $item['date'])->format('Y-m-d');

                    $voucher->voucher_time = Carbon::now();

                    $desc = "";

                    if(isset($item['toll_id'])){
                        $desc.='Toll Id : '.$item['toll_id'];
                    }
                    if(isset($item['transaction_id'])){
                        $desc.='  Transaction Id : '.$item['transaction_id'];
                    }
                    if(isset($item['description'])){
                        $desc.='  Description: '.$item['description'];
                    }


                    $voucher->narration = $desc;
                    $voucher->total_dr_amount = $item['amount'];
                    $voucher->total_cr_amount = $item['amount'];
                    $voucher->fin_year = $request->current_year;
                    
                    $voucher->rndm =  $rand;
                    $voucher->created_by = $request->user->id;
                    $voucher->save(); 
                    // new Voucher SAVED

                        $cr_v_acc_detail = new VoucherAccDetails();
                        $cr_v_acc_detail->voucher_id = $voucher->id;
                        $cr_v_acc_detail->voucher_name = $voucher->voucher_name;
                        $cr_v_acc_detail->voucher_number = $voucher->voucher_number;
                        $cr_v_acc_detail->account_id = $PAYMENT_FROM;
                        $cr_v_acc_detail->nature = 'cr';
                        $cr_v_acc_detail->cr_amount = $item['amount'];
                        $cr_v_acc_detail->fin_year = $voucher->fin_year;
                        $cr_v_acc_detail->rndm =  $rand;
                        $cr_v_acc_detail->save();

                        $v_acc_detail = new VoucherAccDetails();
                        $v_acc_detail->voucher_id = $voucher->id;
                        $v_acc_detail->voucher_name = $voucher->voucher_name;
                        $v_acc_detail->voucher_number = $voucher->voucher_number;
                        $v_acc_detail->account_id = $FASTAG_EXPENSE;
                        $v_acc_detail->nature = 'dr';
                        $v_acc_detail->against = 'VEHICLE_COST_CENTER';
                        $v_acc_detail->dr_amount = $item['amount'];
                        $v_acc_detail->fin_year = $voucher->fin_year;
                        $v_acc_detail->rndm =  $rand;
                        $v_acc_detail->save();




                        if ($v_acc_detail['against']) {
                                $vou_desc = new VoucherDesc();
                                $vou_desc->voucher_id = $voucher->id;
                                $vou_desc->voucher_name = $voucher->voucher_name;
                                $vou_desc->voucher_number = $voucher->voucher_number;
                                $vou_desc->v_acc_det_id = $v_acc_detail->id;
                                $vou_desc->against = 'VEHICLE_COST_CENTER';


                                $vehicle = Vehicle::select('id','vehicle_number')->where('vehicle_number', $item['vehicle_number'])->first();
                              
                                if (!$vehicle) {

                                    // Handle the case where a required field is missing
                                    return response()->json(['status' => false, 'message' => 'Vehicle not exist  is missing.', 'item'=>$item],500);
                                }

                                $vou_desc->against_id = $vehicle['id'];
                                $vou_desc->against_number = $vehicle['vehicle_number'];
                                $vou_desc->received_amt = $item['amount'];
                                $vou_desc->receipt_amt = $item['amount'];
                                $vou_desc->fin_year = $voucher->fin_year;
                                $vou_desc->rndm =  $rand;
                                $vou_desc->save();
                                
                        }
                    
                }

            // TRANSACTIONS COMMITTED
            DB::commit();

                
            return response()->json(['success'=>true], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
        

        

        

    }

    public function upload_party(Request $request) {
        // REQUIRED FIELDS
        $required_fields = ['party_name','party_code'];
        
        // GETING EXEL DATA 
        $excelData = $request->excelData;
        // Check if the first row of the Excel data array contains all the required fields
        foreach ($required_fields as $field) {
            // Check if the required field is missing in the first row
            if (!in_array($field, $excelData[0])) {
                // Handle the case where a required field is missing
                return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.'],500);
            }
        }
        try{
            // TRANSACTION BEGINS
            DB::beginTransaction();  
            $rand = rand();
            for ($i = 1; $i < count($excelData); $i++) {

                if(!isset($excelData[$i]['party_name']) || !isset($excelData[$i]['party_code'])){
                    continue;
                }
                $value = $excelData[$i];
                $party_name = $value['party_name'];
                $party_code = $value['party_code'];
                $address = @$value['address'];
                $address2 = @$value['address2'];
                $city = @$value['city'];
                $state = @$value['state'];
                $mobile_number = @$value['mobile_number'];
                $party_type = $value['party_type'];
                $account_group = $value['account_group'];
                $gst = @$value['gst'];
                $pan_number = @$value['pan_no'];
                $contact_person = @$value['contact_person'];

                $tds_percent = @$value['tds_percent'];

                $CHEACK_IF_EXIST = Party::where('party_name',$party_name)->first();
                if($CHEACK_IF_EXIST){
                    //return response()->json(['status' => false, 'message' => 'Party Already Exist.'],500);
                    $party = Party::where('party_name',$party_name)->first();
                    $party->updated_by = $request->user->id;
                   // continue;
                }
                else{
                    $party = new Party();
                    $party->created_by = $request->user->id;
                }

                
                // SAVING A NEW PARTY
                $party->party_name = $party_name;
                $party->party_code = $party_code;
                $party->address1 = $address;
                $party->address2 = $address2;
                $party->city = $city;
                $party->state = $state;
                $party->mob_number = $mobile_number;
                $party->party_type = $party_type;
                $party->account_head = $account_group;
                $party->gst_number = $gst;
                $party->pan_number = $pan_number;

                $party->tds_percent = $tds_percent;
                $party->contact_person = $contact_person;

                $party->gst_paid_by = 'Consignor';



                $party->preset_dynamics = '{}';

                $party->unit = 'FTL';
                $party->pay_mode = 'TBB';

                $party->branch_id = null;

                if($account_group!=''){
                    $party->add_to_accounts = true;
                }
                // SAVE PARTY
                $party->save();

                if($account_group!='' && !$CHEACK_IF_EXIST){
                    // Saving New Account
                    $grp = Group::where('group_name', $account_group)->first();
                    $new_account = new Account();
                    $new_account->group_id = $grp->id;
                    $new_account->account_name = $party_name;
                    $new_account->account_type = 'Party';
                    $new_account->created_by = $request->user->id;
                    $new_account->save();
                    // New Account Saved

                     // Saving the Balances
                    $bal = new Balances();
                    $bal->account_id = $new_account->id;
                    $bal->opening_bal_dr = 0;
                    $bal->opening_bal_cr = 0;
                    $bal->opening_bal_date = Carbon::now();
                    $bal->fin_year = $request->current_year;
                    $bal->save();
                    // New Balances Saved

                    $party->account_id = $new_account->id;
                    $party->save(); 
                }
                
            }
            
            // TRANSACTIONS COMMITTED
            DB::commit();
            return response()->json(['success'=>true], 200); // RETURNED RESPONSE
            
        }catch(\Exception $e){
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function upload_truck(Request $request){
        // REQUIRED FIELDS
        $required_fields = ['vehicle_number'];

        // GETING EXEL DATA 
        $excelData = $request->excelData;
        // Check if the first row of the Excel data array contains all the required fields
        foreach ($required_fields as $field) {
            // Check if the required field is missing in the first row
            if (!in_array($field, $excelData[0])) {
                // Handle the case where a required field is missing
                return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.'],500);
            }
        }
        try{
            DB::beginTransaction();
            $rand = rand();
            for ($i = 1; $i < count($excelData); $i++) {
                $value = $excelData[$i];

                if(!isset($value['vehicle_number'])){
                    continue;
                }

                $vehicle_number = $value['vehicle_number'];
                // CHECK IF VEHICLE EXIST
                $CHECK_IF_EXIST = Vehicle::where('vehicle_number', $vehicle_number)->first();
                if($CHECK_IF_EXIST){
                    //return response()->json(['status' => false, 'message' => 'Vehicle Already Exist.'], 500);
                    $vehicle = Vehicle::where('vehicle_number', $vehicle_number)->first();
                    $vehicle->updated_by = $request->user->id;

                }
                else{
                    $vehicle = new Vehicle();
                    $vehicle->created_by = $request->user->id;
                    
                }

                $vehicle->vehicle_number = $value['vehicle_number'];
                $vehicle->self_market = @$value['self_market']==''?'SELF':$value['self_market'];
                $vehicle->vehicle_type = @$value['vehicle_type'];
 
                if($vehicle->vehicle_type!=''){
                    $check_vehicle_type = VehicleType::where('vehicle_type',$vehicle->vehicle_type)->exists();
                    if(!$check_vehicle_type){
                        $vehicle_type = new VehicleType();
                        $vehicle_type->vehicle_type = $value['vehicle_type'];
                        $vehicle_type->created_by = $request->user->id;
                        $vehicle_type->save();
                    }
                }


                $vehicle->chasis_number = @$value['chasis_number'];
                $vehicle->engine_number = @$value['engine_no'];

                if($value['owner_name']!=''){
                   $owner_name = @$value['owner_name'];
                   $owner_party = Party::select('id')->where('party_name',$owner_name)->first();

                   if(!$owner_party){
                    return response()->json(['status' => false, 'message' => $owner_name.' Owner Not Found.'], 500);
                   }
                   $vehicle->owner_id = $owner_party->id;
                }
                //$vehicle->pan_no = @$value['pan_no'];
                // Convert dates from DD-MM-YYYY to YYYY-MM-DD using Carbon
                $vehicle->permit_number = @$value['permit_no'];
                $vehicle->permit_exp_date = isset($value['permit_due_date']) ? Carbon::createFromFormat('d-m-Y', $value['permit_due_date'])->format('Y-m-d') : null;
                $vehicle->five_years_permit_number = @$value['five_year_permit_no'];
                $vehicle->five_years_permit_due_date = isset($value['five_year_permit_due_date']) ? Carbon::createFromFormat('d-m-Y', $value['five_year_permit_due_date'])->format('Y-m-d') : null;

                $vehicle->insurance = @$value['insurance_policy_number'];
                $vehicle->insurance_due_date = isset($value['insurance_due_date']) ? Carbon::createFromFormat('d-m-Y', $value['insurance_due_date'])->format('Y-m-d') : null;
                $vehicle->fitness_certificate_number = @$value['fitness_no'];
                $vehicle->fitness_due_date = isset($value['fitness_due_date']) ? Carbon::createFromFormat('d-m-Y', $value['fitness_due_date'])->format('Y-m-d') : null;

                $vehicle->road_tax = @$value['road_tax'];
                $vehicle->road_tax_due_date = isset($value['road_tax_date']) ? Carbon::createFromFormat('d-m-Y', $value['road_tax_date'])->format('Y-m-d') : null;
                
                $vehicle->reg_number = @$value['reg'];
                $vehicle->reg_due_date = isset($value['reg_date']) ? Carbon::createFromFormat('d-m-Y', $value['reg_date'])->format('Y-m-d') : null;
                
                $vehicle->pollution = @$value['pollution'];
                $vehicle->pollution_due_date = isset($value['pollution_date']) ? Carbon::createFromFormat('d-m-Y', $value['pollution_date'])->format('Y-m-d') : null;
                $vehicle->model = @$value['make_model'];
                $vehicle->length = @$value['length'];
                $vehicle->breadth = @$value['width'];
                $vehicle->height = @$value['height'];
                $vehicle->account_head = 'Fixed Assets';
                $vehicle->fuel_type = @$value['fuel_type']==''?'Diesel':$value['fuel_type'];

                $vehicle->created_by = $request->user->id;
                
                
                if($vehicle->account_head != '' && !$CHECK_IF_EXIST){
                    $grp = Group::where('group_name', $vehicle->account_head)->first();
                    $new_account = new Account();
                    $new_account->group_id = $grp->id;
                    $new_account->account_name = $vehicle->vehicle_number;
                    $new_account->account_type = 'Vehicle';
                    $new_account->created_by = $request->user->id;
                    $new_account->save();

                    $bal = new Balances();
                    $bal->account_id = $new_account->id;
                    $bal->opening_bal_dr = 0;
                    $bal->opening_bal_cr = 0;
                    $bal->opening_bal_date = Carbon::now();
                    $bal->fin_year = $request->current_year;
                    $bal->save();

                   $vehicle->account_id = $new_account->id;

                }
            
                $vehicle->save(); 

            }
            DB::commit();
            return response()->json(['success'=>true], 200); // RETURNED RESPONSE
        }catch(\Exception $e){
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }

    }

    public function upload_driver(Request $request){
        // REQUIRED FIELDS
        $required_fields = ['driver_name'];

        // GETING EXEL DATA 
        $excelData = $request->excelData;
        // Check if the first row of the Excel data array contains all the required fields
        foreach ($required_fields as $field) {
            // Check if the required field is missing in the first row
            if (!in_array($field, $excelData[0])) {
                // Handle the case where a required field is missing
                return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.'],500);
            }
        }
        try{
            DB::beginTransaction();
            $rand = rand();

            $company_code = Company::select('company_code')->find($request->company['id'])['company_code'];
            $pre = 'D';
            $cc = $company_code;
            $max = Employee::where('category','Driver')->max('max_number');

            for ($i = 1; $i < count($excelData); $i++) {
                $value = $excelData[$i];

                if(!isset($value['driver_name'])){
                    continue;
                }

                $driver_name = $value['driver_name'];

                if($driver_name){
                // CHECK IF VEHICLE EXIST
                $CHECK_IF_EXIST = Employee::where('emp_name', $driver_name)->first();
                if($CHECK_IF_EXIST){
                    //return response()->json(['status' => false, 'message' => 'Driver Already Exist.'], 500);
                    $employee = Employee::where('emp_name', $driver_name)->first();
                    $employee->updated_by = $request->user->id;
                }
                else{
                    $employee = new Employee();
                    $employee->created_by = $request->user->id;
                    $emp_id = $cc.'-'.$pre.'-'.sprintf('%05d', $max+1);
                    $max++;

                    // Set the values from the request to the employee object
                    $employee->emp_id =  $emp_id ;
                    $employee->max_number = $max;
                }

                $employee->emp_name =  $value['driver_name'];
                $employee->category = 'Driver';


                if(@$value['branch_code']){
                    $branch_id = Branch::select('id')->where("branch_code",@$value['branch_code'])->first()['id'];
                    $employee->branch_id = $branch_id;
                }
                else{
                    $employee->branch_id = null;
                }

                $employee->designation = 'Driver';
                $employee->dob = isset($value['dob']) ? Carbon::createFromFormat('d-m-Y', $value['dob'])->format('Y-m-d') : null;
                $employee->joining_date = isset($value['joining_date']) ? Carbon::createFromFormat('d-m-Y', $value['joining_date'])->format('Y-m-d') : null;
                $employee->resignation_date = isset($value['resignation_date']) ? Carbon::createFromFormat('d-m-Y', $value['resignation_date'])->format('Y-m-d') : null;

                $employee->address = @$value['address'];
                $employee->contact_number = @$value['contact_no'];
                $employee->father_name = @$value['f_name'];
                $employee->father_contact = @$value['f_contact_no'];
                $employee->aadhaar_number = @$value['aadhaar'];

                $employee->license_issue_date = isset($value['license_issue_date']) ? Carbon::createFromFormat('d-m-Y', $value['license_issue_date'])->format('Y-m-d') : null;
                $employee->license_valid_till = isset($value['license_valid_till']) ? Carbon::createFromFormat('d-m-Y', $value['license_valid_till'])->format('Y-m-d') : null;
                $employee->license_verifi_date = isset($value['license_verifi_date']) ? Carbon::createFromFormat('d-m-Y', $value['license_verifi_date'])->format('Y-m-d') : null;
                $employee->license_verifi_status = @$value['license_verifi_status'];
                $employee->age = @$value['age'];
                $employee->religion = @$value['religion'];


                $employee->black_list = false;

                if(@$value['resignation_date']){
                    $employee->status = 'Inactive';
                }else{
                    $employee->status = 'Active'; 
                }

                $employee->license_number = @$value['license_no'];
            
                if(!$CHECK_IF_EXIST){
                    $grp = Group::where('group_name', 'Loans & Advances (Asset)')->first();

                    $new_account = new Account();
                    $new_account->group_id = $grp->id;
                    $new_account->account_name = $value['driver_name'];
                    $new_account->account_type = 'Driver';
                    $new_account->created_by = $request->user->id;
                    $new_account->save();
                    // New Account Saved
                
                    
                    // Saving the Balances
                    $bal = new Balances();
                    $bal->account_id = $new_account->id;
                    $bal->opening_bal_dr = 0;
                    $bal->opening_bal_cr = 0;
                    $bal->opening_bal_date = Carbon::now();
                    $bal->fin_year = $request->current_year;
                    $bal->save();
                    // New Balances Saved
    
                    $employee->account_id = $new_account->id;   

                }
               
                $employee->save(); 
                // NEW Employee SAVED
            }


            }
            DB::commit();
            return response()->json(['success'=>true], 200); // RETURNED RESPONSE
        }catch(\Exception $e){
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }

    }
    
    public function upload_station(Request $request){
        // REQUIRED FIELDS
        $required_fields = ['station_name'];

        // GETING EXEL DATA 
        $excelData = $request->excelData;
        // Check if the first row of the Excel data array contains all the required fields
        foreach ($required_fields as $field) {
            // Check if the required field is missing in the first row
            if (!in_array($field, $excelData[0])) {
                // Handle the case where a required field is missing
                return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.'],500);
            }
        }
        try{
            DB::beginTransaction();
            $rand = rand();

            for ($i = 1; $i < count($excelData); $i++) {
                $value = $excelData[$i];
                $station_name = $value['station_name'];

                if($station_name){
                    // CHECK IF VEHICLE EXIST
                    $CHECK_IF_EXIST = Station::where('station_name', $station_name)->first();
                    if($CHECK_IF_EXIST){
                        //return response()->json(['status' => false, 'message' => 'Driver Already Exist.'], 500);
                        $employee = Station::where('station_name', $station_name)->first();
                        $employee->updated_by = $request->user->id;
                    }
                    else{
                        // SAVING A NEW STATION
                        $station = new Station();
                        $station->created_by = $request->user->id;
                    }


                        $station->station_name = $value['station_name'];
                        $station->state = @$value['state'];
                        $station->district = @$value['district'];
                        $station->pincode = @$value['pincode'];

                        
                        $station->save(); 
                        // NEW STATION SAVED
                }


            }
            DB::commit();
            return response()->json(['success'=>true], 200); // RETURNED RESPONSE
        }catch(\Exception $e){
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }

    }

    public function upload_voucher(Request $request){

        $required_fields = ['voucher_type', 'voucher_date', 'br_code', 'dr_account', 'cr_account', 'amount'];

        

        $excelData = $request->excelData;
        $fin_yr_wise = GeneralSettings::where('setting_name','voucher_number_fin_yr_wise')->where('value',1)->exists();

        $year_stamp = $this->get_short_name($request->current_year);
        // $user_tag = strtoupper(str_replace(' ', '', substr($request->user->username, 0, 4)));


        // Check if the first row of the Excel data array contains all the required fields
        foreach ($required_fields as $field) {
            // Check if the required field is missing in the first row
            if (!in_array($field, $excelData[0])) {
                // Handle the case where a required field is missing
                return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.'],500);
            }
        }


        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

                $rand = rand();
                for ($i = 1; $i < count($excelData); $i++) {
                    $item = $excelData[$i];         

                    $required_err = false;
                    foreach ($required_fields as $field) {
                        // Check if the required field is missing in the first row
                        
                        if (!isset($item[$field])) {
                            // Handle the case where a required field is missing
                           $required_err = true;
                           // return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.', 'item'=>$item],500);
                        }
                    }
                    
                    if($required_err==true || !$item['amount'] || !$item['voucher_date'] || !$item['voucher_type'] || !$item['br_code'] || !$item['dr_account'] || !$item['cr_account']){
                        continue;
                    }

                    $branch_to_use = $item['br_code'];

                    $check_branch = Branch::where('branch_code', $branch_to_use)->first();
                    if(!$check_branch){
                        return response(['message'=>'Contact Admin To Set A Branch To import data']);
                    }

                    if($item['voucher_type']!=='Receipt Voucher' && $item['voucher_type']!=='Payment Voucher' && $item['voucher_type']!=='Journal' && $item['voucher_type']!=='Contra'){
                        return response(['message'=>'Voucher Type Not Valid'], 500);
                    }

                    $max = Voucher::where('branch_id',$check_branch->id)->where('voucher_type', 'SYSTEM')->where('voucher_name',$item['voucher_type'])->where('fin_yr_wise',$fin_yr_wise)->max('max_number');
                
                    
                    if($fin_yr_wise){
                        $voucher_number = $check_branch->branch_code.'/'.$year_stamp.'/'.sprintf('%06d', $max+1);
                    }
                    else{
                        $voucher_number = $check_branch->branch_code.'-'.sprintf('%06d', $max+1);
                    }

                    
                    // SAVING A new Voucher
                    $voucher = new Voucher;
                    $voucher->branch_id = $check_branch['id'];
                    $voucher->voucher_type = 'SYSTEM';
                    $voucher->fin_yr_wise = $fin_yr_wise;
                    $voucher->max_number = $max+1;
                    $voucher->voucher_name = $item['voucher_type'];

                    $voucher->voucher_number = $voucher_number;
                    
                    // $item['date'] COMES IN THIS FORMAT 14/05/2024 10:45:25

                    $voucher->voucher_date =  DateTime::createFromFormat('d-m-Y', $item['voucher_date'])->format('Y-m-d');
                    $voucher->bank_date =  DateTime::createFromFormat('d-m-Y', $item['voucher_date'])->format('Y-m-d');

                    $voucher->voucher_time = Carbon::now();


                    $voucher->narration = $item['remarks'];
                    $voucher->total_dr_amount = $item['amount'];
                    $voucher->total_cr_amount = $item['amount'];
                    $voucher->fin_year = $request->current_year;
                    
                    $voucher->rndm =  $rand;
                    $voucher->created_by = $request->user->id;
                    $voucher->save(); 
                    // new Voucher SAVED


                    $dr_account_id = $this->get_acc_id($item['dr_account']);
                    $cr_account_id = $this->get_acc_id($item['cr_account']);

                    if(!$dr_account_id || !$cr_account_id){
                        return response('The Mandatory '.$item['dr_account'].' or '.$item['cr_account'].' Account not found',500);
                    }

                        $cr_v_acc_detail = new VoucherAccDetails();
                        $cr_v_acc_detail->voucher_id = $voucher->id;
                        $cr_v_acc_detail->voucher_name = $voucher->voucher_name;
                        $cr_v_acc_detail->voucher_number = $voucher->voucher_number;
                        $cr_v_acc_detail->account_id = $cr_account_id;
                        $cr_v_acc_detail->nature = 'cr';
                        $cr_v_acc_detail->cr_amount = $item['amount'];
                        $cr_v_acc_detail->fin_year = $voucher->fin_year;
                        $cr_v_acc_detail->rndm =  $rand;
                        $cr_v_acc_detail->save();


                        $v_acc_detail = new VoucherAccDetails();
                        $v_acc_detail->voucher_id = $voucher->id;
                        $v_acc_detail->voucher_name = $voucher->voucher_name;
                        $v_acc_detail->voucher_number = $voucher->voucher_number;
                        $v_acc_detail->account_id = $dr_account_id;
                        $v_acc_detail->nature = 'dr';
                        $v_acc_detail->against = $item['against'] ?? null;
                        $v_acc_detail->dr_amount = $item['amount'];
                        $v_acc_detail->fin_year = $voucher->fin_year;
                        $v_acc_detail->rndm =  $rand;
                        $v_acc_detail->save();


                        if( isset($item['against']) && isset($item['against_number']) ){
                            if ($item['against'] && $item['against_number'] ) {

                                $AGAINST_ID = null;
                                $AGAINST_NUMBER = null;



                                if($item['against']=='VEHICLE_COST_CENTER'){
                                    $vehicle = Vehicle::select('id','vehicle_number')->where('vehicle_number', $item['against_number'])->first();

                                    if (!$vehicle) {
                                        // Handle the case where a required field is missing
                                        return response()->json(['status' => false, 'message' => 'Vehicle not exist  is missing.', 'item'=>$item],500);
                                    }

                                    $AGAINST_ID = $vehicle['id'];
                                    $AGAINST_NUMBER = $vehicle['vehicle_number'];
                                }

                                
                                    $vou_desc = new VoucherDesc();
                                    $vou_desc->voucher_id = $voucher->id;
                                    $vou_desc->voucher_name = $voucher->voucher_name;
                                    $vou_desc->voucher_number = $voucher->voucher_number;
                                    $vou_desc->v_acc_det_id = $v_acc_detail->id;
                                    $vou_desc->against = $item['against'];
                                    $vou_desc->against_id = $AGAINST_ID;
                                    $vou_desc->against_number = $AGAINST_NUMBER;
                                    $vou_desc->received_amt = $item['amount'];
                                    $vou_desc->receipt_amt = $item['amount'];
                                    $vou_desc->from_opening = false;
                                    $vou_desc->fin_year = $voucher->fin_year;
                                    $vou_desc->save();                    
                            }

                        }
                       

                    
                }

            // TRANSACTIONS COMMITTED
            DB::commit();

                
            return response()->json(['success'=>true], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
        

        

        

    }

    public function upload_purchase(Request $request){

        $required_fields = ['bill_type','br_code' ,'bill_date',	'party_name' , 'product' , 'dr_account' , 'qty', 'rate' , 'gst_type' , 'gst_percent', 'net_amount'];

        

        $excelData = $request->excelData;
       // $fin_yr_wise = GeneralSettings::where('setting_name','voucher_number_fin_yr_wise')->where('value',1)->exists();

        $year_stamp = $this->get_short_name($request->current_year);
        $user_tag = strtoupper(str_replace(' ', '', substr($request->user->username, 0, 4)));


        // Check if the first row of the Excel data array contains all the required fields
        foreach ($required_fields as $field) {
            // Check if the required field is missing in the first row
            if (!in_array($field, $excelData[0])) {
                // Handle the case where a required field is missing
                return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.'],500);
            }
        }


        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

                $rand = rand();
                for ($i = 1; $i < count($excelData); $i++) {
                    $item = $excelData[$i];         

                    $required_err = false;
                    foreach ($required_fields as $field) {
                        // Check if the required field is missing in the first row
                        
                        if (!isset($item[$field])) {
                            // Handle the case where a required field is missing
                           $required_err = true;
                           // return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.', 'item'=>$item],500);
                        }
                    }
                    
                    
                    if($required_err==true || !$item['bill_type']  ||  !$item['br_code'] || !$item['bill_date'] || !$item['party_name'] || !$item['product'] || !$item['dr_account'] || !$item['qty'] || !$item['rate'] || !$item['gst_type'] ){
                       
                        
                        continue;
                    }

                    $branch_to_use = $item['br_code'];

                    $check_branch = Branch::where('branch_code', $branch_to_use)->first();
                    if(!$check_branch){
                        return response(['message'=>'Contact Admin To Set A Branch To import data']);
                    }


                    if($item['bill_type']!=='Purchase' && $item['bill_type']!=='Service'){
                        return response(['message'=>'Bill Type Not Valid'], 500);
                    }

                    $max = Purchase::where('branch_id',$check_branch->id)->where('bill_type', 'SYSTEM')->where('bill_name',$item['bill_type'])->where('created_by',$request->user->id)->max('max_number');
                
                    if($item['bill_type']=='Purchase'){
                        $prefix = 'PUR';
                    }
                    elseif($item['bill_type']=='Service'){
                        $prefix = 'SER';
                    }
                    
                    $bill_number = $prefix.'-'.$check_branch->branch_code.'-'.$user_tag.'-'.sprintf('%06d', $max+1);   

                    
                    $cr_account_id = $this->get_acc_id($item['party_name']);
                    $dr_account_id = $this->get_acc_id($item['dr_account']);
                    if(!$dr_account_id || !$cr_account_id){
                        return response('The Mandatory '.$item['dr_account'].' or '.$item['party_name'].' Account not found',500);
                    }


                    $total_without_gst = $item['rate']* $item['qty'];
                    $total_cgst_amount = 0;
                    $total_sgst_amount = 0;
                    $total_igst_amount = 0;
                    $cgst_percent = 0;
                    $sgst_percent = 0;
                    $igst_percent = 0;


                    if($item['gst_type']=='IGST'){
                        $total_igst_amount = $total_without_gst * $item['gst_percent']/100;
                        $total_cgst_amount = 0;
                        $total_sgst_amount = 0;

                        $cgst_percent = 0;
                        $sgst_percent = 0;
                        $igst_percent = $item['gst_percent'];
                    }
                    elseif($item['gst_type']=='CGST/SGST'){
                        $total_igst_amount = 0;
                        $total_cgst_amount = $total_without_gst * $item['gst_percent']/200;
                        $total_sgst_amount = $total_without_gst * $item['gst_percent']/200;

                        $cgst_percent = $item['gst_percent']/2;
                        $sgst_percent = $item['gst_percent']/2;
                        $igst_percent = 0;
                    }
                    $total_without_rounding = $total_without_gst + $total_igst_amount + $total_cgst_amount + $total_sgst_amount;
                    $rounded = round($total_without_rounding);
                    $rounded_off = $rounded - $total_without_rounding;
                    $grand_total_amount = $total_without_rounding + $rounded_off;


                    // SAVING A NEW Bill
                    $bill = new Purchase();
                    $bill->bill_name = $item['bill_type'];
                    $bill->branch_id = $check_branch->id;
                    $bill->bill_type = 'SYSTEM';
                    $bill->prefix = null;
                    $bill->max_number = $max+1;
                    $bill->bill_number = $bill_number;
                    $bill->bill_date = Carbon::createFromFormat('d-m-Y', $item['bill_date'])->format('Y-m-d');
                    $bill->acc_party_id = $cr_account_id;
                    $bill->invoice_number = $item['invc_number']??'';
                    $bill->invoice_date = $item['invc_date']?Carbon::createFromFormat('d-m-Y', $item['invc_date'])->format('Y-m-d'):null;
                    $bill->remarks = $item['remarks']??"";
                    $bill->total_without_gst = $total_without_gst;
                    $bill->total_igst_amount = $total_igst_amount;
                    $bill->total_cgst_amount = $total_cgst_amount;
                    $bill->total_sgst_amount = $total_sgst_amount;
                    $bill->total_without_rounding = $total_without_rounding;
                    $bill->rounded_off = $rounded_off;
                    $bill->grand_total_amount = $grand_total_amount;
                    
                    $bill->fin_year = $request->current_year;

                    $bill->created_by = $request->user->id;

                    // Save the model to the database
                    $bill->save();


                    $vehicle_id = null;

                    if(isset($item['vehicle'])){
                        $vehicle = Vehicle::select('id','vehicle_number')->where('vehicle_number', $item['vehicle'])->first();
                        if (!$vehicle) {
                            // Handle the case where a required field is missing
                            return response()->json(['status' => false, 'message' => 'Vehicle not exist  is missing.', 'item'=>$item],500);
                        }
                        $vehicle_id = $vehicle['id'];
                    }

                    $product_id = null;

                    if(isset($item['product'])){
                        $product = Product::select('id','product_name')->where('product_name', $item['product'])->first();
                        if (!$product) {
                            // Handle the case where a required field is missing
                            return response()->json(['status' => false, 'message' => 'Product not exist  is missing.', 'item'=>$item],500);
                        }
                        $product_id = $product['id'];
                    }

                
                            $bill_desc = new PurchaseDesc();
                            $bill_desc->bill_id = $bill->id;
                            $bill_desc->bill_number = $bill->bill_number;
                            $bill_desc->product_id = $product_id;
                            $bill_desc->dr_account_id = $dr_account_id;
                            $bill_desc->vehicle_id = $vehicle_id;
                            $bill_desc->hsn = $item['hsn']??'';
                            $bill_desc->qty = $item['qty'];
                            $bill_desc->rate = $item['rate'];
                            $bill_desc->amount = $total_without_gst;
                            $bill_desc->cgst_percent = $cgst_percent;
                            $bill_desc->cgst_amount = $total_cgst_amount;
                            $bill_desc->sgst_percent = $sgst_percent;
                            $bill_desc->sgst_amount = $total_sgst_amount;
                            $bill_desc->igst_percent = $igst_percent;
                            $bill_desc->igst_amount = $total_igst_amount;
                            $bill_desc->net_amount = $total_without_rounding;
                            $bill_desc->fin_year = $request->current_year;
                            $bill_desc->save();
                        
                    // BILL DESC SAVED


                    $this->record_transaction_in_purchase($request, $bill);
                       

                    
                }

            // TRANSACTIONS COMMITTED
            DB::commit();

                
            return response()->json(['success'=>true], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
        
    }


    public function upload_diesel_receipt(Request $request){

        $required_fields = ['branch_code','date','dr_account','filling_station','amount','vehicle_number'];

        $excelData = $request->excelData;

        // Check if the first row of the Excel data array contains all the required fields
        foreach ($required_fields as $field) {
            // Check if the required field is missing in the first row
            if (!in_array($field, $excelData[0])) {
                // Handle the case where a required field is missing
                return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.'],500);
            }
        }


        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

                $rand = rand();
                for ($i = 1; $i < count($excelData); $i++) {
                    $item = $excelData[$i];

                    

                    $required_err = false;
                    foreach ($required_fields as $field) {
                        // Check if the required field is missing in the first row
                        
                        if (!isset($item[$field])) {
                            // Handle the case where a required field is missing
                           $required_err = true;
                           // return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.', 'item'=>$item],500);
                        }
                    }
                    
                    if($required_err==true || !$item['amount']){
                        continue;
                    }

                    $branch_to_use = $item['branch_code'] ?? 'DL';

                    $dr_account = $this->get_acc_id($item['dr_account']);
                    $filling_station = $this->get_acc_id($item['filling_station']);
            
                    if(!$dr_account || !$filling_station){
                        return response('The Mandatory Debit or Filling  Payment Account not found',500);
                    }
            
                    $check_branch = Branch::where('branch_code', $branch_to_use)->first();
                    if(!$check_branch){
                        return response(['message'=>'Contact Admin To Set A Branch To import data']);
                    }
                    
                    
                    $branch_id = $check_branch->id;
                    $branch_code = $check_branch->branch_code;
                    
                    $max = DieselReceipt::where('branch_id',$branch_id)->max('max_number');
                    $dr_number = $branch_code.'-'.sprintf('%06d', $max+1);
            
                    // SAVING A NEW DR
                    $dr = new DieselReceipt();
                    $dr->branch_id = $branch_id;
                    $dr->max_number = $max+1;
                    $dr->dr_number = $dr_number;
                    $dr->against = $item['against'];
                      
                    $vehicle = Vehicle::select('id','vehicle_number')->where('vehicle_number', $item['vehicle_number'])->first();

                    if(!$vehicle){
                        return response($item['vehicle_number'].' Vehicle Not Found ',500);
                    }

                    $dr->vehicle_id = $vehicle ? $vehicle->id : null;
                    $dr->vehicle_number = $vehicle ? $vehicle->vehicle_number : null;
                    
                    if($item['against']){

                        if($item['against']=='GR'){
                            $gr = BookGr::select('id','gr_number','gr_date')->where('gr_number', $item['against_number'])->first();
                            $dr->booking_id = $gr ? $gr->id : null;
                            $dr->booking_number =  $gr ? $gr->gr_number : null;
                            $dr->bk_date = $gr ? $gr->gr_date : null;
                        }
                        else if($item['against']=='Loading Slip'){
                            $ls = LoadingSlip::select('id','ls_number','ls_date')->where('ls_number', $item['against_number'])->first();
                            $dr->booking_id = $ls ? $ls->id : null;
                            $dr->booking_number =  $ls ? $ls->ls_number : null;
                            $dr->bk_date = $ls ? $ls->ls_date : null;
                        }
                        else if($item['against']=='Challan'){
                            $ch = Challan::select('id','challan_number','challan_date')->where('challan_number', $item['against_number'])->first();
                            $dr->booking_id = $ch ? $ch->id : null;
                            $dr->booking_number =  $ch ? $ch->challan_number : null;
                            $dr->bk_date = $ch ? $ch->challan_date : null;
                        }

                    }

                    $dr->booking_date = DateTime::createFromFormat('d-m-Y', $item['date'])->format('Y-m-d');
                    $dr->bill_number = $item['bill_no'];
                    $dr->dr_on_account = $dr_account;
                    $dr->filling_station = $filling_station;
                    $dr->slip_number = $item['slip_no'];
                    $dr->fuel_type = $item['fuel_type'];
                    $dr->qty = $item['qty'];
                    $dr->rate = $item['rate'];
                    $dr->amount = $item['amount'];
                    $dr->remarks = $item['remarks']??"";
                    $dr->reading = $item['reading'];
                    $dr->cash_amount = $item['cash_amount'];

                    $dr->rand = $rand;

                    $dr->driver_on_account = $this->get_acc_id($item['driver_account']);

                    $dr->fin_year = $request->current_year; 

                    $dr->created_by = $request->user->id;

                    // Save the model to the database
                    $dr->save();
                    
                    
                    // RECORD THE TRANSACTION IN ACCOUNTS
                    $this->record_transaction_in_diesel_receipts($request, $dr);
                    
                }

            // TRANSACTIONS COMMITTED
            DB::commit();

                
            return response()->json(['success'=>true], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
            

    }

    public function upload_price_master(Request $request){

        $required_fields = ['price_type','eff_from','eff_till','source','destination','rate','p_w_unit'];

        $excelData = $request->excelData;

        

        // Check if the first row of the Excel data array contains all the required fields
        foreach ($required_fields as $field) {
            // Check if the required field is missing in the first row
            if (!in_array($field, $excelData[0])) {
                // Handle the case where a required field is missing
                return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

                $rand = rand();
                for ($i = 1; $i < count($excelData); $i++) {
                    $pl = $excelData[$i];

                   
                    
                    $required_err = false;
                    foreach ($required_fields as $field) {
                        // Check if the required field is missing in the first row
                        
                        if (!isset($pl[$field])) {
                            // Handle the case where a required field is missing
                           $required_err = true;
                           // return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.', 'item'=>$item],500);
                        }
                    }
                    
                    if($required_err==true){
                        continue;
                    }
                    
                    $accid = $this->get_acc_id($pl['acc_party']);
                    if(!$accid){
                        return response()->json(['status' => false, 'message' => 'Required PARTY ' . $pl['acc_party'] . ' is missing.'],500);
                    }

                    $sourceid = Station::where('station_name', $pl['source'])->first();
                    if(!$sourceid){
                        $sourceid = new Station();
                        $sourceid->station_name = $pl['source'];
                        $sourceid->created_by = $request->user->id;
                        $sourceid->save();
                        
                       
                    }
                    $sourceid = $sourceid->id;
                    
                    

                    $station = Station::where('station_name', $pl['destination'])->first();

                    if (!$station) {
                        $station = new Station();
                        $station->station_name = $pl['destination'];
                        $station->created_by = $request->user->id;
                        $station->save();
                    }
                    
                    $destinationid = $station->id;
                  
                  

                    // if($pl['consignor']){
                    // $consignorid = Party::select('id','party_name')->where('party_name', $pl['consignor'])->value('id');
                    // if(!$consignorid){
                    // return response()->json(['status' => false, 'message' => 'Required consignor ' . $pl['consignor'] . ' is missing.'],500);
                    //     dd($pl['consignor'].' consignor Not Found ');
                    // }
                    //}

                    // if($pl['consignee']){
                    // $consigneerid = Party::select('id','party_name')->where('party_name', $pl['consignee'])->value('id');
                    // if(!$consigneerid){
                    //  return response()->json(['status' => false, 'message' => 'Required consignee ' . $pl['consignee'] . ' is missing.'],500);
                    // }
                    //}

                    $materialid = null;

                    if(isset($pl['material_desc'])){
                        $materialid = Material::select('id','material_name')->where('material_name', $pl['material_desc'])->value('id');
                        if(!$materialid){
                            return response()->json(['status' => false, 'message' => 'Required material_desc ' . $pl['material_desc'] . ' is missing.'],500); 
                           // dd($pl['material_desc'].' material_desc Not Found ');
                        }

                    }

                    
                   

                    if( !$accid || !$pl['eff_from'] || !$pl['eff_till'] || !$sourceid || !$destinationid || !$pl['rate'] || !$pl['p_w_unit'])
                    continue;
                    
                    // Check if a similar price list already exists
                    $exists = $this->find_duplicate($pl,$accid,$sourceid,$destinationid,$materialid);

                    //dd(e)
                   
                   

                    $ind = $i + 1;
                    if ($exists) {
                        return response()->json(['errors' => ['type'=>'Duplicate Entry', 'msg'=>"Mathing Record At Price Number ".$exists['id'], 'index'=>$exists, 'id'=>$exists['id'] ,'exist'=>$exists ]], 422);
                    }
                    
                    $pri_lis = new PriceList();

                    $pri_lis->acc_party_id = $pl['acc_party'] ? $accid : null;
                    
                    $pri_lis->eff_from = DateTime::createFromFormat('d-m-Y', $pl['eff_from'])->format('Y-m-d');
                    $pri_lis->eff_till = DateTime::createFromFormat('d-m-Y', $pl['eff_till'])->format('Y-m-d');

                    $pri_lis->source_id = $pl['source'] ? $sourceid : null;
                    $pri_lis->destination_id = $pl['destination'] ? $destinationid : null;
                    
            
                    // $pri_lis->consignor_id = $pl['consignor'] ? $consignorid : null;
                    // $pri_lis->consignee_id = $pl['consignee'] ? $consigneerid : null;

                    $pri_lis->material_id = isset($pl['material_desc']) ? $materialid : null;

                    $check_vehicle_type = VehicleType::where('vehicle_type', $pl['vehicle_type'])->first();
                    if (!$check_vehicle_type) {
                        // Vehicle type doesn't exist, create a new one
                        $vehicle_type = new VehicleType();
                        $vehicle_type->vehicle_type = $pl['vehicle_type'];
                        $vehicle_type->created_by = $request->user->id;
                        $vehicle_type->save();
                        $vehicle_type_id = $vehicle_type->id;
                    } else {
                        $vehicle_type_id = $check_vehicle_type->id;
                    }
                    
                    $pri_lis->vehicle_type = $pl['vehicle_type'];                     
                    $pri_lis->min_wt = @$pl['wt'];
                    $pri_lis->min_freight = $pl['min_frt'];
                    $pri_lis->rate = $pl['rate'];
                    $pri_lis->price_type = $pl['price_type'];
                    $pri_lis->min_wt_unit = $pl['p_w_unit'];
                    
                    $pri_lis->created_by = $request->user->id;
                    // dd($pri_lis);
                    $pri_lis->save();

                }

            // TRANSACTIONS COMMITTED
            DB::commit();

                
            return response()->json(['success'=>true], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
        

        

        

    }


    protected function upload_gr(Request $request){

        $required_fields = ['branch_code', 'gr_no', 'gr_date','source', 'destination', 'vehicle_number', 'pay_mode', 'consignor_name', 'on_account_party', 'chg_unit', 'total_pkg'];

        $excelData = $request->excelData;

        

        // Check if the first row of the Excel data array contains all the required fields
        foreach ($required_fields as $field) {
            // Check if the required field is missing in the first row
            if (!in_array($field, $excelData[0])) {
                // Handle the case where a required field is missing
                return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.'],500);
            }
        }

        try {
            // TRANSACTION BEGINS
            DB::beginTransaction();  

                $rand = rand();
                for ($i = 1; $i < count($excelData); $i++) {
                    $pl = $excelData[$i];

                   
                    
                    $required_err = false;
                    foreach ($required_fields as $field) {
                        // Check if the required field is missing in the first row
                        
                        if (!isset($pl[$field])) {
                            // Handle the case where a required field is missing
                           $required_err = true;
                           // return response()->json(['status' => false, 'message' => 'Required field ' . $field . ' is missing.', 'item'=>$item],500);
                        }
                    }
                
                    if($required_err==true){
                        continue;
                    }

                    $gr_check = BookGr::where('gr_number', $pl['gr_no'])->first();
                    if($gr_check){
                        return response('GR NUMBER ALREADY EXISTS '.$pl['gr_no'],500);
                    }
                    
                    $branch_to_use = $pl['branch_code'];
                    $branch_check = Branch::where('branch_code', $branch_to_use)->first();
                    if(!$branch_check){
                        return response('BRANCH NOT FOUND '.$pl['branch_code'],500);
                    }

                    $accid = $this->get_acc_id($pl['on_account_party']);
                    if(!$accid){
                        return response('The Mandatory '.$pl['on_account_party'].' Party Account not found',500);
                    }

                    // $source = Station::select('id','station_name')->where('station_name', $pl['source'])->first();
                    // if(!$source){
                    //     return response('The Mandatory '.$pl['source'].' source Not Found',500);
                    // }
                    

                    // $destination = Station::select('id','station_name')->where('station_name', $pl['destination'])->first();
                    // if(!$destination){
                    //     return response('The Mandatory '.$pl['destination'].' destination Not Found',500);
                    // }

                    // $vehicle = Vehicle::select('id','vehicle_number')->where('vehicle_number', $pl['vehicle_number'])->first();
                    // if(!$vehicle){
                    //     return response('The Mandatory '.$pl['vehicle_number'].' vehicle_number Not Found',500);
                    // }

                    if($pl['consignor_name']){
                    $consignor = Party::select('id','party_name','gst_number','mob_number','address1')->where('party_name', $pl['consignor_name'])->first();
                    if(!$consignor){
                        return response($pl['consignor_name'].' consignor Not Found ',500);
                    }
                    }

                    if(isset($pl['consignee_name'])){
                    $consignee = Party::select('id','party_name','gst_number','mob_number','address1')->where('party_name', $pl['consignee_name'])->first();
                    if(!$consignee){
                        return response($pl['consignee_name'].' consignee Not Found ',500);
                    }
                    }

                    // if($pl['material']){
                    //     $materialid = Material::select('id','material_name')->where('material_name', $pl['material'])->value('id');
                    //     if(!$materialid){
                    //         return response('The Mandatory '.$pl['material'].' Material Not Found',500);
                    //     }
                    // }

                    if(isset($pl['driver_name']) && $pl['driver_name']){
                        $driver = Employee::select('id','emp_name')->where('emp_name', $pl['driver_name'])->first();
                        if(!$driver){
                            return response('The Mandatory '.$pl['driver_name'].' Driver Not Found',500);
                        }
                    }

                    
                   

                    if( !$accid || !$pl['gr_no'] || !$pl['gr_date'] || !$pl['pay_mode'] || !$pl['chg_unit'] || !$pl['total_pkg']) {
                        continue;
                    }
                    
                   
                    $max_number = $this->get_max_number($pl['gr_no']);
                    
                    // SAVING A NEW GR
                    $gr = new BookGr();
                    $gr->branch_id = $branch_check->id;
                    $gr->gr_type = 'PRINTED';
                    $gr->prefix = $request->prefix;
                    $gr->max_number = $max_number;
                    $gr->gr_number = $pl['gr_no'];

                    $gr->shipment_number = @$pl['shipment_number'];
                    $gr->po_no = @$pl['po_no'];

                    

                    $gr->gr_date = DateTime::createFromFormat('d-m-Y', $pl['gr_date'])->format('Y-m-d');

                    $gr->gr_time = Carbon::now()->format('H:i:s');


                    $source_obj = ['id'=>null, 'station_name'=>$pl['source']];
                    $source_id = $this->add_if_not_in_master('Station', $source_obj, $request->current_year);

                    $gr->source_id = $source_id;
                    $gr->source_name = $pl['source'];

                    
                    $destination_obj = ['id'=>null, 'station_name'=>$pl['destination']];
                    $destination_id = $this->add_if_not_in_master('Station', $destination_obj, $request->current_year);
                    $gr->destination_id = $destination_id;
                    $gr->destination_name = $pl['destination'];


               

                    $gr->tpt_mode = @$pl['tpt_mode'] ? $pl['tpt_mode'] : 'Surface';
                    
                    $vehicle_obj = ['id'=>null, 'vehicle_number'=>$pl['vehicle_number']];
                    $vehicle_id = $this->add_if_not_in_master('Vehicle', $vehicle_obj, $request->current_year);
                    $gr->vehicle_id = $vehicle_id;
                    $gr->vehicle_number = $pl['vehicle_number'];

                    
                    $gr->vehicle_type = @$pl['vehicle_type'];
                    $gr->driver_id = @$driver->id;
                    $gr->driver_name = @$driver->emp_name;
                    $gr->driver_mob_number = @$pl['driver_contact'];
                    $gr->edd = isset($pl['edd']) &&  $pl['edd']  ? DateTime::createFromFormat('d-m-Y', $pl['edd'])->format('Y-m-d') : null;
                    $gr->pay_mode = $pl['pay_mode'];
                    
                    if($pl['pay_mode'] == 'TO PAY'){
                        $gr->income_account_id = 6;  // TO PAY
                    }elseif($pl['pay_mode'] == 'PAID'){
                        $gr->income_account_id = 7;  // PAID
                    }


                    $gr->delivery_pt = @$pl['delivery_pt']??"Godown Delivery";

                    $gr->consignor_id = $pl['consignor_name'] ? $consignor->id : null;
                    $gr->consignor_name = $pl['consignor_name'];
                    $gr->consignor_gst_number = @$consignor->gst_number;
                    $gr->consignor_phone_number = @$consignor->mob_number;
                    $gr->consignor_address = @$consignor->address1;

                    $gr->payment_dynamic_fields = '{}';

                    $gr->consignee_id = isset($pl['consignee_name'] ) && $pl['consignee_name'] ? $consignee->id : null;
                    $gr->consignee_name = isset($pl['consignee_name'] ) && $pl['consignee_name'] ? $pl['consignee_name'] : null;
                    $gr->consignee_gst_number = @$consignee->gst_number;
                    $gr->consignee_phone_number = @$consignee->mob_number;
                    $gr->consignee_address = @$consignee->address1;

                    $gr->acc_party_id = $pl['on_account_party'] ? $accid : null;

                    $gr->total_pkg = $pl['total_pkg'];
                    $gr->total_material_weight = @$pl['total_weight'];
                    $gr->total_chg_qty = @$pl['total_chg_qty'];
                    $gr->total_desc_value = @$pl['freight'];
                    $gr->total_invoice_value = @$pl['invoice_value'];

                    $gr->remarks = @$pl['remarks'];

                    $gr->freight = @$pl['freight'];
                    $gr->gr_chg = @$pl['gr_chg'];
                    $gr->gst_paid_by = $pl['gst_paid_by']??'Consignor';
                    $gr->total_without_gst = @$pl['total_without_gst'];
                    $gr->igst_percent = @$pl['igst_percent'];
                    $gr->cgst_percent = @$pl['cgst_percent'];
                    $gr->sgst_percent = @$pl['sgst_percent'];
                    $gr->igst_amount = @$pl['igst_amount'];
                    $gr->cgst_amount = @$pl['cgst_amount'];
                    $gr->sgst_amount = @$pl['sgst_amount'];

                    $gr->total_without_rounding = @$pl['grand_total'];
                    $gr->rounded_off = 0;
                    $gr->grand_total = @$pl['grand_total'];

                    $gr->dr_on_account = $this->get_acc_id('Lorry Hire Freight Exp.');
                    if(isset($pl['lorry_hire_from']) &&  isset($pl['lh_freight'])) {
                        $gr->lorry_hire_from = $this->get_acc_id($pl['lorry_hire_from']);
                        $gr->lh_freight = $pl['lh_freight'];
                        $gr->lh_net_freight = $pl['lh_freight'];
                        $gr->lh_balance = $pl['lh_freight'];
                        
                    }
                    

                    $gr->freight_bill = false;
                    $gr->way_exp_bill = false;

                    $gr->fin_year = $request->current_year; 

                    $gr->created_by = $request->user->id;

                    // Save the model to the database
                    $gr->save();


                       if(isset($pl['chg_unit']) && $pl['total_chg_qty'] ){

                       
                        $pkg_det = new GrDesc();
                        $pkg_det->gr_id = $gr->id;
                        $pkg_det->gr_number = $gr->gr_number;

                        if(isset($pl['material'])){
                        $material_obj = ['id'=>null, 'material_name'=>$pl['material']];
                        $material_id = $this->add_if_not_in_master('Material', $material_obj , $request->current_year);
                        }
                        else{
                            $material_id = null;
                        }
                        $pkg_det->material_id = $material_id;
                        $pkg_det->destination_id = $destination_id;
                        $pkg_det->packing_method = @$pl['packing_method'];
                        $pkg_det->pkg = $pl['total_pkg'];
                        $pkg_det->material_weight = @$pl['total_weight'];
                        $pkg_det->chg_unit = @$pl['chg_unit'];
                        $pkg_det->chg_qty = @$pl['total_chg_qty'];
                        $pkg_det->rate = @$pl['rate'];
                        $pkg_det->total = @$pl['freight'];
                        $pkg_det->invoice_number = @$pl['invoice_number'];
                        $pkg_det->invoice_date = isset($pl['invoice_date']) && $pl['invoice_date'] ? DateTime::createFromFormat('d-m-Y', @$pl['invoice_date'])->format('Y-m-d') : null;
                        $pkg_det->invoice_value = @$pl['invoice_value'];
                        $pkg_det->fin_year = $request->current_year;

                        $pkg_det->free_pkgs = @$pl['total_pkg'];
                        $pkg_det->save();


                        // SAVING PACKAGE POSITIONS
                        $pkg_position = new PackagePosition();
                        $pkg_position->gr_id = $gr->id;
                        $pkg_position->gr_number = $gr->gr_number;
                        $pkg_position->gr_desc_id = $pkg_det->id;
                        $pkg_position->position = $branch_check->id;
                        $pkg_position->pkg = $pl['total_pkg'];
                        $pkg_position->origin = $branch_check->id;
                        $pkg_position->save();
                        // PACKAGE POSITIONS SAVED

                    }
                        

                        
                        if(isset($pl['eway_number'])){
                            $pkg_det = new EwayGr();
                            $pkg_det->gr_id = $gr->id;
                            $pkg_det->gr_number = $gr->gr_number;
                            $pkg_det->eway_number = @$pl['eway_number'];
                            $pkg_det->eway_exp_date = isset($pl['eway_exp_date']) && $pl['eway_exp_date'] ? DateTime::createFromFormat('d-m-Y', @$pl['eway_exp_date'])->format('Y-m-d') : null;
                            $pkg_det->fin_year = $request->current_year;
                            $pkg_det->save();
                        }
                    
                    // RECORD THE TRANSACTION IN ACCOUNTS
                    
                    $this->record_transaction_in_gr($request, $gr);
                    // TRANSACTION RECORDED IN ACCOUNTS

                }

            // TRANSACTIONS COMMITTED
            DB::commit();

                
            return response()->json(['success'=>true], 200); // RETURNED RESPONSE

        } 
        // IF ANY ERROR WHILE SAVING
        catch (\Exception $e) {
            DB::rollback();
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }


    protected function record_transaction_in_gr($request, $gr){
        $t_data = [];
        $t_data['t_against'] = 'GR';
        $t_data['t_against_id'] = $gr->id;
        $t_data['fin_year'] = $request->current_year;
        $t_data['t_against_no'] = $gr->gr_number;
        
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = ['id'=>$gr->branch_id, 'branch_code'=>Branch::find($gr->branch_id)->branch_code];
        $t_data['voucher_date'] = $gr->gr_date;
        $t_data['voucher_time'] = $gr->gr_time;
        $t_data['t_against_narration'] = $gr->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];
        
        
            $t_data['cancellable'] = 1;
            if($gr->pay_mode == 'TO PAY' || $gr->pay_mode == 'PAID'){
                if($gr->income_account_id && $gr->total_without_rounding>0){

                    $AFTER_GST = $gr->green_tax + $gr->kanta + $gr->detention; 
                    $t_data['voucher_accounting_details']=[];
                    // DEBIT PARTY ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->acc_party_id, 'nature'=>'dr' , 'dr_amount'=>$gr->total_without_rounding, 'cr_amount'=>0 ];
                    

                    // CREDIT SIDE
                    if($gr->cgst_amount > 0){
                        // CREDIT CGST ACCOUNT
                        $t_data['voucher_accounting_details'][] = ['account_id'=>12, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$gr->cgst_amount ];
                    }
                    if($gr->sgst_amount > 0){
                        // CREDIT SGST ACCOUNT
                        $t_data['voucher_accounting_details'][] = ['account_id'=>13, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$gr->sgst_amount ];
                    }
                    if($gr->igst_amount > 0){
                        // CREDIT IGST ACCOUNT
                        $t_data['voucher_accounting_details'][] = ['account_id'=>14, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$gr->igst_amount ];
                    }

                    // CREDIT INCOME ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->income_account_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$gr->total_without_gst+$AFTER_GST ];

                    
                    //CHECK TOTAL DR AND CR
                    $total_dr = 0;
                    $total_cr = 0;

                    foreach ($t_data['voucher_accounting_details'] as $vad) {
                        $total_dr += $vad['dr_amount'];
                        $total_cr += $vad['cr_amount'];
                    }

                                // Round both $total_dr and $total_cr to two decimal places
                    $total_dr_rounded = round(floatval($total_dr), 2);
                    $total_cr_rounded = round(floatval($total_cr), 2);

                    // Check if they are not equal
                    if ($total_dr_rounded != $total_cr_rounded) {
                        // Output the values for debugging
                        dd($total_dr_rounded, $total_cr_rounded);
                        
                        // Return response with an error message
                        return response()->json(['message' => 'Error In Accounting Entries'], 422);
                    }


                                $t_data['total_dr_amount'] = $total_dr;
                                $t_data['total_cr_amount'] = $total_cr; 


                                $this->create_transaction_voucher( $t_data );
                            }
            }

            $t_data['cancellable'] = 0;
            if($gr->driver_payment_amt>0 && $gr->driver && $gr->driver_payment_from){
                $driver_account_id = $this->get_acc_id($gr->driver['emp_name']);
                $t_data['voucher_accounting_details']=[];
                // DEBIT PARTY ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$driver_account_id, 'nature'=>'dr' , 'dr_amount'=>$gr->driver_payment_amt, 'cr_amount'=>0 ];
                

                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->driver_payment_from, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$gr->driver_payment_amt ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }

                $total_dr_rounded = round(floatval($total_dr), 2);
                $total_cr_rounded = round(floatval($total_cr), 2);

                // Check if they are not equal
                if ($total_dr_rounded != $total_cr_rounded) {
                            // Output the values for debugging
                            dd($total_dr_rounded, $total_cr_rounded);
                    
                    // Return response with an error message
                    return response()->json(['message' => 'Error In Accounting Entries'], 422);
                }


                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 


                $this->create_transaction_voucher( $t_data );
            }
      

            $t_data['cancellable'] = 0;
            if($gr->lh_net_freight>0 && $gr->lorry_hire_from){
                $t_data['voucher_accounting_details']=[];
                // DEBIT PARTY ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->dr_on_account, 'nature'=>'dr' , 'dr_amount'=>$gr->lh_net_freight, 'cr_amount'=>0 ];
                

                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$gr->lh_net_freight ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }

                if($total_dr != $total_cr){
                    return response()->json(['message'=>'Error In Accounting Entries'], 422);
                }


                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 


                $this->create_transaction_voucher( $t_data );
            }

            // $t_data['cancellable'] = 0;
            // if($gr->lh_advance>0 && $gr->lh_payment_acc1 && $gr->lorry_hire_from){
            //     $t_data['voucher_accounting_details']=[];
            //     // DEBIT PARTY ACCOUNT
            //     $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->lh_advance, 'cr_amount'=>0 ];
                

            //     // CREDIT INCOME ACCOUNT
            //     $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lh_payment_acc1, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->lh_advance ];

                
            //     //CHECK TOTAL DR AND CR
            //     $total_dr = 0;
            //     $total_cr = 0;

            //     foreach ($t_data['voucher_accounting_details'] as $vad) {
            //         $total_dr += $vad['dr_amount'];
            //         $total_cr += $vad['cr_amount'];
            //     }

            //     if($total_dr != $total_cr){
            //         return response()->json(['message'=>'Error In Accounting Entries'], 422);
            //     }


            //     $t_data['total_dr_amount'] = $total_dr;
            //     $t_data['total_cr_amount'] = $total_cr; 


            //     $this->create_transaction_voucher( $t_data );
            // }

            // $t_data['cancellable'] = 0;
            // if($gr->lh_payment>0 && $gr->lh_payment_acc2 && $gr->lorry_hire_from){
            //     $t_data['voucher_accounting_details']=[];
            //     // DEBIT PARTY ACCOUNT
            //     $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->lh_payment, 'cr_amount'=>0 ];
                

            //     // CREDIT INCOME ACCOUNT
            //     $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lh_payment_acc2, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->lh_payment ];

                
            //     //CHECK TOTAL DR AND CR
            //     $total_dr = 0;
            //     $total_cr = 0;

            //     foreach ($t_data['voucher_accounting_details'] as $vad) {
            //         $total_dr += $vad['dr_amount'];
            //         $total_cr += $vad['cr_amount'];
            //     }

            //     if($total_dr != $total_cr){
            //         return response()->json(['message'=>'Error In Accounting Entries'], 422);
            //     }


            //     $t_data['total_dr_amount'] = $total_dr;
            //     $t_data['total_cr_amount'] = $total_cr; 


            //     $this->create_transaction_voucher( $t_data );
            // }

            // $t_data['cancellable'] = 0;
            // if($gr->lh_payment2>0 && $gr->lh_payment_acc3 && $gr->lorry_hire_from){
            //     $t_data['voucher_accounting_details']=[];
            //     // DEBIT PARTY ACCOUNT
            //     $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->lh_payment2, 'cr_amount'=>0 ];
                

            //     // CREDIT INCOME ACCOUNT
            //     $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lh_payment_acc3, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->lh_payment2 ];

                
            //     //CHECK TOTAL DR AND CR
            //     $total_dr = 0;
            //     $total_cr = 0;

            //     foreach ($t_data['voucher_accounting_details'] as $vad) {
            //         $total_dr += $vad['dr_amount'];
            //         $total_cr += $vad['cr_amount'];
            //     }

            //     if($total_dr != $total_cr){
            //         return response()->json(['message'=>'Error In Accounting Entries'], 422);
            //     }


            //     $t_data['total_dr_amount'] = $total_dr;
            //     $t_data['total_cr_amount'] = $total_cr; 


            //     $this->create_transaction_voucher( $t_data );
            // }

            // $t_data['cancellable'] = 0;
            // if($gr->diesel_amt1>0 && $gr->diesel_acc1 && $gr->lorry_hire_from){
            //     $t_data['voucher_accounting_details']=[];
            //     // DEBIT PARTY ACCOUNT
            //     $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->diesel_amt1, 'cr_amount'=>0 ];
                

            //     // CREDIT INCOME ACCOUNT
            //     $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->diesel_acc1, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->diesel_amt1 ];

                
            //     //CHECK TOTAL DR AND CR
            //     $total_dr = 0;
            //     $total_cr = 0;

            //     foreach ($t_data['voucher_accounting_details'] as $vad) {
            //         $total_dr += $vad['dr_amount'];
            //         $total_cr += $vad['cr_amount'];
            //     }

            //     if($total_dr != $total_cr){
            //         return response()->json(['message'=>'Error In Accounting Entries'], 422);
            //     }


            //     $t_data['total_dr_amount'] = $total_dr;
            //     $t_data['total_cr_amount'] = $total_cr; 


            //     $this->create_transaction_voucher( $t_data );
            // }

            // $t_data['cancellable'] = 0;
            // if($gr->diesel_amt2>0 && $gr->diesel_acc1 && $gr->lorry_hire_from){
            //     $t_data['voucher_accounting_details']=[];
            //     // DEBIT PARTY ACCOUNT
            //     $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->lorry_hire_from, 'nature'=>'dr' , 'dr_amount'=>$request->diesel_amt2, 'cr_amount'=>0 ];
                

            //     // CREDIT INCOME ACCOUNT
            //     $t_data['voucher_accounting_details'][] = ['account_id'=>$gr->diesel_acc2, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$request->diesel_amt2 ];

                
            //     //CHECK TOTAL DR AND CR
            //     $total_dr = 0;
            //     $total_cr = 0;

            //     foreach ($t_data['voucher_accounting_details'] as $vad) {
            //         $total_dr += $vad['dr_amount'];
            //         $total_cr += $vad['cr_amount'];
            //     }

            //     if($total_dr != $total_cr){
            //         return response()->json(['message'=>'Error In Accounting Entries'], 422);
            //     }


            //     $t_data['total_dr_amount'] = $total_dr;
            //     $t_data['total_cr_amount'] = $total_cr; 


            //     $this->create_transaction_voucher( $t_data );
            // }

}

    protected function record_transaction_in_diesel_receipts($request, $dr){
        $t_data = [];
        $t_data['t_against'] = 'DIESEL RECEIPT';
        $t_data['t_against_id'] = $dr->id;
        $t_data['t_against_no'] = $dr->dr_number;
        $t_data['fin_year'] = $request['current_year'];
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = $dr->branch_id ? ['id'=>$dr->branch_id] : null;
        $t_data['voucher_date'] = $dr['booking_date'];
        $t_data['voucher_time'] = null;
        $t_data['t_against_narration'] = $dr->remarks;
        $t_data['fin_year'] = $request['current_year'];

        $t_data['voucher_accounting_details'] = [];
        
        $t_data['cancellable'] = 1;
        if($dr->dr_on_account && $dr->filling_station && $dr['amount']>0){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$dr->dr_on_account, 'nature'=>'dr' , 'dr_amount'=>$dr['amount'], 'cr_amount'=>0 ];
            

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$dr->filling_station, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$dr['amount'] ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 

            $this->create_transaction_voucher( $t_data );
        }

        $t_data['cancellable'] = 1;
        if($dr->driver_on_account && $dr->filling_station && $dr['cash_amount']>0){
            $t_data['voucher_accounting_details']=[];
            // DEBIT PARTY ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$dr->driver_on_account, 'nature'=>'dr' , 'dr_amount'=>$dr['cash_amount'], 'cr_amount'=>0 ];
            

            // CREDIT INCOME ACCOUNT
            $t_data['voucher_accounting_details'][] = ['account_id'=>$dr->filling_station, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$dr['cash_amount'] ];

            
            //CHECK TOTAL DR AND CR
            $total_dr = 0;
            $total_cr = 0;

            foreach ($t_data['voucher_accounting_details'] as $vad) {
                $total_dr += $vad['dr_amount'];
                $total_cr += $vad['cr_amount'];
            }

            if($total_dr != $total_cr){
                return response()->json(['message'=>'Error In Accounting Entries'], 422);
            }


            $t_data['total_dr_amount'] = $total_dr;
            $t_data['total_cr_amount'] = $total_cr; 

            $this->create_transaction_voucher( $t_data );
        }

    }


    protected function record_transaction_in_purchase($request, $bill){
        $t_data = [];
        $t_data['t_against'] = 'Purchase Bills';
        $t_data['t_against_id'] = $bill->id;
        $t_data['t_against_no'] = $bill->bill_number;
        $t_data['fin_year'] = $request->current_year;
        $this->delete_transaction_voucher($t_data);
        
        $t_data['branch'] = $bill->branch_id ? ['id'=>$bill->branch_id] : null;
        $t_data['voucher_date'] = $bill->bill_date;
        $t_data['voucher_time'] = $bill->bill_time;
        $t_data['t_against_narration'] = $bill->remarks;
        $t_data['fin_year'] = $request->current_year;

        $t_data['voucher_accounting_details'] = [];


        if($bill->bill_name=='Purchase' || $bill->bill_name=='Service'){
            $purchaseDescQuery = PurchaseDesc::where('bill_id', $bill->id);

            $bill_desc = $purchaseDescQuery->get();

            $sums = $purchaseDescQuery->selectRaw('SUM(cgst_amount) as total_cgst, SUM(sgst_amount) as total_sgst, SUM(igst_amount) as total_igst, SUM(amount) as total_amount')
                ->first();

            
            $total_cgst = $sums->total_cgst ?? 0;
            $total_sgst = $sums->total_sgst ?? 0;
            $total_igst = $sums->total_igst ?? 0;
            $total_amount = $sums->total_amount ?? 0;

            
            $t_data['cancellable'] = 1;
            
            if($bill->acc_party_id && $total_amount>0 ){
                foreach ($bill_desc as $key) {
                    // DEBIT ALL THE ACCOUNTS IN DESC
                    if($key['dr_account_id'] && $key['amount']){
                        $t_data['voucher_accounting_details'][] = ['account_id'=>$key['dr_account_id'], 'nature'=>'dr' , 'dr_amount'=>$key['amount'], 'cr_amount'=>0 ];
                    }
                }

                // CREDIT PARTY AND GST
                // CREDIT SIDE
                if($total_cgst > 0){ 
                    // CREDIT CGST ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>9, 'nature'=>'dr' , 'dr_amount'=>$total_cgst, 'cr_amount'=>0 ];
                }
                if($total_sgst > 0){
                    // CREDIT SGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>10, 'nature'=>'dr' , 'dr_amount'=>$total_sgst, 'cr_amount'=>0 ];
                }
                if($total_igst > 0){
                    // CREDIT IGST ACCOUNT
                    $t_data['voucher_accounting_details'][] = ['account_id'=>11, 'nature'=>'dr' , 'dr_amount'=>$total_igst, 'cr_amount'=>0 ];
                }

                // CREDIT INCOME ACCOUNT
                $t_data['voucher_accounting_details'][] = ['account_id'=>$bill->acc_party_id, 'nature'=>'cr' , 'dr_amount'=>0, 'cr_amount'=>$bill->total_without_rounding ];

                
                //CHECK TOTAL DR AND CR
                $total_dr = 0;
                $total_cr = 0;

                foreach ($t_data['voucher_accounting_details'] as $vad) {
                    $total_dr += $vad['dr_amount'];
                    $total_cr += $vad['cr_amount'];
                }


                // Round both $total_dr and $total_cr to two decimal places
            $total_dr_rounded = round(floatval($total_dr), 2);
            $total_cr_rounded = round(floatval($total_cr), 2);

            // Check if they are not equal
            if ($total_dr_rounded != $total_cr_rounded) {
                // Output the values for debugging
                dd($total_dr_rounded, $total_cr_rounded);
                
                // Return response with an error message
                return response()->json(['message' => 'Error In Accounting Entries'], 422);
            }


                


                $t_data['total_dr_amount'] = $total_dr;
                $t_data['total_cr_amount'] = $total_cr; 

                

                $this->create_transaction_voucher( $t_data );
            }
        }
       

        

    }




    protected function get_max_number($input){
        // Extract the integer value using regular expression
        $pattern = '/[-\/](\d+)$/';
        preg_match($pattern, $input, $matches);
        
        if(empty($matches)){
            return intval($input);
        }

        $integerValue = intval(end($matches));
        return $integerValue;
    }


    protected function find_duplicate($item,$accid,$sourceid,$destinationid,$madetialid){
        
        $eff_from = DateTime::createFromFormat('d-m-Y', $item['eff_from'])->format('Y-m-d');
        $eff_till = DateTime::createFromFormat('d-m-Y', $item['eff_till'])->format('Y-m-d');
        
        $exists = PriceList::with(['source:id,station_name','destination:id,station_name'])->where('acc_party_id', $accid)
                        ->where('source_id', $sourceid)
                        ->where('destination_id', $destinationid) 
                        ->where('material_id', isset($item['material_desc']) ? $madetialid : null)
                        ->when(isset($item['wt']) && $item['wt'], function ($query) use ($item) {
                            $query->where('min_wt', $item['wt']);
                        })  
                        ->where('min_wt_unit', $item['p_w_unit'])
                        ->where('vehicle_type', $item['vehicle_type'])
                        
                        ->where(function ($query) use ($eff_from, $eff_till) {
                            $query->whereBetween('eff_from', [$eff_from, $eff_till])
                                  ->orWhereBetween('eff_till', [$eff_from, $eff_till])
                                  ->orWhere(function ($query) use ($eff_from, $eff_till) {
                                      $query->where('eff_from', '<=', $eff_from)
                                            ->where('eff_till', '>=', $eff_till);
                                  });
                        })
                  
                        
                        
                        ->when(isset($item['id']), function ($query) use ($item) {
                            $query->where('id', '!=', $item['id']);
                        })
                        ->first();


                        
        return $exists;
    }


}
