<?php

namespace App\Http\Traits;

use App\Models\Account\Account;
use App\Models\Account\Balances;
use App\Models\Account\Group;
use App\Models\Account\Voucher;
use App\Models\Account\VoucherAccDetails;
use App\Models\Account\VoucherDesc;
use App\Models\Bill\BillDesc;
use App\Models\Bill\CoverLetterDesc;
use App\Models\Bill\CreditNoteDesc;
use App\Models\FleetOwner\DieselReceipt;
use App\Models\FleetOwner\DriverSettlementDesc;
use App\Models\FleetOwner\LorryHireBillDesc;
use App\Models\FleetOwner\LorryHireDesc;
use App\Models\GoodsMove\Challan;
use App\Models\GoodsMove\ChallanDesc;
use App\Models\GoodsMove\ChallanReceiptDesc;
use App\Models\Inventory\Brand;
use App\Models\Inventory\InventoryBalances;
use App\Models\Inventory\Product;
use App\Models\Inventory\ProductCategory;
use App\Models\Inventory\PurchaseDesc;
use App\Models\Master\Branch;
use App\Models\Master\Material;
use App\Models\Master\Station;
use App\Models\Master\Vehicle;
use App\Models\Pod\PodDesc;
use Carbon\Carbon;

trait CommonTrait
{
    protected function get_min_max_date($financial_year)
    {
        // Split the financial year into two parts: start year and end year
        list($start_year, $end_year) = explode('-', $financial_year);

        // Calculate the start and end dates of the financial year
        $min_date = $start_year . '-04-01';
        $max_date = $end_year . '-03-31';

        // Create an object with the min and max dates
        $date_range = (object) [
            'min_date' => $min_date,
            'max_date' => $max_date,
        ];

        return $date_range;
    }

    protected function get_start_end_year($financial_year)
    {
        // Split the financial year into two parts: start year and end year
        list($start_year, $end_year) = explode('-', $financial_year);


        // Create an object with the min and max dates
        $date_range = (object) [
            'start_year' => $start_year,
            'end_year' => $end_year,
        ];

        return $date_range;
    }


    protected function get_short_name($financial_year)
    {
        // Split the financial year into two parts: start year and end year
        list($start_year, $end_year) = explode('-', $financial_year);

        // Create a short name for the financial year (e.g., '23-24')
        $short_name = substr($start_year, -2) . '-' . substr($end_year, -2);

        return $short_name;
    }

    
    protected function get_short_name2($financial_year)
    {
        // Split the financial year into two parts: start year and end year
        list($start_year, $end_year) = explode('-', $financial_year);

        // Create a short name for the financial year (e.g., '23-24')
        $short_name = substr($start_year, -2) . '/' . substr($end_year, -2);

        return $short_name;
    }

    protected function update_balance_for_account( $currentYear, $account_ids) {
       foreach($account_ids as $pk){

        $allowed_fin_years = explode(',', config('app.ALLOWED_FIN_YEAR'));

            
            $index = array_search($currentYear, $allowed_fin_years);
        
            if ($index !== false) {
                // Get an array from the index and so on
                $update_fin_years = array_slice($allowed_fin_years, $index);
        
                // Loop through the array and update the balances
                for ($i = 0; $i < count($update_fin_years); $i++) {
                    $fin_year = $update_fin_years[$i];
        
                    $min_max_date = $this->get_min_max_date($fin_year);
                    $min_date = $min_max_date->min_date;
                    $max_date = $min_max_date->max_date;
        
                    $from_date = $min_date;
            
                    //echo $pk.' '.$min_date.' '.$from_date.' '.$max_date.' '.$fin_year."<br>";
                    $balance = $this->calculate_balance_for_account($pk, $min_date, $from_date, $max_date, $fin_year, null);

                    

                    // If not in the last iteration
                    if ($i < count($update_fin_years) - 1) { 
                        $opening_for_next_year = Balances::where('account_id', $pk)
                            ->where('fin_year', $update_fin_years[$i + 1])
                            ->first();

                        $closing_for_current_year = Balances::where('account_id', $pk)
                            ->where('fin_year', $update_fin_years[$i])
                            ->first();
                        
                        if(!$opening_for_next_year){
                            $opening_for_next_year = new Balances();
                            $opening_for_next_year->fin_year = $update_fin_years[$i + 1];
                            $opening_for_next_year->account_id = $pk;
                        }
                        
                        if ($opening_for_next_year) {
                            if ($balance['nature_of_balance'] == 'Dr') {
                                $opening_for_next_year->opening_bal_dr = $balance['current_balance'];
                                $opening_for_next_year->opening_bal_cr = 0.00;

                                if($closing_for_current_year){
                                    $closing_for_current_year->closing_bal_dr = $balance['current_balance'];
                                    $closing_for_current_year->closing_bal_cr = 0.00;
                                    $closing_for_current_year->closing_bal_date = $max_date;
                                    $closing_for_current_year->save();
                                }
                            } else {
                                $opening_for_next_year->opening_bal_dr = 0.00;
                                $opening_for_next_year->opening_bal_cr = $balance['current_balance'];

                                if($closing_for_current_year){
                                    $closing_for_current_year->closing_bal_dr = 0.00;
                                    $closing_for_current_year->closing_bal_cr = $balance['current_balance'];
                                    $closing_for_current_year->closing_bal_date = $max_date;
                                    $closing_for_current_year->save();
                                }
                            }
        
                            $min_date_for_next = $this->get_min_max_date($update_fin_years[$i + 1])->min_date;
                            $opening_for_next_year->opening_bal_date = $min_date_for_next;
        
                            $opening_for_next_year->save();
                        }
                    }
                    else{
                        $closing_for_last_year = Balances::where('account_id', $pk)
                            ->where('fin_year', $update_fin_years[$i])
                            ->first();
                        
                        
                        if ($closing_for_last_year) {
                            if ($balance['nature_of_balance'] == 'Dr') {
                                $closing_for_last_year->closing_bal_dr = $balance['current_balance'];
                                $closing_for_last_year->closing_bal_cr = 0.00;
                            } else {
                                $closing_for_last_year->closing_bal_dr = 0.00;
                                $closing_for_last_year->closing_bal_cr = $balance['current_balance'];
                            }
        
                            $max_date_for_current = $this->get_min_max_date($update_fin_years[$i])->max_date;
                            $closing_for_last_year->closing_bal_date = $max_date_for_current;
        
                            $closing_for_last_year->save();
                        }
                    }
                }
            }

       }

        
    }
    

    protected function calculate_balance_for_account($pk,$min_date,$from_date,$to_date,$currentYear,$branch_id){
        
        // If we create a account in future years, its account balance is 0 for previous years
        $check_bal = Balances::where('account_id', $pk)
                            ->where('fin_year', $currentYear)
                            ->first();

        if(!$check_bal){
            $new = new Balances();
            $new->fin_year = $currentYear;
            $new->account_id = $pk;
            $new->opening_bal_dr = 0;
            $new->closing_bal_cr = 0;
            $new->opening_bal_date = $min_date;
            $new->save();
        }


        $account = Account::select('id','account_name','account_type','group_id')->with(['group:id,group_name,nature',
                            'balances'=> function ($query) use ($currentYear) {
                                $query->select('id','account_id','opening_bal_dr','opening_bal_cr','fin_year')->where('fin_year', $currentYear);
                            },
                            'voucher_acc_details'=> function ($qr) use ($currentYear,$branch_id) {
                                $qr->select('id','voucher_id','account_id','nature','dr_amount','cr_amount','active_status')->where('fin_year', $currentYear)->where('active_status',1);

                                if($branch_id){
                                    $qr->whereHas('voucher', function ($subQuery) use ($branch_id) {
                                        $subQuery->where('branch_id', $branch_id);
                                    });
                                }
                            }
                            ])
                            ->find($pk);

        $account_nature = $account->group->nature;

        $opening_balance = isset($account->balances[0]) ? $account->balances[0]->opening_bal_dr - $account->balances[0]->opening_bal_cr : 0;


        // Calculate opening balance
        if ($min_date !== $from_date) {
            $opening_bal = $account->voucher_acc_details
                                ->filter(function ($voucher) use ($min_date, $from_date) {
                                    return $voucher->voucher->voucher_date >= $min_date && $voucher->voucher->voucher_date < $from_date && $voucher->active_status;
                                })
                                ->sum(function ($voucher) {
                                    return $voucher->dr_amount - $voucher->cr_amount;
                                });
                           
                         
            $opening_balance = $opening_balance+$opening_bal;
        }
        
        if($opening_balance>=0){
            $nature_of_op_bal = 'Dr';
        }
        else{
            $nature_of_op_bal = 'Cr';
        }

        // If $branch_id exists than i want to calculate balance for that branch only

        $sums = $account->voucher_acc_details()
            ->join('vouchers', 'voucher_acc_details.voucher_id', '=', 'vouchers.id')
            ->where('voucher_acc_details.fin_year',$currentYear)
            ->where('voucher_acc_details.active_status',1)
            ->whereBetween('vouchers.voucher_date', [$from_date, $to_date])
            ->when($branch_id, function ($query, $branch_id) {
                return $query->where('vouchers.branch_id', $branch_id);
            })
            ->selectRaw('SUM(dr_amount) as total_dr_amount, SUM(cr_amount) as total_cr_amount')           
            ->first();

        $totalDrAmount = $sums->total_dr_amount;
        $totalCrAmount = $sums->total_cr_amount;


        $diff = $opening_balance + $totalDrAmount - $totalCrAmount;

        $current_balance = abs($diff);
        $nature_of_balance = 'Dr';

        if($diff>=0 ){
            $current_balance = $diff;
            $nature_of_balance = 'Dr';
        }
        else if($diff<0){
            $current_balance = abs($diff);
            $nature_of_balance = 'Cr';
        }
            

        return ['id'=>$pk, 'account_name'=>$account->account_name, 'current_balance' => $current_balance, 'nature_of_balance' => $nature_of_balance, 'opening_balance'=>abs($opening_balance), 'nature_of_op_bal'=>$nature_of_op_bal ,'account_nature' => $account_nature, 'totalDrAmount'=>$totalDrAmount, 'totalCrAmount'=>$totalCrAmount];
    }

    protected function update_balance_for_inventory( $currentYear, $product_ids) {
       
        foreach($product_ids as $pk){
 
            $allowed_fin_years = explode(',', config('app.ALLOWED_FIN_YEAR'));
 
             $index = array_search($currentYear, $allowed_fin_years);
             
             if ($index !== false) {
                 // Get an array from the index and so on
                 $update_fin_years = array_slice($allowed_fin_years, $index);

                
         
                 // Loop through the array and update the balances
                 for ($i = 0; $i < count($update_fin_years); $i++) {
                     $fin_year = $update_fin_years[$i];
         
                     $min_max_date = $this->get_min_max_date($fin_year);
                     $min_date = $min_max_date->min_date;
                     $max_date = $min_max_date->max_date;
         
                     $from_date = $min_date;
             
                     //echo $pk.' '.$min_date.' '.$from_date.' '.$max_date.' '.$fin_year."<br>";
                    //  $branch_ids = InventoryBalances::select('branch_id')
                    //                 ->where('product_id', $pk)
                    //                 ->where('fin_year', $currentYear)
                    //                 ->distinct()
                    //                 ->pluck('branch_id');

                     $branch_ids = Branch::select('id')->pluck('id');

                     foreach ($branch_ids as $branch_pk) {


                        $balance = $this->calculate_balance_for_inventory_stock($pk, $min_date, $from_date, $max_date, $fin_year, $branch_pk);
 
                     
 
                        // If not in the last iteration
                        if ($i < count($update_fin_years) - 1) { 
                            $opening_for_next_year = InventoryBalances::where('product_id', $pk)
                                ->where('branch_id', $branch_pk)
                                ->where('fin_year', $update_fin_years[$i + 1])
                                ->first();
    
                            $closing_for_current_year = InventoryBalances::where('product_id', $pk)
                                ->where('branch_id', $branch_pk)
                                ->where('fin_year', $update_fin_years[$i])
                                ->first();
                            
                            if(!$opening_for_next_year){
                                $opening_for_next_year = new InventoryBalances();
                                $opening_for_next_year->fin_year = $update_fin_years[$i + 1];
                                $opening_for_next_year->product_id = $pk;
                                $opening_for_next_year->branch_id = $branch_pk;
                            }
                            
                            if ($opening_for_next_year) {
                                    $opening_for_next_year->opening_bal = $balance['current_balance'];
    
                                    if($closing_for_current_year){
                                        $closing_for_current_year->closing_bal = $balance['current_balance'];
                                        $closing_for_current_year->closing_bal_date = $max_date;
                                        $closing_for_current_year->save();
                                    }

                                $min_date_for_next = $this->get_min_max_date($update_fin_years[$i + 1])->min_date;
                                $opening_for_next_year->opening_bal_date = $min_date_for_next;
            
                                $opening_for_next_year->save();
                            } 
            
                                
                        }
                        else{
                            $closing_for_last_year = InventoryBalances::where('product_id', $pk)
                                ->where('branch_id', $branch_pk)
                                ->where('fin_year', $update_fin_years[$i])
                                ->first();
                            
                            
                            if ($closing_for_last_year) {
                                $closing_for_last_year->closing_bal = $balance['current_balance'];
                                $max_date_for_current = $this->get_min_max_date($update_fin_years[$i])->max_date;
                                $closing_for_last_year->closing_bal_date = $max_date_for_current;
            
                                $closing_for_last_year->save();
                            }
                            
                        }
                    
                     }
                     
                 }
             }
 
        }
 
         
     }

     

    
     protected function calculate_balance_for_inventory_stock($pk,$min_date,$from_date,$to_date,$currentYear,$branch_id){
        $product = Product::with([
                        'balances' => function($qr) use ($currentYear,$branch_id) {
                            $qr->select('id', 'product_id', 'opening_bal')->where('branch_id',$branch_id)->where('fin_year', $currentYear);
                        }
                    ])->find($pk);



        $opening_balance =  isset($product->balances[0]) ? $product->balances[0]->opening_bal : 0;

        // Calculate opening balance

        if ($min_date !== $from_date) {
            $purchase_sum_for_period = PurchaseDesc::where('product_id', $pk)
                                ->where('vehicle_id', null)
                                ->whereHas('purchase', function($qr) use ($min_date, $from_date,$branch_id) {
                                    $qr->where('bill_date', '>=', $min_date)
                                    ->where('bill_date', '<', $from_date)
                                    ->where('branch_id',$branch_id)
                                    ->where('bill_name', '!=', 'Stock Issue')
                                    ->where('bill_name', '!=', 'Sale')
                                    ->where('bill_name', '!=', 'Stock Inwards');
                                })
                                ->where('fin_year', $currentYear)
                                ->sum('qty');
            
            $purchase_sum_for_STOCK_INWARDS = PurchaseDesc::where('product_id', $pk)
                                ->whereHas('purchase', function($qr) use ($min_date, $from_date,$branch_id) {
                                    $qr->where('bill_date', '>=', $min_date)
                                    ->where('bill_date', '<', $from_date)
                                    ->where('branch_id',$branch_id)
                                    ->where('bill_name', 'Stock Inwards');
                                })
                                ->where('fin_year', $currentYear)
                                ->sum('qty');

            $issue_sum_for_period = PurchaseDesc::where('product_id', $pk)
                                ->whereHas('purchase', function($qr) use ($min_date, $from_date, $branch_id) {
                                    $qr->where('bill_date', '>=', $min_date)->where('bill_date', '<', $from_date)->where('bill_name', 'Stock Issue')->where('branch_id',$branch_id);
                                })
                                ->where('fin_year', $currentYear)
                                ->sum('qty');

            $sale_sum_for_period = PurchaseDesc::where('product_id', $pk)
                                ->whereHas('purchase', function($qr) use ($min_date, $from_date, $branch_id) {
                                    $qr->where('bill_date', '>=', $min_date)->where('bill_date', '<', $from_date)->where('bill_name', 'Sale')->where('branch_id',$branch_id);
                                })
                                ->where('fin_year', $currentYear)
                                ->sum('qty');

            $opening_balance = $opening_balance+$purchase_sum_for_period+$purchase_sum_for_STOCK_INWARDS-$issue_sum_for_period-$sale_sum_for_period;
        }

        $sum_of_qty_in_purchase_for_period = PurchaseDesc::where('product_id', $pk)
                                ->where('vehicle_id', null)
                                ->whereHas('purchase', function($qr) use ($from_date, $to_date, $branch_id) {
                                    $qr->where('bill_date', '>=', $from_date)->where('bill_date', '<=', $to_date)->where('branch_id',$branch_id)
                                    ->where('bill_name', '!=', 'Stock Issue')
                                    ->where('bill_name', '!=', 'Sale')
                                    ->where('bill_name', '!=', 'Stock Inwards');
                                })
                                ->where('fin_year', $currentYear)
                                ->sum('qty');

        $sum_of_qty_in_purchase_for_STOCK_INWARDS = PurchaseDesc::where('product_id', $pk)
                                ->whereHas('purchase', function($qr) use ($from_date, $to_date, $branch_id) {
                                    $qr->where('bill_date', '>=', $from_date)->where('bill_date', '<=', $to_date)->where('branch_id',$branch_id)
                                    ->where('bill_name', 'Stock Inwards');
                                })
                                ->where('fin_year', $currentYear)
                                ->sum('qty');
        $issue_sum_for_period = PurchaseDesc::where('product_id', $pk)
                                ->whereHas('purchase', function($qr) use ($from_date, $to_date, $branch_id) {
                                    $qr->where('bill_date', '>=', $from_date)->where('bill_date', '<=', $to_date)->where('bill_name', 'Stock Issue')->where('branch_id',$branch_id);
                                })
                                ->where('fin_year', $currentYear)
                                ->sum('qty');
        
        $sale_sum_for_period = PurchaseDesc::where('product_id', $pk)
                                ->whereHas('purchase', function($qr) use ($from_date, $to_date, $branch_id) {
                                    $qr->where('bill_date', '>=', $from_date)->where('bill_date', '<=', $to_date)->where('bill_name', 'Sale')->where('branch_id',$branch_id);
                                })
                                ->where('fin_year', $currentYear)
                                ->sum('qty');
                            

        $diff = $opening_balance + $sum_of_qty_in_purchase_for_period + $sum_of_qty_in_purchase_for_STOCK_INWARDS - $issue_sum_for_period - $sale_sum_for_period;

        $current_balance = $diff;

        return ['id'=>$pk, 'product_name'=>$product->product_name, 'current_balance' => $current_balance, 'opening_balance'=>$opening_balance];
    }
   
   
    protected function create_transaction_voucher( $t_data ){
        // GET LAST VOUCHER NUMBER WITH VOUCHER NAME = 'TRANSACTION'
        $voucher = Voucher::select('voucher_number')->where('voucher_name', 'TRANSACTION')->orderBy('voucher_number', 'desc')->first();
        $voucher_no = $voucher ?  $voucher->voucher_number + 1 : 1;

        $t_voucher = new Voucher();
        $t_voucher->branch_id = $t_data['branch']['id'];
        $t_voucher->voucher_name = 'TRANSACTION';
        $t_voucher->voucher_number = $voucher_no;
        $t_voucher->voucher_date = $t_data['voucher_date'];
        $t_voucher->voucher_time = $t_data['voucher_time'];
        $t_voucher->total_dr_amount = $t_data['total_dr_amount'];
        $t_voucher->total_cr_amount = $t_data['total_cr_amount'];
        $t_voucher->t_against = $t_data['t_against'];
        $t_voucher->t_against_id = $t_data['t_against_id'];
        $t_voucher->t_against_no = $t_data['t_against_no'];
        $t_voucher->narration = $t_data['t_against_narration'];
        $t_voucher->fin_year = $t_data['fin_year'];
        $t_voucher->save();

        $voucher_accounting_details = $t_data['voucher_accounting_details'];

        foreach ($voucher_accounting_details as $index => $vad) {
            if (!$vad['account_id']) {
                continue;
            }
            $v_acc_detail = new VoucherAccDetails();
            $v_acc_detail->voucher_id = $t_voucher->id;
            $v_acc_detail->voucher_name = 'TRANSACTION';
            $v_acc_detail->voucher_number = $t_voucher->voucher_number;
            $v_acc_detail->account_id = $vad['account_id'];
            $v_acc_detail->nature = $vad['nature'];
            $v_acc_detail->dr_amount = $vad['dr_amount'];
            $v_acc_detail->cr_amount = $vad['cr_amount'];
            $v_acc_detail->cancellable = $t_data['cancellable'];
            $v_acc_detail->fin_year = $t_voucher->fin_year;
            $v_acc_detail->save();
        }
    }

    // protected function update_transaction_voucher($t_data){
    //     $t_voucher = Voucher::where('voucher_name', 'TRANSACTION')->where('t_against',$t_data['t_against'])->where('t_against_no',$t_data['t_against_no'])->first();
    //     $t_voucher->voucher_date = $t_data['voucher_date'];
    //     $t_voucher->voucher_time = $t_data['voucher_time'];
    //     $t_voucher->total_dr_amount = $t_data['total_dr_amount'];
    //     $t_voucher->total_cr_amount = $t_data['total_cr_amount'];
    //     $t_voucher->narration = $t_data['t_against_narration'];
    //     $t_voucher->save();

    //     $v_acc_detail_deleted = VoucherAccDetails::where('voucher_id',$t_voucher->id)->delete();

    //     $voucher_accounting_details = $t_data['voucher_accounting_details'];

    //     foreach ($voucher_accounting_details as $index => $vad) {
    //         if (!$vad['account_id']) {
    //             continue;
    //         }
    //         $v_acc_detail = new VoucherAccDetails();
    //         $v_acc_detail->voucher_id = $t_voucher->id;
    //         $v_acc_detail->voucher_name = 'TRANSACTION';
    //         $v_acc_detail->voucher_number = $t_voucher->voucher_number;
    //         $v_acc_detail->account_id = $vad['account_id'];
    //         $v_acc_detail->nature = $vad['nature'];
    //         $v_acc_detail->dr_amount = $vad['dr_amount'];
    //         $v_acc_detail->cr_amount = $vad['cr_amount'];
    //         $v_acc_detail->fin_year = $t_voucher->fin_year;
    //         $v_acc_detail->save();
    //     }

    // }

    protected function delete_transaction_voucher($t_data){
        // Correct the query to fetch the $t_voucher records using get() instead of pluck()
        $t_voucher = Voucher::select('id')
            ->where('voucher_name', 'TRANSACTION')
            ->where('t_against', $t_data['t_against'])
            ->where('t_against_no', $t_data['t_against_no'])
            ->where('fin_year', $t_data['fin_year'] )
            ->get();

        // Check if any $t_voucher records were found before proceeding
        if ($t_voucher->count() > 0) {
            // Use pluck to extract the 'id' values from the collection
            $voucher_ids = $t_voucher->pluck('id')->all();

            // Delete related records in VoucherAccDetails
            VoucherAccDetails::whereIn('voucher_id', $voucher_ids)->where('fin_year', $t_data['fin_year'] )->delete();

            // Now you can delete the $t_voucher records
            $t_voucher->each(function ($voucher) {
                $voucher->delete();
            });
        }
    }

    protected function set_transaction_status($t_data, $status)
    {
        $t_vouchers = Voucher::select('id')->where('voucher_name', 'TRANSACTION')
            ->where('t_against', $t_data['t_against'])
            ->where('t_against_no', $t_data['t_against_no'])
            ->get();

        if ($t_vouchers->isNotEmpty()) {
            $voucherIds = $t_vouchers->pluck('id');

            // Update the active_status of VoucherAccDetails records
            VoucherAccDetails::whereIn('voucher_id', $voucherIds)->where('cancellable', 1)->update(['active_status' => $status]);

            // Update the active_status of the Voucher records
            Voucher::whereIn('id', $voucherIds)->update(['active_status' => $status]);
        }
    }

    protected function get_acc_id($acc_name){
        $acc = Account::select('id')->where('account_name',$acc_name)->first();
        return $acc ? $acc->id : null;
    }

    protected function get_acc_name($acc_id){
        $acc = Account::select('account_name')->where('id',$acc_id)->first();
        return $acc ? $acc->account_name : null;
    }


    protected function check_in_challan_desc($select, $return, $pk_column , $pk){
        $challan_desc = ChallanDesc::select($select)->where($pk_column, $pk)->first();
        if($challan_desc){
            return ['status'=>true, 'message'=>'Stopped By Challan Number '.$challan_desc[$return], 'exists_in'=>'challan_desc', 'index'=>$challan_desc[$return] ];   
        }
        return ['status'=>false];
    }

    protected function check_in_challan_receipt_desc($column,$return, $pk_column , $pk){
        $challan_res_desc = ChallanReceiptDesc::select($column)->where($pk_column, $pk)->first();
        if($challan_res_desc){
            return ['status'=>true, 'message'=>'Stopped By Challan Rec. Num. '.$challan_res_desc[$return], 'exists_in'=>'challan_receipt_desc', 'index'=>$challan_res_desc[$return] ];   
        }
        return ['status'=>false];
    }

    protected function check_in_pod_desc($column,$return, $pk_column , $pk){
        $pod_desc = PodDesc::select($column)->with('pod:id,pod_number')->where($pk_column, $pk)->first();
        if($pod_desc){
            return ['status'=>true, 'message'=>'Stopped By POD Number '.$pod_desc->pod[$return], 'exists_in'=>'pod_desc', 'index'=>$pod_desc->pod[$return] ];   
        }
        return ['status'=>false];
    }

    protected function check_in_challan($column,$return, $pk_column , $pk){
        $challan = Challan::select($column)->where($pk_column, $pk)->first();
        if($challan){
            return ['status'=>true, 'message'=>'Stopped By Challan Number '.$challan[$return], 'exists_in'=>'challan', 'index'=>$challan[$return] ];   
        }
        return ['status'=>false];
    }

    protected function check_in_bill_desc($column,$return, $pk_column , $pk){
        $bill_desc = BillDesc::select($column)->with('bill:id,bill_number')->where($pk_column, $pk)->first();
        if($bill_desc){
            return ['status'=>true, 'message'=>'Stopped By Bill Number '.$bill_desc->bill[$return], 'exists_in'=>'bill_desc', 'index'=>$bill_desc->bill[$return] ];   
        }
        return ['status'=>false];
    }

    protected function check_in_voucher_desc($column,$return, $against , $against_number){
        $voucher_desc = VoucherDesc::select($column,'voucher_name')->where('against', $against)->where('against_number',$against_number)->first();
        if($voucher_desc){
            return ['status'=>true, 'message'=>'Stopped By Voucher '.$voucher_desc[$return], 'exists_in'=>'voucher_desc', 'index'=>$voucher_desc[$return] ];   
        }
        return ['status'=>false];
    }

    protected function check_in_lorry_hire_desc($column,$return, $against , $against_number){
        $lorry_hire_desc = LorryHireDesc::select($column)->where('against', $against)->where('against_number',$against_number)->first();
        if($lorry_hire_desc){
            return ['status'=>true, 'message'=>'Stopped By Lorry Hire '.$lorry_hire_desc[$return], 'exists_in'=>'lorry_hire_desc', 'index'=>$lorry_hire_desc[$return] ];   
        }
        return ['status'=>false];
    }

    protected function check_in_lorry_hire_bill_desc($column,$return, $against , $against_number){
        $lorry_hire_desc = LorryHireBillDesc::select($column)->where('challan_number',$against_number)->first();
        if($lorry_hire_desc){
            return ['status'=>true, 'message'=>'Stopped By Lorry Hire Bill '.$lorry_hire_desc[$return], 'exists_in'=>'lorry_hire_bill_desc', 'index'=>$lorry_hire_desc[$return] ];   
        }
        return ['status'=>false];
    }


    protected function check_in_diesel_receipt($column,$return, $against , $against_number){
        $diesel_receipt = DieselReceipt::select($column)->where('against', $against)->where('booking_number',$against_number)->first();
        if($diesel_receipt){
            return ['status'=>true, 'message'=>'Stopped By Diesel Receipt '.$diesel_receipt[$return], 'exists_in'=>'diesel_receipt', 'index'=>$diesel_receipt[$return] ];   
        }
        return ['status'=>false];
    }

    protected function check_in_driver_stl_desc($column,$return, $against , $against_number){
        $driver_stl_desc = DriverSettlementDesc::select($column)->where('against', $against)->where('against_number',$against_number)->first();
        if($driver_stl_desc){
            return ['status'=>true, 'message'=>'Stopped By Driver Settlement '.$driver_stl_desc[$return], 'exists_in'=>'driver_settlement_desc', 'index'=>$driver_stl_desc[$return] ];   
        }
        return ['status'=>false];
    }

    protected function check_in_credit_note_desc($column,$return,  $pk_column , $pk){
        $credit_note_desc = CreditNoteDesc::select($column)->where($pk_column,$pk)->first();
        if($credit_note_desc){
            return ['status'=>true, 'message'=>'Stopped By Credit Note '.$credit_note_desc[$return], 'exists_in'=>'credit_note_desc', 'index'=>$credit_note_desc[$return] ];   
        }
        return ['status'=>false];
    }

    protected function check_in_cover_letter_desc($column,$return,  $pk_column , $pk){
        $cover_letter_desc = CoverLetterDesc::select($column)->where($pk_column,$pk)->first();
        if($cover_letter_desc){
            return ['status'=>true, 'message'=>'Stopped By Cover Letter '.$cover_letter_desc[$return], 'exists_in'=>'cover_letter_desc', 'index'=>$cover_letter_desc[$return] ];   
        }
        return ['status'=>false];       
    }


    protected function add_if_not_in_master($model, $ref_object, $currentYear){
        $return_id = null;

        if(isset($ref_object['id']))
        if($ref_object['id']){
            return $ref_object['id'];
        }


        if($model == 'Material'){
            if($ref_object){
                $return_id = $ref_object['id'];

                $check_mat = Material::select('id')->where('material_name', $ref_object['material_name'])->first();
                if(!$check_mat){
                    $object = new Material();
                    $object->material_name = $ref_object['material_name'];
                    $object->save();

                    $return_id = $object->id;
                }else{
                    $return_id = $check_mat->id;
                }
                
            }
        }

        if($model == 'Station'){
            if($ref_object){
                $return_id = $ref_object['id'];
                
                $check_mat = Station::select('id')->where('station_name', $ref_object['station_name'])->first();
                if(!$check_mat){
                    $object = new Station();
                    $object->station_name = $ref_object['station_name'];
                    $object->save();

                    $return_id = $object->id;
                }else{
                    $return_id = $check_mat->id;
                }
                
            }
        }

        if($model == 'Brand'){
            if($ref_object){
                $return_id = $ref_object['id'];
                
                $check_mat = Brand::select('id')->where('brand_name', $ref_object['brand_name'])->first();
                if(!$check_mat){
                    $object = new Brand();
                    $object->brand_name = $ref_object['brand_name'];
                    $object->save();

                    $return_id = $object->id;
                }else{
                    $return_id = $check_mat->id;
                }
                
            }
        }

        if($model == 'ProductCategory'){
            if($ref_object){
                $return_id = $ref_object['id'];
                
                $check_mat = ProductCategory::select('id')->where('product_category', $ref_object['product_category'])->first();
                if(!$check_mat){
                    $object = new ProductCategory();
                    $object->product_category = $ref_object['product_category'];
                    $object->save();

                    $return_id = $object->id;
                }else{
                    $return_id = $check_mat->id;
                }
                
            }
        }

        if($model == 'Vehicle'){
            if($ref_object){
                $return_id = $ref_object['id'];
                
                $check_mat = Vehicle::select('id')->where('vehicle_number', $ref_object['vehicle_number'])->first();
                if(!$check_mat){
                    $object = new Vehicle();
                    $object->vehicle_number = $ref_object['vehicle_number'];

                    $group_name = 'Fixed Assets';

                    $object->account_head = $group_name ;
                    $object->fuel_type = 'Diesel';

                
                    $grp = Group::where('group_name', $group_name )->first();
            
                    $new_account = new Account();
                    $new_account->group_id = $grp->id ;
                    $new_account->account_name = $object->vehicle_number;
                    $new_account->account_type = 'Vehicle';
                    $new_account->save();

                    $bal = new Balances();
                    $bal->account_id = $new_account->id;
                    $bal->opening_bal_dr = 0;
                    $bal->opening_bal_cr = 0;
                    $bal->opening_bal_date = Carbon::now();
                    $bal->fin_year = $currentYear;
                    $bal->save();


                    $object->account_id = $new_account->id;

                    $object->save();


                    $return_id = $object->id;
                }else{
                    $return_id = $check_mat->id;
                }
                
            }
        }


        return $return_id;

    }


}
